@@ -15,18 +15,35 @@
  */
 package org.gradle.util;
 
+import com.google.common.base.Equivalence;
+import com.google.common.base.Function;
+import com.google.common.collect.ImmutableList;
+import com.google.common.collect.ImmutableListMultimap;
+import com.google.common.collect.ImmutableSet;
+import com.google.common.collect.Iterables;
 import org.gradle.api.Action;
+import org.gradle.api.Nullable;
 import org.gradle.api.Transformer;
-import org.gradle.api.internal.Transformers;
 import org.gradle.api.specs.Spec;
+import org.gradle.internal.Cast;
+import org.gradle.internal.Factory;
+import org.gradle.internal.Pair;
+import org.gradle.internal.Transformers;
 
 import java.lang.reflect.Array;
 import java.util.*;
 
-import static org.gradle.api.internal.Cast.cast;
+import static org.gradle.internal.Cast.cast;
 
 public abstract class CollectionUtils {
 
+    public static <T> Collection<? extends T> checkedCast(Class<T> type, Collection<?> input) {
+        for (Object o : input) {
+            cast(type, o);
+        }
+        return Cast.uncheckedCast(input);
+    }
+
     public static <T> T findFirst(Iterable<? extends T> source, Spec<? super T> filter) {
         for (T item : source) {
             if (filter.isSatisfiedBy(item)) {
@@ -37,6 +54,24 @@ public abstract class CollectionUtils {
         return null;
     }
 
+    public static <T> T findFirst(T[] source, Spec<? super T> filter) {
+        for (T thing : source) {
+            if (filter.isSatisfiedBy(thing)) {
+                return thing;
+            }
+        }
+
+        return null;
+    }
+
+    public static <T> boolean any(Iterable<? extends T> source, Spec<? super T> filter) {
+        return findFirst(source, filter) != null;
+    }
+
+    public static <T> boolean any(T[] source, Spec<? super T> filter) {
+        return findFirst(source, filter) != null;
+    }
+
     public static <T> Set<T> filter(Set<? extends T> set, Spec<? super T> filter) {
         return filter(set, new LinkedHashSet<T>(), filter);
     }
@@ -45,18 +80,29 @@ public abstract class CollectionUtils {
         return filter(list, new LinkedList<T>(), filter);
     }
 
+    public static <T> List<T> filter(T[] array, Spec<? super T> filter) {
+        return filter(Arrays.asList(array), new LinkedList<T>(), filter);
+    }
+
+
+    /**
+     * Returns a sorted copy of the provided collection of things. Uses the provided comparator to sort.
+     */
     public static <T> List<T> sort(Iterable<? extends T> things, Comparator<? super T> comparator) {
-        List<T> copy;
-        if (things instanceof Collection) {
-            //noinspection unchecked
-            copy = new ArrayList<T>((Collection<? extends T>) things);
-        } else {
-            copy = toList(things);
-        }
+        List<T> copy = toMutableList(things);
         Collections.sort(copy, comparator);
         return copy;
     }
 
+    /**
+     * Returns a sorted copy of the provided collection of things. Uses the natural ordering of the things.
+     */
+    public static <T extends Comparable> List<T> sort(Iterable<T> things) {
+        List<T> copy = toMutableList(things);
+        Collections.sort(copy);
+        return copy;
+    }
+
     public static <T, C extends Collection<T>> C filter(Iterable<? extends T> source, C destination, Spec<? super T> filter) {
         for (T item : source) {
             if (filter.isSatisfiedBy(item)) {
@@ -81,7 +127,8 @@ public abstract class CollectionUtils {
     }
 
     public static <R, I> R[] collectArray(I[] list, Class<R> newType, Transformer<? extends R, ? super I> transformer) {
-        return collectArray(list, (R[]) Array.newInstance(newType, list.length), transformer);
+        @SuppressWarnings("unchecked") R[] destination = (R[]) Array.newInstance(newType, list.length);
+        return collectArray(list, destination, transformer);
     }
 
     public static <R, I> R[] collectArray(I[] list, R[] destination, Transformer<? extends R, ? super I> transformer) {
@@ -104,6 +151,10 @@ public abstract class CollectionUtils {
         return collect(set, new HashSet<R>(), transformer);
     }
 
+    public static <R, I> List<R> collect(Iterable<? extends I> source, Transformer<? extends R, ? super I> transformer) {
+        return collect(source, new LinkedList<R>(), transformer);
+    }
+
     public static <R, I, C extends Collection<R>> C collect(Iterable<? extends I> source, C destination, Transformer<? extends R, ? super I> transformer) {
         for (I item : source) {
             destination.add(transformer.transform(item));
@@ -123,8 +174,8 @@ public abstract class CollectionUtils {
      * @param things The things to flatten
      * @return A flattened list of the given things
      */
-    public static List<?> flattenToList(Object... things) {
-        return flattenToList(Object.class, things);
+    public static List<?> flattenCollections(Object... things) {
+        return flattenCollections(Object.class, things);
     }
 
     /**
@@ -138,7 +189,7 @@ public abstract class CollectionUtils {
      * @param <T> The target type in the flattened list
      * @return A flattened list of the given things
      */
-    public static <T> List<T> flattenToList(Class<T> type, Object... things) {
+    public static <T> List<T> flattenCollections(Class<T> type, Object... things) {
         if (things == null) {
             return Collections.singletonList(null);
         } else if (things.length == 0) {
@@ -150,20 +201,23 @@ public abstract class CollectionUtils {
                 return Collections.singletonList(null);
             }
 
+            // Casts to Class below are to workaround Eclipse compiler bug
+            // See: https://github.com/gradle/gradle/pull/200
+
             if (thing.getClass().isArray()) {
                 Object[] thingArray = (Object[]) thing;
                 List<T> list = new ArrayList<T>(thingArray.length);
                 for (Object thingThing : thingArray) {
-                    list.addAll(flattenToList(type, thingThing));
+                    list.addAll(flattenCollections(type, thingThing));
                 }
                 return list;
             }
 
-            if (thing instanceof Iterable) {
-                Iterable<?> iterableThing = (Iterable<?>) thing;
+            if (thing instanceof Collection) {
+                Collection<?> collection = (Collection<?>) thing;
                 List<T> list = new ArrayList<T>();
-                for (Object thingThing : iterableThing) {
-                    list.addAll(flattenToList(type, thingThing));
+                for (Object element : collection) {
+                    list.addAll(flattenCollections(type, element));
                 }
                 return list;
             }
@@ -172,20 +226,32 @@ public abstract class CollectionUtils {
         } else {
             List<T> list = new ArrayList<T>();
             for (Object thing : things) {
-                list.addAll(flattenToList(type, thing));
+                list.addAll(flattenCollections(type, thing));
             }
             return list;
         }
     }
 
     public static <T> List<T> toList(Iterable<? extends T> things) {
+        if (things instanceof List) {
+            @SuppressWarnings("unchecked") List<T> castThings = (List<T>) things;
+            return castThings;
+        }
+        return toMutableList(things);
+    }
+
+    public static <T> List<T> toList(Enumeration<? extends T> things) {
+        AbstractList<T> list = new ArrayList<T>();
+        while (things.hasMoreElements()) {
+            list.add(things.nextElement());
+        }
+        return list;
+    }
+
+    private static <T> List<T> toMutableList(Iterable<? extends T> things) {
         if (things == null) {
             return new ArrayList<T>(0);
         }
-        if (things instanceof List) {
-            return (List<T>) things;
-        }
-
         List<T> list = new ArrayList<T>();
         for (T thing : things) {
             list.add(thing);
@@ -193,12 +259,39 @@ public abstract class CollectionUtils {
         return list;
     }
 
+
+    public static <T> List<T> intersection(Collection<? extends Collection<T>> availableValuesByDescriptor) {
+        List<T> result = new ArrayList<T>();
+        Iterator<? extends Collection<T>> iterator = availableValuesByDescriptor.iterator();
+        if (iterator.hasNext()) {
+            Collection<T> firstSet = iterator.next();
+            result.addAll(firstSet);
+            while (iterator.hasNext()) {
+                Collection<T> next = iterator.next();
+                result.retainAll(next);
+            }
+        }
+        return result;
+
+    }
+
+    public static <T> List<T> toList(T[] things) {
+        if (things == null || things.length == 0) {
+            return new ArrayList<T>(0);
+        }
+
+        List<T> list = new ArrayList<T>(things.length);
+        Collections.addAll(list, things);
+        return list;
+    }
+
     public static <T> Set<T> toSet(Iterable<? extends T> things) {
         if (things == null) {
             return new HashSet<T>(0);
         }
         if (things instanceof Set) {
-            return (Set<T>) things;
+            @SuppressWarnings("unchecked") Set<T> castThings = (Set<T>) things;
+            return castThings;
         }
 
         Set<T> set = new LinkedHashSet<T>();
@@ -235,7 +328,7 @@ public abstract class CollectionUtils {
         return collect(source, destination, Transformers.asString());
     }
 
-    public static List<String> stringize(List<?> source) {
+    public static List<String> stringize(Collection<?> source) {
         return stringize(source, new ArrayList<String>(source.size()));
     }
 
@@ -258,12 +351,30 @@ public abstract class CollectionUtils {
         }
     }
 
+    /**
+     * Given a set of values, derive a set of keys and return a map
+     */
     public static <K, V> Map<K, V> collectMap(Iterable<? extends V> items, Transformer<? extends K, ? super V> keyGenerator) {
         Map<K, V> map = new LinkedHashMap<K, V>();
         collectMap(map, items, keyGenerator);
         return map;
     }
 
+    public static <K, V> void collectMapValues(Map<K, V> destination, Iterable<? extends K> keys, Transformer<? extends V, ? super K> keyGenerator) {
+        for (K item : keys) {
+            destination.put(item, keyGenerator.transform(item));
+        }
+    }
+
+    /**
+     * Given a set of keys, derive a set of values and return a map
+     */
+    public static <K, V> Map<K, V> collectMapValues(Iterable<? extends K> keys, Transformer<? extends V, ? super K> keyGenerator) {
+        Map<K, V> map = new LinkedHashMap<K, V>();
+        collectMapValues(map, keys, keyGenerator);
+        return map;
+    }
+
     public static <T> boolean every(Iterable<? extends T> things, Spec<? super T> predicate) {
         for (T thing : things) {
             if (!predicate.isSatisfiedBy(thing)) {
@@ -289,6 +400,19 @@ public abstract class CollectionUtils {
         return t1;
     }
 
+    /**
+     * Utility for adding an array to a collection.
+     *
+     * @param t1 The collection to add to
+     * @param t2 The iterable to add each item of to the collection
+     * @param <T> The element type of t1
+     * @return t1
+     */
+    public static <T> Collection<T> addAll(Collection<T> t1, T... t2) {
+        Collections.addAll(t1, t2);
+        return t1;
+    }
+
     /**
      * The result of diffing two sets.
      *
@@ -296,13 +420,8 @@ public abstract class CollectionUtils {
      * @see CollectionUtils#diffSetsBy(java.util.Set, java.util.Set, org.gradle.api.Transformer)
      */
     public static class SetDiff<T> {
-        public static class Pair<T> {
-            public T left;
-            public T right;
-        }
-
         public Set<T> leftOnly = new HashSet<T>();
-        public Set<Pair<T>> common = new HashSet<Pair<T>>();
+        public Set<Pair<T, T>> common = new HashSet<Pair<T, T>>();
         public Set<T> rightOnly = new HashSet<T>();
     }
 
@@ -337,9 +456,7 @@ public abstract class CollectionUtils {
             if (rightValue == null) {
                 setDiff.leftOnly.add(leftEntry.getValue());
             } else {
-                SetDiff.Pair<T> pair = new SetDiff.Pair<T>();
-                pair.left = leftEntry.getValue();
-                pair.right = rightValue;
+                Pair<T, T> pair = Pair.of(leftEntry.getValue(), rightValue);
                 setDiff.common.add(pair);
             }
         }
@@ -395,14 +512,14 @@ public abstract class CollectionUtils {
             throw new NullPointerException("The 'objects' cannot be null");
         }
 
-        boolean first = true;
         StringBuilder string = new StringBuilder();
-        for (Object object : objects) {
-            if (!first) {
+        Iterator<?> iterator = objects.iterator();
+        if (iterator.hasNext()) {
+            string.append(iterator.next().toString());
+            while (iterator.hasNext()) {
                 string.append(separator);
+                string.append(iterator.next().toString());
             }
-            string.append(object.toString());
-            first = false;
         }
         return string.toString();
     }
@@ -442,4 +559,57 @@ public abstract class CollectionUtils {
         return target;
     }
 
-}
\ No newline at end of file
+    public static <K, V> ImmutableListMultimap<K, V> groupBy(Iterable<? extends V> iterable, Transformer<? extends K, V> grouper) {
+        ImmutableListMultimap.Builder<K, V> builder = ImmutableListMultimap.builder();
+
+        for (V element : iterable) {
+            K key = grouper.transform(element);
+            builder.put(key, element);
+        }
+
+        return builder.build();
+    }
+
+    public static <T> Iterable<? extends T> unpack(final Iterable<? extends Factory<? extends T>> factories) {
+        return new Iterable<T>() {
+            private final Iterator<? extends Factory<? extends T>> delegate = factories.iterator();
+
+            public Iterator<T> iterator() {
+                return new Iterator<T>() {
+                    public boolean hasNext() {
+                        return delegate.hasNext();
+                    }
+
+                    public T next() {
+                        return delegate.next().create();
+                    }
+
+                    public void remove() {
+                        throw new UnsupportedOperationException();
+                    }
+                };
+            }
+        };
+    }
+
+    @Nullable
+    public static <T> List<T> nonEmptyOrNull(Iterable<T> iterable) {
+        ImmutableList<T> list = ImmutableList.copyOf(iterable);
+        return list.isEmpty() ? null : list;
+    }
+
+    public static <T> List<T> dedup(Iterable<T> source, final Equivalence<T> equivalence) {
+        Iterable<Equivalence.Wrapper<T>> wrappers = Iterables.transform(source, new Function<T, Equivalence.Wrapper<T>>() {
+            public Equivalence.Wrapper<T> apply(@Nullable T input) {
+                return equivalence.wrap(input);
+            }
+        });
+        Set<Equivalence.Wrapper<T>> deduped = ImmutableSet.copyOf(wrappers);
+        return ImmutableList.copyOf(Iterables.transform(deduped, new Function<Equivalence.Wrapper<T>, T>() {
+            public T apply(Equivalence.Wrapper<T> input) {
+                return input.get();
+            }
+        }));
+    }
+
+}
