@@ -17,6 +17,7 @@
 package org.gradle.api.artifacts.component;
 
 import org.gradle.api.Incubating;
+import org.gradle.api.Nullable;
 
 /**
  * Criteria for selecting a library instance that is built as part of the current build.
@@ -24,6 +25,22 @@ import org.gradle.api.Incubating;
  */
 @Incubating
 public interface LibraryComponentSelector extends ComponentSelector {
+    /**
+     * Return the project path of the selected library.
+     *
+     * @return the project path of the library
+     */
     String getProjectPath();
+
+    /**
+     * Return the library name of the selected library.
+     * If the library name is null then it is expected to find a single library defined in same project
+     * as the requesting component or dependency resolution will fail.
+     * If not <code>null</code> then the name will never be empty.
+     *
+     * @return the library name
+     */
+    @Nullable
     String getLibraryName();
+
 }
