@@ -16,11 +16,11 @@
 
 package org.gradle.api.internal.project;
 
+import com.google.common.collect.Maps;
 import groovy.lang.Closure;
 import groovy.lang.MissingPropertyException;
-import groovy.lang.Script;
 import org.gradle.api.*;
-import org.gradle.api.artifacts.Module;
+import org.gradle.api.artifacts.ConfigurationContainer;
 import org.gradle.api.artifacts.dsl.ArtifactHandler;
 import org.gradle.api.artifacts.dsl.DependencyHandler;
 import org.gradle.api.artifacts.dsl.RepositoryHandler;
@@ -31,38 +31,54 @@ import org.gradle.api.file.CopySpec;
 import org.gradle.api.file.FileTree;
 import org.gradle.api.initialization.dsl.ScriptHandler;
 import org.gradle.api.internal.*;
-import org.gradle.api.internal.artifacts.configurations.ConfigurationContainerInternal;
+import org.gradle.api.internal.artifacts.ModuleInternal;
 import org.gradle.api.internal.artifacts.configurations.DependencyMetaDataProvider;
 import org.gradle.api.internal.file.FileOperations;
 import org.gradle.api.internal.file.FileResolver;
-import org.gradle.api.internal.initialization.ScriptClassLoaderProvider;
+import org.gradle.api.internal.initialization.ClassLoaderScope;
+import org.gradle.api.internal.initialization.ScriptHandlerFactory;
 import org.gradle.api.internal.plugins.DefaultObjectConfigurationAction;
+import org.gradle.api.internal.plugins.ExtensionContainerInternal;
+import org.gradle.api.internal.plugins.PluginManagerInternal;
+import org.gradle.api.internal.project.taskfactory.ITaskFactory;
 import org.gradle.api.internal.tasks.TaskContainerInternal;
 import org.gradle.api.logging.Logger;
 import org.gradle.api.logging.Logging;
-import org.gradle.api.logging.LoggingManager;
 import org.gradle.api.plugins.Convention;
 import org.gradle.api.plugins.ExtensionContainer;
-import org.gradle.api.plugins.PluginContainer;
 import org.gradle.api.resources.ResourceHandler;
-import org.gradle.api.tasks.Directory;
 import org.gradle.api.tasks.WorkResult;
-import org.gradle.configuration.ProjectEvaluator;
-import org.gradle.configuration.ScriptPlugin;
 import org.gradle.configuration.ScriptPluginFactory;
+import org.gradle.configuration.project.ProjectConfigurationActionContainer;
+import org.gradle.configuration.project.ProjectEvaluator;
 import org.gradle.groovy.scripts.ScriptSource;
+import org.gradle.internal.Actions;
+import org.gradle.internal.Factories;
 import org.gradle.internal.Factory;
+import org.gradle.internal.event.ListenerBroadcast;
 import org.gradle.internal.reflect.Instantiator;
+import org.gradle.internal.service.ServiceRegistry;
+import org.gradle.internal.service.scopes.ServiceRegistryFactory;
+import org.gradle.internal.typeconversion.TypeConverter;
 import org.gradle.listener.ClosureBackedMethodInvocationDispatch;
-import org.gradle.listener.ListenerBroadcast;
 import org.gradle.logging.LoggingManagerInternal;
 import org.gradle.logging.StandardOutputCapture;
+import org.gradle.model.RuleSource;
+import org.gradle.model.dsl.internal.NonTransformedModelDslBacking;
+import org.gradle.model.dsl.internal.TransformedModelDslBacking;
+import org.gradle.model.internal.core.*;
+import org.gradle.model.internal.manage.instance.ManagedProxyFactory;
+import org.gradle.model.internal.manage.schema.ModelSchemaStore;
+import org.gradle.model.internal.registry.ModelRegistry;
 import org.gradle.process.ExecResult;
+import org.gradle.process.ExecSpec;
+import org.gradle.process.JavaExecSpec;
 import org.gradle.util.Configurable;
 import org.gradle.util.ConfigureUtil;
 import org.gradle.util.DeprecationLogger;
 import org.gradle.util.Path;
 
+import javax.inject.Inject;
 import java.io.File;
 import java.net.URI;
 import java.util.*;
@@ -71,12 +87,12 @@ import static java.util.Collections.singletonMap;
 import static org.gradle.util.GUtil.addMaps;
 import static org.gradle.util.GUtil.isTrue;
 
-/**
- * @author Hans Dockter
- */
-public abstract class AbstractProject implements ProjectInternal, DynamicObjectAware {
+public abstract class AbstractProject extends AbstractPluginAware implements ProjectInternal, DynamicObjectAware {
+
     private static Logger buildLogger = Logging.getLogger(Project.class);
-    private ServiceRegistryFactory services;
+    private final ClassLoaderScope classLoaderScope;
+    private final ClassLoaderScope baseClassLoaderScope;
+    private ServiceRegistry services;
 
     private final ProjectInternal rootProject;
 
@@ -98,17 +114,13 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
 
     private Object status;
 
-    private final Map<String, Project> childProjects = new HashMap<String, Project>();
+    private final Map<String, Project> childProjects = Maps.newTreeMap();
 
     private List<String> defaultTasks = new ArrayList<String>();
 
-    private Set<Project> dependsOnProjects = new HashSet<Project>();
-
     private ProjectStateInternal state;
 
     private FileResolver fileResolver;
-    private FileOperations fileOperations;
-    private ProcessOperations processOperations;
 
     private Factory<AntBuilder> antBuilderFactory;
 
@@ -116,34 +128,18 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
 
     private Object buildDir = Project.DEFAULT_BUILD_DIR_NAME;
 
-    private PluginContainer pluginContainer;
-
     private final int depth;
 
     private TaskContainerInternal taskContainer;
 
-    private TaskContainerInternal implicitTasksContainer;
-
-    private IProjectRegistry<ProjectInternal> projectRegistry;
-
     private DependencyHandler dependencyHandler;
 
-    private ConfigurationContainerInternal configurationContainer;
+    private ConfigurationContainer configurationContainer;
 
     private ArtifactHandler artifactHandler;
 
-    private RepositoryHandler repositoryHandler;
-
-    private ScriptHandler scriptHandler;
-
-    private ScriptClassLoaderProvider scriptClassLoaderProvider;
-
     private ListenerBroadcast<ProjectEvaluationListener> evaluationListener = new ListenerBroadcast<ProjectEvaluationListener>(ProjectEvaluationListener.class);
 
-    private LoggingManagerInternal loggingManager;
-
-    private SoftwareComponentContainer softwareComponentContainer;
-
     private ExtensibleDynamicObject extensibleDynamicObject;
 
     private String description;
@@ -155,7 +151,11 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
                            File projectDir,
                            ScriptSource buildScriptSource,
                            GradleInternal gradle,
-                           ServiceRegistryFactory serviceRegistryFactory) {
+                           ServiceRegistryFactory serviceRegistryFactory,
+                           ClassLoaderScope selfClassLoaderScope,
+                           ClassLoaderScope baseClassLoaderScope) {
+        this.classLoaderScope = selfClassLoaderScope;
+        this.baseClassLoaderScope = baseClassLoaderScope;
         assert name != null;
         this.rootProject = parent != null ? parent.getRootProject() : this;
         this.projectDir = projectDir;
@@ -175,23 +175,7 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
         }
 
         services = serviceRegistryFactory.createFor(this);
-        fileResolver = services.get(FileResolver.class);
-        antBuilderFactory = services.getFactory(AntBuilder.class);
         taskContainer = services.newInstance(TaskContainerInternal.class);
-        implicitTasksContainer = services.newInstance(TaskContainerInternal.class);
-        fileOperations = services.get(FileOperations.class);
-        processOperations = services.get(ProcessOperations.class);
-        projectEvaluator = services.get(ProjectEvaluator.class);
-        repositoryHandler = services.get(RepositoryHandler.class);
-        configurationContainer = services.get(ConfigurationContainerInternal.class);
-        pluginContainer = services.get(PluginContainer.class);
-        artifactHandler = services.get(ArtifactHandler.class);
-        dependencyHandler = services.get(DependencyHandler.class);
-        scriptHandler = services.get(ScriptHandler.class);
-        scriptClassLoaderProvider = services.get(ScriptClassLoaderProvider.class);
-        projectRegistry = services.get(IProjectRegistry.class);
-        loggingManager = services.get(LoggingManagerInternal.class);
-        softwareComponentContainer = services.get(SoftwareComponentContainer.class);
 
         extensibleDynamicObject = new ExtensibleDynamicObject(this, services.get(Instantiator.class));
         if (parent != null) {
@@ -200,6 +184,73 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
         extensibleDynamicObject.addObject(taskContainer.getTasksAsDynamicObject(), ExtensibleDynamicObject.Location.AfterConvention);
 
         evaluationListener.add(gradle.getProjectEvaluationBroadcaster());
+
+        populateModelRegistry(services.get(ModelRegistry.class));
+    }
+
+    static class BasicServicesRules extends RuleSource {
+        @Service
+        ITaskFactory taskFactory(ServiceRegistry serviceRegistry) {
+            return serviceRegistry.get(ITaskFactory.class);
+        }
+
+        @Service
+        ModelSchemaStore schemaStore(ServiceRegistry serviceRegistry) {
+            return serviceRegistry.get(ModelSchemaStore.class);
+        }
+
+        @Service
+        ManagedProxyFactory proxyFactory(ServiceRegistry serviceRegistry) {
+            return serviceRegistry.get(ManagedProxyFactory.class);
+        }
+
+        @Service
+        NodeInitializerRegistry nodeInitializerRegistry(ModelSchemaStore schemaStore) {
+            return new DefaultNodeInitializerRegistry(schemaStore);
+        }
+
+        @Service
+        TypeConverter typeConverter(ServiceRegistry serviceRegistry) {
+            return serviceRegistry.get(TypeConverter.class);
+        }
+    }
+
+    private void populateModelRegistry(ModelRegistry modelRegistry) {
+        registerServiceOn(modelRegistry, "serviceRegistry", ServiceRegistry.class, services, instanceDescriptorFor("serviceRegistry"));
+        // TODO:LPTR This ignores changes to Project.buildDir after model node has been created
+        registerFactoryOn(modelRegistry, "buildDir", File.class, new Factory<File>() {
+            @Override
+            public File create() {
+                return getBuildDir();
+            }
+        });
+        registerInstanceOn(modelRegistry, "projectIdentifier", ProjectIdentifier.class, this);
+        registerInstanceOn(modelRegistry, "extensionContainer", ExtensionContainer.class, getExtensions());
+        modelRegistry.apply(BasicServicesRules.class);
+    }
+
+    private <T> void registerInstanceOn(ModelRegistry modelRegistry, String path, Class<T> type, T instance) {
+        registerFactoryOn(modelRegistry, path, type, Factories.constant(instance));
+    }
+
+    private <T> void registerFactoryOn(ModelRegistry modelRegistry, String path, Class<T> type, Factory<T> factory) {
+        modelRegistry.registerOrReplace(ModelRegistrations
+            .unmanagedInstance(ModelReference.of(path, type), factory)
+            .descriptor(instanceDescriptorFor(path))
+            .ephemeral(true)
+            .hidden(true)
+            .build());
+    }
+
+    private <T> void registerServiceOn(ModelRegistry modelRegistry, String path, Class<T> type, T instance, String descriptor) {
+        modelRegistry.registerOrReplace(ModelRegistrations.serviceInstance(ModelReference.of(path, type), instance)
+            .descriptor(descriptor)
+            .build()
+        );
+    }
+
+    private String instanceDescriptorFor(String path) {
+        return "Project.<init>." + path + "()";
     }
 
     public ProjectInternal getRootProject() {
@@ -210,11 +261,10 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
         return gradle;
     }
 
-    public PluginContainer getPlugins() {
-        return pluginContainer;
-    }
-
     public ProjectEvaluator getProjectEvaluator() {
+        if (projectEvaluator == null) {
+            projectEvaluator = services.get(ProjectEvaluator.class);
+        }
         return projectEvaluator;
     }
 
@@ -222,32 +272,19 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
         this.projectEvaluator = projectEvaluator;
     }
 
+    @Inject
     public ScriptHandler getBuildscript() {
-        return scriptHandler;
-    }
-
-    public void beforeCompile(ScriptPlugin configurer) {
-        if (configurer.getSource() != buildScriptSource) {
-            return;
-        }
-        configurer.setScriptBaseClass(ProjectScript.class);
-        configurer.setClassLoaderProvider(scriptClassLoaderProvider);
-    }
-
-    public void afterCompile(ScriptPlugin configurer, org.gradle.groovy.scripts.Script script) {
-        if (configurer.getSource() != buildScriptSource) {
-            return;
-        }
-        setScript(script);
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
     }
 
     public File getBuildFile() {
         return getBuildscript().getSourceFile();
     }
 
-    public void setScript(Script buildScript) {
+    public void setScript(groovy.lang.Script buildScript) {
         extensibleDynamicObject.addObject(new BeanDynamicObject(buildScript).withNoProperties().withNotImplementsMissing(),
-                ExtensibleDynamicObject.Location.BeforeConvention);
+            ExtensibleDynamicObject.Location.BeforeConvention);
     }
 
     public ScriptSource getBuildScriptSource() {
@@ -327,15 +364,14 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
         this.defaultTasks = defaultTasks;
     }
 
-    public Set<Project> getDependsOnProjects() {
-        return dependsOnProjects;
-    }
-
     public ProjectStateInternal getState() {
         return state;
     }
 
     public FileResolver getFileResolver() {
+        if (fileResolver == null) {
+            fileResolver = services.get(FileResolver.class);
+        }
         return fileResolver;
     }
 
@@ -348,6 +384,9 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
     }
 
     public ArtifactHandler getArtifacts() {
+        if (artifactHandler == null) {
+            artifactHandler = services.get(ArtifactHandler.class);
+        }
         return artifactHandler;
     }
 
@@ -355,15 +394,20 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
         this.artifactHandler = artifactHandler;
     }
 
+    @Inject
     public RepositoryHandler getRepositories() {
-        return repositoryHandler;
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
     }
 
-    public ConfigurationContainerInternal getConfigurations() {
+    public ConfigurationContainer getConfigurations() {
+        if (configurationContainer == null) {
+            configurationContainer = services.get(ConfigurationContainer.class);
+        }
         return configurationContainer;
     }
 
-    public void setConfigurationContainer(ConfigurationContainerInternal configurationContainer) {
+    public void setConfigurationContainer(ConfigurationContainer configurationContainer) {
         this.configurationContainer = configurationContainer;
     }
 
@@ -379,8 +423,10 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
         return depth;
     }
 
-    public IProjectRegistry<ProjectInternal> getProjectRegistry() {
-        return projectRegistry;
+    @Inject
+    public ProjectRegistry<ProjectInternal> getProjectRegistry() {
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
     }
 
     public int depthCompare(Project otherProject) {
@@ -416,15 +462,15 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
         if (!isTrue(path)) {
             throw new InvalidUserDataException("A path must be specified!");
         }
-        return projectRegistry.getProject(absoluteProjectPath(path));
+        return getProjectRegistry().getProject(absoluteProjectPath(path));
     }
 
     public Set<Project> getAllprojects() {
-        return new TreeSet<Project>(projectRegistry.getAllProjects(getPath()));
+        return new TreeSet<Project>(getProjectRegistry().getAllProjects(getPath()));
     }
 
     public Set<Project> getSubprojects() {
-        return new TreeSet<Project>(projectRegistry.getSubProjects(getPath()));
+        return new TreeSet<Project>(getProjectRegistry().getSubProjects(getPath()));
     }
 
     public void subprojects(Action<? super Project> action) {
@@ -450,7 +496,7 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
     }
 
     public AntBuilder createAntBuilder() {
-        return antBuilderFactory.create();
+        return getAntBuilderFactory().create();
     }
 
     /**
@@ -461,19 +507,25 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
     }
 
     public AbstractProject evaluate() {
-        projectEvaluator.evaluate(this, state);
+        getProjectEvaluator().evaluate(this, state);
         state.rethrowFailure();
         return this;
     }
 
+    @Override
+    public ProjectInternal bindAllModelRules() {
+        try {
+            getModelRegistry().bindAllReferences();
+        } catch (Exception e) {
+            throw new ProjectConfigurationException(String.format("A problem occurred configuring %s.", this), e);
+        }
+        return this;
+    }
+
     public TaskContainerInternal getTasks() {
         return taskContainer;
     }
 
-    public TaskContainerInternal getImplicitTasks() {
-        return implicitTasksContainer;
-    }
-
     public void defaultTasks(String... defaultTasks) {
         if (defaultTasks == null) {
             throw new InvalidUserDataException("Default tasks must not be null!");
@@ -487,28 +539,6 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
         }
     }
 
-    public Task createTask(Map args, String name, Closure action) {
-        warnCreateTaskDeprecated();
-        Map<String, Object> allArgs = new HashMap<String, Object>(args);
-        allArgs.put(Task.TASK_NAME, name);
-        allArgs.put(Task.TASK_ACTION, action);
-        return taskContainer.add(allArgs);
-    }
-
-    public Task createTask(Map<String, ?> args, String name, Action<? super Task> action) {
-        warnCreateTaskDeprecated();
-        Map<String, Object> allArgs = new HashMap<String, Object>(args);
-        allArgs.put(Task.TASK_NAME, name);
-        if (action != null) {
-            allArgs.put(Task.TASK_ACTION, action);
-        }
-        return taskContainer.add(allArgs);
-    }
-
-    private void warnCreateTaskDeprecated() {
-        DeprecationLogger.nagUserOfReplacedMethod("Project.createTask()", "task()");
-    }
-
     public void addChildProject(ProjectInternal childProject) {
         childProjects.put(childProject.getName(), childProject);
     }
@@ -525,26 +555,6 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
         buildDir = path;
     }
 
-    public void dependsOn(final String path) {
-        DeprecationLogger.nagUserOfDiscontinuedMethod("Project.dependsOn(String path)");
-        DeprecationLogger.whileDisabled(new Factory<Void>() {
-            public Void create() {
-                dependsOn(path, true);
-                return null;
-            }
-        });
-    }
-
-    public void dependsOn(String path, boolean evaluateDependsOnProject) {
-        if (!isTrue(path)) {
-            throw new InvalidUserDataException("You must specify a project!");
-        }
-        dependsOnProjects.add(project(path));
-        if (evaluateDependsOnProject) {
-            evaluationDependsOn(path);
-        }
-    }
-
     public void evaluationDependsOnChildren() {
         for (Project project : childProjects.values()) {
             DefaultProject defaultProjectToEvaluate = (DefaultProject) project;
@@ -563,53 +573,20 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
     private Project evaluationDependsOn(DefaultProject projectToEvaluate) {
         if (projectToEvaluate.getState().getExecuting()) {
             throw new CircularReferenceException(String.format("Circular referencing during evaluation for %s.",
-                    projectToEvaluate));
+                projectToEvaluate));
         }
         return projectToEvaluate.evaluate();
     }
 
-    public Project childrenDependOnMe() {
-        DeprecationLogger.nagUserOfDiscontinuedMethod("Project.childrenDependOnMe()");
-        DeprecationLogger.whileDisabled(new Factory<Void>() {
-            public Void create() {
-                for (Project project : childProjects.values()) {
-                    project.dependsOn(getPath(), false);
-                }
-                return null;
-            }
-        });
-
-        return this;
-    }
-
-    public Project dependsOnChildren() {
-        DeprecationLogger.nagUserOfDiscontinuedMethod("Project.dependsOnChildren()");
-        return DeprecationLogger.whileDisabled(new Factory<Project>() {
-            public Project create() {
-               return dependsOnChildren(false);
-            }
-        });
-    }
-
-    public Project dependsOnChildren(final boolean evaluateDependsOnProject) {
-        DeprecationLogger.nagUserOfDiscontinuedMethod("Project.dependsOnChildren(boolean)");
-        DeprecationLogger.whileDisabled(new Factory<Void>() {
-            public Void create() {
-                for (Project project : childProjects.values()) {
-                    dependsOn(project.getPath(), evaluateDependsOnProject);
-                }
-                return null;
-            }
-        });
-        return this;
-    }
-
     public String toString() {
-        if (parent != null) {
-            return String.format("project '%s'", path);
-        } else {
-            return String.format("root project '%s'", name);
+        StringBuilder builder = new StringBuilder();
+        if (parent == null) {
+            builder.append("root ");
         }
+        builder.append("project '");
+        builder.append(parent == null ? name : path);
+        builder.append("'");
+        return builder.toString();
     }
 
     public Map<Project, Set<Task>> getAllTasks(boolean recursive) {
@@ -634,6 +611,9 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
         final Set<Task> foundTasks = new HashSet<Task>();
         Action<Project> action = new Action<Project>() {
             public void execute(Project project) {
+                // Don't force evaluation of rules here, let the task container do what it needs to
+                ((ProjectInternal) project).evaluate();
+
                 Task task = project.getTasks().findByName(name);
                 if (task != null) {
                     foundTasks.add(task);
@@ -648,103 +628,79 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
         return foundTasks;
     }
 
+    @Inject
+    protected FileOperations getFileOperations() {
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
+    }
+
     public File file(Object path) {
-        return fileOperations.file(path);
+        return getFileOperations().file(path);
     }
 
     public File file(Object path, PathValidation validation) {
-        return fileOperations.file(path, validation);
+        return getFileOperations().file(path, validation);
     }
 
     public URI uri(Object path) {
-        return fileOperations.uri(path);
+        return getFileOperations().uri(path);
     }
 
     public ConfigurableFileCollection files(Object... paths) {
-        return fileOperations.files(paths);
+        return getFileOperations().files(paths);
     }
 
     public ConfigurableFileCollection files(Object paths, Closure closure) {
-        return fileOperations.files(paths, closure);
+        return ConfigureUtil.configure(closure, getFileOperations().files(paths));
     }
 
     public ConfigurableFileTree fileTree(Object baseDir) {
-        return fileOperations.fileTree(baseDir);
+        return getFileOperations().fileTree(baseDir);
     }
 
     public ConfigurableFileTree fileTree(Object baseDir, Closure closure) {
-        return fileOperations.fileTree(baseDir, closure);
+        return ConfigureUtil.configure(closure, getFileOperations().fileTree(baseDir));
     }
 
     public ConfigurableFileTree fileTree(Map<String, ?> args) {
-        return fileOperations.fileTree(args);
-    }
-
-    public ConfigurableFileTree fileTree(Closure closure) {
-        DeprecationLogger.nagUserOfDeprecated("fileTree(Closure)", "Use fileTree((Object){ baseDir }) to have the closure used as the file tree base directory");
-        return fileOperations.fileTree(closure);
+        return getFileOperations().fileTree(args);
     }
 
     public FileTree zipTree(Object zipPath) {
-        return fileOperations.zipTree(zipPath);
+        return getFileOperations().zipTree(zipPath);
     }
 
     public FileTree tarTree(Object tarPath) {
-        return fileOperations.tarTree(tarPath);
+        return getFileOperations().tarTree(tarPath);
     }
 
     public ResourceHandler getResources() {
-        return fileOperations.getResources();
+        return getFileOperations().getResources();
     }
 
     public String relativePath(Object path) {
-        return fileOperations.relativePath(path);
+        return getFileOperations().relativePath(path);
     }
 
     public File mkdir(Object path) {
-        return fileOperations.mkdir(path);
+        return getFileOperations().mkdir(path);
     }
 
     public boolean delete(Object... paths) {
-        return fileOperations.delete(paths);
-    }
-
-    /**
-     * @deprecated Use the {@link #mkdir(Object)} instead.
-     */
-    @Deprecated
-    public Directory dir(String path) {
-        DeprecationLogger.nagUserOfReplacedMethod("AbstractProject.dir()", "mkdir()");
-        String[] pathElements = path.split("/");
-        String name = "";
-        Directory dirTask = null;
-        for (String pathElement : pathElements) {
-            name += name.length() != 0 ? "/" + pathElement : pathElement;
-            Task task = taskContainer.findByName(name);
-            if (task instanceof Directory) {
-                dirTask = (Directory) task;
-            } else if (task != null) {
-                throw new InvalidUserDataException(String.format("Cannot add directory task '%s' as a non-directory task with this name already exists.", name));
-            } else {
-                dirTask = taskContainer.add(name, Directory.class);
-            }
-        }
-        return dirTask;
-    }
-
-    public void setTaskContainer(TaskContainerInternal taskContainer) {
-        this.taskContainer = taskContainer;
+        return getFileOperations().delete(paths);
     }
 
     public Factory<AntBuilder> getAntBuilderFactory() {
+        if (antBuilderFactory == null) {
+            antBuilderFactory = services.getFactory(AntBuilder.class);
+        }
         return antBuilderFactory;
     }
 
-    public void setAntBuilderFactory(Factory<AntBuilder> antBuilderFactory) {
-        this.antBuilderFactory = antBuilderFactory;
-    }
-
     public DependencyHandler getDependencies() {
+        if (dependencyHandler == null) {
+            dependencyHandler = services.get(DependencyHandler.class);
+        }
         return dependencyHandler;
     }
 
@@ -777,15 +733,19 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
     }
 
     public StandardOutputCapture getStandardOutputCapture() {
-        return loggingManager;
+        return getLogging();
     }
 
-    public LoggingManager getLogging() {
-        return loggingManager;
+    @Inject
+    public LoggingManagerInternal getLogging() {
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
     }
 
+    @Inject
     public SoftwareComponentContainer getComponents() {
-        return softwareComponentContainer;
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
     }
 
     public Object property(String propertyName) throws MissingPropertyException {
@@ -809,41 +769,61 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
     }
 
     public WorkResult copy(Closure closure) {
-        return fileOperations.copy(closure);
+        return copy(new ClosureBackedAction<CopySpec>(closure));
+    }
+
+    public WorkResult copy(Action<? super CopySpec> action) {
+        return getFileOperations().copy(action);
+    }
+
+    public WorkResult sync(Action<? super CopySpec> action) {
+        return getFileOperations().sync(action);
     }
 
     public CopySpec copySpec(Closure closure) {
-        return fileOperations.copySpec(closure);
+        return copySpec(new ClosureBackedAction<CopySpec>(closure));
+    }
+
+    public CopySpec copySpec(Action<? super CopySpec> action) {
+        return Actions.with(copySpec(), action);
+    }
+
+    public CopySpec copySpec() {
+        return getFileOperations().copySpec();
+    }
+
+    @Inject
+    protected ProcessOperations getProcessOperations() {
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
     }
 
     public ExecResult javaexec(Closure closure) {
-        return processOperations.javaexec(closure);
+        return javaexec(new ClosureBackedAction<JavaExecSpec>(closure));
+    }
+
+    public ExecResult javaexec(Action<? super JavaExecSpec> action) {
+        return getProcessOperations().javaexec(action);
     }
 
     public ExecResult exec(Closure closure) {
-        return processOperations.exec(closure);
+        return exec(new ClosureBackedAction<ExecSpec>(closure));
     }
 
-    public ServiceRegistryFactory getServices() {
+    public ExecResult exec(Action<? super ExecSpec> action) {
+        return getProcessOperations().exec(action);
+    }
+
+    public ServiceRegistry getServices() {
         return services;
     }
 
-    public Module getModule() {
-        return getServices().get(DependencyMetaDataProvider.class).getModule();
+    public ServiceRegistryFactory getServiceRegistryFactory() {
+        return services.get(ServiceRegistryFactory.class);
     }
 
-    public void apply(Closure closure) {
-        DefaultObjectConfigurationAction action = new DefaultObjectConfigurationAction(fileResolver, services.get(
-                ScriptPluginFactory.class), this);
-        configure(action, closure);
-        action.execute();
-    }
-
-    public void apply(Map<String, ?> options) {
-        DefaultObjectConfigurationAction action = new DefaultObjectConfigurationAction(fileResolver, services.get(
-                ScriptPluginFactory.class), this);
-        ConfigureUtil.configureByMap(options, action);
-        action.execute();
+    public ModuleInternal getModule() {
+        return services.get(DependencyMetaDataProvider.class).getModule();
     }
 
     public AntBuilder ant(Closure configureClosure) {
@@ -894,15 +874,15 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
     }
 
     public Task task(String task) {
-        return taskContainer.add(task);
+        return taskContainer.create(task);
     }
 
     public Task task(Object task) {
-        return taskContainer.add(task.toString());
+        return taskContainer.create(task.toString());
     }
 
     public Task task(String task, Closure configureClosure) {
-        return taskContainer.add(task).configure(configureClosure);
+        return taskContainer.create(task).configure(configureClosure);
     }
 
     public Task task(Object task, Closure configureClosure) {
@@ -910,7 +890,7 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
     }
 
     public Task task(Map options, String task) {
-        return taskContainer.add(addMaps(options, singletonMap(Task.TASK_NAME, task)));
+        return taskContainer.create(addMaps(options, singletonMap(Task.TASK_NAME, task)));
     }
 
     public Task task(Map options, Object task) {
@@ -918,13 +898,62 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
     }
 
     public Task task(Map options, String task, Closure configureClosure) {
-        return taskContainer.add(addMaps(options, singletonMap(Task.TASK_NAME, task))).configure(configureClosure);
+        return taskContainer.create(addMaps(options, singletonMap(Task.TASK_NAME, task))).configure(configureClosure);
     }
 
     public Task task(Map options, Object task, Closure configureClosure) {
         return task(options, task.toString(), configureClosure);
     }
 
+    @Inject
+    public ProjectConfigurationActionContainer getConfigurationActions() {
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
+    }
+
+    @Inject
+    public ModelRegistry getModelRegistry() {
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
+    }
+
+    @Inject
+    protected ModelSchemaStore getModelSchemaStore() {
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
+    protected DefaultObjectConfigurationAction createObjectConfigurationAction() {
+        return new DefaultObjectConfigurationAction(getFileResolver(), getScriptPluginFactory(), getScriptHandlerFactory(), getBaseClassLoaderScope(), this);
+    }
+
+    @Inject
+    public PluginManagerInternal getPluginManager() {
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
+    }
+
+    @Inject
+    protected ScriptPluginFactory getScriptPluginFactory() {
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
+    }
+
+    @Inject
+    protected ScriptHandlerFactory getScriptHandlerFactory() {
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
+    }
+
+    public ClassLoaderScope getClassLoaderScope() {
+        return classLoaderScope;
+    }
+
+    public ClassLoaderScope getBaseClassLoaderScope() {
+        return baseClassLoaderScope;
+    }
+
     /**
      * This is called by the task creation DSL. Need to find a cleaner way to do this...
      */
@@ -947,7 +976,32 @@ public abstract class AbstractProject implements ProjectInternal, DynamicObjectA
         return instantiator.newInstance(FactoryNamedDomainObjectContainer.class, type, instantiator, new DynamicPropertyNamer(), factoryClosure);
     }
 
-    public ExtensionContainer getExtensions() {
-        return getConvention();
+    public ExtensionContainerInternal getExtensions() {
+        return (ExtensionContainerInternal) getConvention();
+    }
+
+    // Not part of the public API
+    public void model(Closure<?> modelRules) {
+        ModelRegistry modelRegistry = getModelRegistry();
+        if (TransformedModelDslBacking.isTransformedBlock(modelRules)) {
+            ClosureBackedAction.execute(new TransformedModelDslBacking(modelRegistry, this.getRootProject().getFileResolver()), modelRules);
+        } else {
+            new NonTransformedModelDslBacking(modelRegistry).configure(modelRules);
+        }
+    }
+
+    @Inject
+    protected DeferredProjectConfiguration getDeferredProjectConfiguration() {
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
+    }
+
+    public void addDeferredConfiguration(Runnable configuration) {
+        getDeferredProjectConfiguration().add(configuration);
+    }
+
+    @Override
+    public void fireDeferredConfiguration() {
+        getDeferredProjectConfiguration().fire();
     }
 }
