@@ -23,17 +23,20 @@ import org.gradle.api.internal.ExceptionAnalyser;
 import org.gradle.api.internal.GradleInternal;
 import org.gradle.api.logging.Logging;
 import org.gradle.api.logging.StandardOutputListener;
-import org.gradle.cache.CacheRepository;
 import org.gradle.configuration.BuildConfigurer;
+import org.gradle.deployment.internal.DeploymentRegistry;
+import org.gradle.execution.BuildConfigurationActionExecuter;
 import org.gradle.execution.BuildExecuter;
-import org.gradle.initialization.buildsrc.BuildSourceBuilder;
-import org.gradle.initialization.layout.BuildLayoutFactory;
 import org.gradle.internal.event.ListenerManager;
 import org.gradle.internal.featurelifecycle.ScriptUsageLocationReporter;
-import org.gradle.internal.progress.*;
+import org.gradle.internal.progress.BuildOperationExecutor;
+import org.gradle.internal.progress.BuildProgressFilter;
+import org.gradle.internal.progress.BuildProgressLogger;
+import org.gradle.internal.progress.LoggerProvider;
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.internal.service.scopes.BuildScopeServices;
+import org.gradle.internal.service.scopes.BuildSessionScopeServices;
 import org.gradle.internal.service.scopes.ServiceRegistryFactory;
 import org.gradle.invocation.DefaultGradle;
 import org.gradle.logging.LoggingManagerInternal;
@@ -48,8 +51,8 @@ public class DefaultGradleLauncherFactory implements GradleLauncherFactory {
     private final NestedBuildTracker tracker;
     private final BuildProgressLogger buildProgressLogger;
 
-    public DefaultGradleLauncherFactory(ServiceRegistry globalServices) {
-        sharedServices = globalServices;
+    public DefaultGradleLauncherFactory(ServiceRegistry sharedServices) {
+        this.sharedServices = sharedServices;
         tracker = new NestedBuildTracker();
 
         // Register default loggers
@@ -67,7 +70,7 @@ public class DefaultGradleLauncherFactory implements GradleLauncherFactory {
         sharedServices.get(ListenerManager.class).removeListener(listener);
     }
 
-    public DefaultGradleLauncher newInstance(StartParameter startParameter) {
+    public GradleLauncher newInstance(StartParameter startParameter) {
         BuildRequestMetaData requestMetaData;
         BuildCancellationToken cancellationToken;
         BuildEventConsumer buildEventConsumer;
@@ -81,18 +84,28 @@ public class DefaultGradleLauncherFactory implements GradleLauncherFactory {
             cancellationToken = new DefaultBuildCancellationToken();
             buildEventConsumer = new NoOpBuildEventConsumer();
         }
-        return doNewInstance(startParameter, cancellationToken, requestMetaData, buildEventConsumer);
+
+        final BuildScopeServices buildScopeServices = BuildScopeServices.singleSession(sharedServices, startParameter);
+        return doNewInstance(startParameter, cancellationToken, requestMetaData, buildEventConsumer, buildScopeServices);
     }
 
     @Override
-    public GradleLauncher newInstance(StartParameter startParameter, BuildRequestContext requestContext) {
+    public GradleLauncher newInstance(StartParameter startParameter, BuildRequestContext requestContext, ServiceRegistry parentRegistry) {
         // This should only be used for top-level builds
         assert tracker.getCurrentBuild() == null;
-        return doNewInstance(startParameter, requestContext.getCancellationToken(), requestContext, requestContext.getEventConsumer());
+
+        if (!(parentRegistry instanceof BuildSessionScopeServices)) {
+            throw new IllegalArgumentException("Service registry must be of build session scope");
+        }
+
+        BuildScopeServices buildScopeServices = BuildScopeServices.forSession((BuildSessionScopeServices) parentRegistry);
+        DefaultGradleLauncher launcher = doNewInstance(startParameter, requestContext.getCancellationToken(), requestContext, requestContext.getEventConsumer(), buildScopeServices);
+        DeploymentRegistry deploymentRegistry = parentRegistry.get(DeploymentRegistry.class);
+        deploymentRegistry.onNewBuild(launcher.getGradle());
+        return launcher;
     }
 
-    private DefaultGradleLauncher doNewInstance(StartParameter startParameter, BuildCancellationToken cancellationToken, BuildRequestMetaData requestMetaData, BuildEventConsumer buildEventConsumer) {
-        final BuildScopeServices serviceRegistry = new BuildScopeServices(sharedServices, startParameter);
+    private DefaultGradleLauncher doNewInstance(StartParameter startParameter, BuildCancellationToken cancellationToken, BuildRequestMetaData requestMetaData, BuildEventConsumer buildEventConsumer, BuildScopeServices serviceRegistry) {
         serviceRegistry.add(BuildRequestMetaData.class, requestMetaData);
         serviceRegistry.add(BuildClientMetaData.class, requestMetaData.getClient());
         serviceRegistry.add(BuildEventConsumer.class, buildEventConsumer);
@@ -124,24 +137,15 @@ public class DefaultGradleLauncherFactory implements GradleLauncherFactory {
         return new DefaultGradleLauncher(
             gradle,
             serviceRegistry.get(InitScriptHandler.class),
-            new SettingsHandler(
-                new DefaultSettingsFinder(
-                    new BuildLayoutFactory()),
-                serviceRegistry.get(SettingsProcessor.class),
-                new BuildSourceBuilder(
-                    this,
-                    serviceRegistry.get(ClassLoaderScopeRegistry.class).getCoreAndPluginsScope(),
-                    serviceRegistry.get(CacheRepository.class))
-            ),
-            serviceRegistry.get(BuildLoader.class),
+            serviceRegistry.get(SettingsLoader.class),
             serviceRegistry.get(BuildConfigurer.class),
             serviceRegistry.get(ExceptionAnalyser.class),
             loggingManager,
             gradle.getBuildListenerBroadcaster(),
             listenerManager.getBroadcaster(ModelConfigurationListener.class),
-            listenerManager.getBroadcaster(TasksCompletionListener.class),
             listenerManager.getBroadcaster(BuildCompletionListener.class),
             serviceRegistry.get(BuildOperationExecutor.class),
+            gradle.getServices().get(BuildConfigurationActionExecuter.class),
             gradle.getServices().get(BuildExecuter.class),
             serviceRegistry
         );
