@@ -33,17 +33,6 @@ public final class OperationIdGenerator {
         return generateId(task.getProject().getGradle()) + "-" + task.getPath();
     }
 
-    /**
-     * Generates an operation id for the given build operation type.
-     *
-     * @param operationType the build operation type
-     * @param gradle te Gradle instance to which the build operation belongs
-     * @return the operation id
-     */
-    public static Object generateId(BuildOperationType operationType, Gradle gradle) {
-        return generateId(gradle) + "-" + operationType.getName();
-    }
-
     /**
      * Generates an operation id for the given Gradle instance.
      *
