@@ -20,10 +20,11 @@ import groovy.lang.MissingPropertyException;
 import org.gradle.api.*;
 import org.gradle.api.specs.Spec;
 import org.gradle.api.specs.Specs;
+import org.gradle.internal.reflect.DirectInstantiator;
 import org.gradle.util.ConfigureUtil;
 import org.gradle.util.GUtil;
-import org.gradle.util.HelperUtil;
 import org.gradle.util.TestClosure;
+import org.gradle.util.TestUtil;
 import org.jmock.Expectations;
 import org.jmock.integration.junit4.JMock;
 import org.jmock.integration.junit4.JUnit4Mockery;
@@ -32,8 +33,8 @@ import org.junit.runner.RunWith;
 
 import java.util.Iterator;
 
-import static org.gradle.util.HelperUtil.call;
-import static org.gradle.util.HelperUtil.toClosure;
+import static org.gradle.util.TestUtil.call;
+import static org.gradle.util.TestUtil.toClosure;
 import static org.gradle.util.WrapUtil.toList;
 import static org.hamcrest.Matchers.*;
 import static org.junit.Assert.*;
@@ -41,8 +42,12 @@ import static org.junit.Assert.*;
 
 @RunWith(JMock.class)
 public class DefaultNamedDomainObjectSetTest {
-    private final org.gradle.internal.reflect.Instantiator instantiator = new ClassGeneratorBackedInstantiator(new AsmBackedClassGenerator(), new org.gradle.internal.reflect.DirectInstantiator());
-    private final Namer<Bean> namer = new Namer<Bean>() { public String determineName(Bean bean) { return bean.name; } };
+    private final org.gradle.internal.reflect.Instantiator instantiator = new ClassGeneratorBackedInstantiator(new AsmBackedClassGenerator(), DirectInstantiator.INSTANCE);
+    private final Namer<Bean> namer = new Namer<Bean>() {
+        public String determineName(Bean bean) {
+            return bean.name;
+        }
+    };
     @SuppressWarnings("unchecked")
     private final DefaultNamedDomainObjectSet<Bean> container = instantiator.newInstance(DefaultNamedDomainObjectSet.class, Bean.class, instantiator, namer);
     private final JUnit4Mockery context = new JUnit4Mockery();
@@ -63,11 +68,11 @@ public class DefaultNamedDomainObjectSetTest {
         Bean bean1 = new Bean("a");
         Bean bean2 = new Bean("b");
         Bean bean3 = new Bean("c");
-    
+
         container.add(bean2);
         container.add(bean1);
         container.add(bean3);
-    
+
         assertThat(toList(container), equalTo(toList(bean1, bean2, bean3)));
     }
 
@@ -82,11 +87,11 @@ public class DefaultNamedDomainObjectSetTest {
         Bean bean1 = new Bean("a");
         Bean bean2 = new Bean("b");
         Bean bean3 = new Bean("c");
-    
+
         container.add(bean2);
         container.add(bean1);
         container.add(bean3);
-    
+
         Iterator<Bean> iterator = container.iterator();
         assertThat(iterator.next(), sameInstance(bean1));
         assertThat(iterator.next(), sameInstance(bean2));
@@ -167,7 +172,7 @@ public class DefaultNamedDomainObjectSetTest {
         container.add(bean3);
 
         assertThat(toList(container.matching(spec)), equalTo(toList(bean2, bean3)));
-        assertThat(toList(container.matching(HelperUtil.toClosure(testClosure))), equalTo(toList(bean2, bean3)));
+        assertThat(toList(container.matching(TestUtil.toClosure(testClosure))), equalTo(toList(bean2, bean3)));
         assertThat(container.matching(spec).findByName("a"), nullValue());
         assertThat(container.matching(spec).findByName("b"), sameInstance(bean2));
     }
@@ -199,7 +204,9 @@ public class DefaultNamedDomainObjectSetTest {
             public OtherBean(String name) {
                 super(name);
             }
-            public OtherBean() {}
+
+            public OtherBean() {
+            }
         }
         final Action<OtherBean> action = context.mock(Action.class);
         Bean bean1 = new Bean("b1");
@@ -208,7 +215,7 @@ public class DefaultNamedDomainObjectSetTest {
         container.add(bean1);
         container.add(bean2);
 
-        context.checking(new Expectations(){{
+        context.checking(new Expectations() {{
             one(action).execute(bean2);
         }});
 
@@ -221,7 +228,9 @@ public class DefaultNamedDomainObjectSetTest {
             public OtherBean(String name) {
                 super(name);
             }
-            public OtherBean() {}
+
+            public OtherBean() {
+            }
         }
         final TestClosure closure = context.mock(TestClosure.class);
         Bean bean1 = new Bean("b1");
@@ -230,11 +239,11 @@ public class DefaultNamedDomainObjectSetTest {
         container.add(bean1);
         container.add(bean2);
 
-        context.checking(new Expectations(){{
+        context.checking(new Expectations() {{
             one(closure).call(bean2);
         }});
 
-        container.withType(OtherBean.class, HelperUtil.toClosure(closure));
+        container.withType(OtherBean.class, TestUtil.toClosure(closure));
     }
 
     @Test
@@ -306,7 +315,7 @@ public class DefaultNamedDomainObjectSetTest {
             }
         };
 
-        container.matching(spec).whenObjectAdded(HelperUtil.toClosure(closure));
+        container.matching(spec).whenObjectAdded(TestUtil.toClosure(closure));
 
         container.add(bean);
         container.add(new Bean());
@@ -461,7 +470,7 @@ public class DefaultNamedDomainObjectSetTest {
             one(closure).call(bean);
         }});
 
-        container.whenObjectAdded(HelperUtil.toClosure(closure));
+        container.whenObjectAdded(TestUtil.toClosure(closure));
         container.add(bean);
     }
 
@@ -552,7 +561,7 @@ public class DefaultNamedDomainObjectSetTest {
         }});
 
         container.add(bean);
-        container.all(HelperUtil.toClosure(closure));
+        container.all(TestUtil.toClosure(closure));
     }
 
     @Test
@@ -577,7 +586,7 @@ public class DefaultNamedDomainObjectSetTest {
             one(closure).call(bean);
         }});
 
-        container.all(HelperUtil.toClosure(closure));
+        container.all(TestUtil.toClosure(closure));
         container.add(bean);
     }
 
@@ -662,7 +671,7 @@ public class DefaultNamedDomainObjectSetTest {
 
         Bean withType = new Bean("withType");
         container.add(withType);
-        
+
         // Try with an element with the same name as a method
         ConfigureUtil.configure(toClosure("{ withType.beanProperty = 'value 6' }"), container);
         assertThat(withType.getBeanProperty(), equalTo("value 6"));
@@ -704,7 +713,7 @@ public class DefaultNamedDomainObjectSetTest {
     public void addRuleByClosure() {
         String testPropertyKey = "org.gradle.test.addRuleByClosure";
         String expectedTaskName = "someTaskName";
-        Closure ruleClosure = HelperUtil.toClosure(String.format("{ taskName -> System.setProperty('%s', '%s') }",
+        Closure ruleClosure = TestUtil.toClosure(String.format("{ taskName -> System.setProperty('%s', '%s') }",
                 testPropertyKey, expectedTaskName));
         container.addRule("description", ruleClosure);
         container.getRules().get(0).apply(expectedTaskName);
