@@ -15,8 +15,8 @@
  */
 package org.gradle.api.internal.artifacts;
 
-import org.gradle.api.artifacts.DependencySet;
 import org.gradle.api.internal.artifacts.configurations.ResolutionStrategyInternal;
+import org.gradle.internal.component.model.ComponentResolveMetaData;
 
 /**
  * Represents something that can be resolved.
@@ -27,10 +27,8 @@ public interface ResolveContext {
 
     String getDisplayName();
 
-    DependencySet getDependencies();
-
-    DependencySet getAllDependencies();
-
     ResolutionStrategyInternal getResolutionStrategy();
 
+    ComponentResolveMetaData toRootComponentMetaData();
+
 }
