@@ -24,7 +24,7 @@ import org.junit.Test;
 
 import static org.junit.Assert.fail;
 
-@LeaksFileHandles
+@SuppressWarnings("ResultOfMethodCallIgnored")
 public class BuildScriptClasspathIntegrationTest extends AbstractIntegrationTest {
     @Test
     public void providesADefaultBuildForBuildSrcProject() {
@@ -34,6 +34,7 @@ public class BuildScriptClasspathIntegrationTest extends AbstractIntegrationTest
     }
 
     @Test
+    @LeaksFileHandles("second build opens repo/test-1.3.jar and doesn't release")
     public void canExtendTheDefaultBuildForBuildSrcProject() {
         ArtifactBuilder builder = artifactBuilder();
         builder.sourceFile("org/gradle/test/DepClass.java").writelns(
@@ -84,6 +85,7 @@ public class BuildScriptClasspathIntegrationTest extends AbstractIntegrationTest
     }
 
     @Test
+    @LeaksFileHandles("second build opens repo/test-1.3.jar and doesn't release")
     public void canDeclareClasspathInBuildScript() {
         ArtifactBuilder builder = artifactBuilder();
         builder.sourceFile("org/gradle/test/ImportedClass.java").writelns(
@@ -174,6 +176,7 @@ public class BuildScriptClasspathIntegrationTest extends AbstractIntegrationTest
     }
 
     @Test
+    @LeaksFileHandles("second build opens repo/test-1.3.jar and doesn't release")
     public void inheritsClassPathOfParentProject() {
         ArtifactBuilder builder = artifactBuilder();
         builder.sourceFile("org/gradle/test/BuildClass.java").writelns(
