@@ -19,27 +19,29 @@ import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Sets;
 import groovy.lang.Closure;
 import org.gradle.api.Action;
+import org.gradle.api.Transformer;
 import org.gradle.api.internal.ClosureBackedAction;
 import org.gradle.api.internal.initialization.DefaultClassLoaderScope;
 import org.gradle.api.logging.Logger;
 import org.gradle.api.logging.Logging;
+import org.gradle.internal.UncheckedException;
 import org.gradle.internal.jvm.Jvm;
 import org.gradle.launcher.daemon.configuration.DaemonParameters;
 import org.gradle.launcher.daemon.configuration.GradleProperties;
 import org.gradle.listener.ActionBroadcast;
-import org.gradle.process.internal.JvmOptions;
+import org.gradle.process.internal.streams.SafeStreams;
 import org.gradle.test.fixtures.file.TestDirectoryProvider;
 import org.gradle.test.fixtures.file.TestFile;
+import org.gradle.util.CollectionUtils;
 import org.gradle.util.DeprecationLogger;
-import org.gradle.util.TextUtil;
 
-import java.io.ByteArrayInputStream;
-import java.io.File;
-import java.io.InputStream;
+import java.io.*;
 import java.nio.charset.Charset;
 import java.util.*;
 
-import static java.util.Arrays.asList;
+import static org.gradle.launcher.daemon.client.DefaultDaemonConnector.DISABLE_STARTING_DAEMON_MESSAGE_PROPERTY;
+import static org.gradle.util.CollectionUtils.collect;
+import static org.gradle.util.CollectionUtils.join;
 import static org.gradle.util.Matchers.containsLine;
 import static org.gradle.util.Matchers.matchesRegexp;
 
@@ -51,6 +53,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     }
 
     private static final String DEBUG_SYSPROP = "org.gradle.integtest.debug";
+    private static final String PROFILE_SYSPROP = "org.gradle.integtest.profile";
 
     protected static final List<String> DEBUG_ARGS = ImmutableList.of(
         "-Xdebug",
@@ -78,14 +81,16 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     private File buildScript;
     private File projectDir;
     private File settingsFile;
-    private InputStream stdin;
+    private PipedOutputStream stdinPipe;
     private String defaultCharacterEncoding;
     private String tmpDir;
     private Locale defaultLocale;
     private int daemonIdleTimeoutSecs = 60;
+    private boolean requireDaemon;
     private File daemonBaseDir = buildContext.getDaemonBaseDir();
-    private final List<String> gradleOpts = new ArrayList<String>();
-    private boolean noDefaultJvmArgs;
+    private final List<String> buildJvmOpts = new ArrayList<String>();
+    private final List<String> commandLineJvmOpts = new ArrayList<String>();
+    private boolean useOnlyRequestedJvmOpts;
     private boolean requireGradleHome;
     private boolean daemonStartingMessageDisabled = true;
 
@@ -100,6 +105,8 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     private final GradleDistribution distribution;
 
     private boolean debug = Boolean.getBoolean(DEBUG_SYSPROP);
+    private String profiler = System.getProperty(PROFILE_SYSPROP, "");
+
     protected boolean interactive;
 
     protected AbstractGradleExecuter(GradleDistribution distribution, TestDirectoryProvider testDirectoryProvider) {
@@ -127,19 +134,21 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         executable = null;
         javaHome = null;
         environmentVars.clear();
-        stdin = null;
+        stdinPipe = null;
         defaultCharacterEncoding = null;
         tmpDir = null;
         defaultLocale = null;
-        noDefaultJvmArgs = false;
+        commandLineJvmOpts.clear();
+        buildJvmOpts.clear();
+        useOnlyRequestedJvmOpts = false;
         deprecationChecksOn = true;
         stackTraceChecksOn = true;
         debug = Boolean.getBoolean(DEBUG_SYSPROP);
+        profiler = System.getProperty(PROFILE_SYSPROP, "");
         interactive = false;
         return this;
     }
 
-
     public GradleDistribution getDistribution() {
         return distribution;
     }
@@ -198,7 +207,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         }
         executer.withTasks(tasks);
         executer.withArguments(args);
-        executer.withEnvironmentVars(getMergedEnvironmentVars());
+        executer.withEnvironmentVars(environmentVars);
         executer.usingExecutable(executable);
         if (quiet) {
             executer.withQuietLogging();
@@ -213,9 +222,11 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         if (userHomeDir != null) {
             executer.withUserHomeDir(userHomeDir);
         }
-        if (stdin != null) {
-            executer.withStdIn(stdin);
+
+        if (stdinPipe != null) {
+            executer.withStdinPipe(stdinPipe);
         }
+
         if (defaultCharacterEncoding != null) {
             executer.withDefaultCharacterEncoding(defaultCharacterEncoding);
         }
@@ -225,9 +236,10 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         if (defaultLocale != null) {
             executer.withDefaultLocale(defaultLocale);
         }
-        executer.withGradleOpts(gradleOpts.toArray(new String[gradleOpts.size()]));
-        if (noDefaultJvmArgs) {
-            executer.withNoDefaultJvmArgs();
+        executer.withCommandLineGradleOpts(commandLineJvmOpts);
+        executer.withBuildJvmOpts(buildJvmOpts);
+        if (useOnlyRequestedJvmOpts) {
+            executer.useDefaultBuildJvmArgs();
         }
         executer.noExtraLogging();
 
@@ -246,8 +258,12 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         if (!daemonStartingMessageDisabled) {
             executer.withDaemonStartingMessageEnabled();
         }
+        if (requireDaemon) {
+            executer.requireDaemon();
+        }
 
         executer.withDebug(debug);
+        executer.withProfiler(profiler);
         executer.withForceInteractive(interactive);
         return executer;
     }
@@ -289,23 +305,85 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return userHomeDir;
     }
 
+    protected GradleInvocation buildInvocation() {
+        validateDaemonVisibility();
+
+        GradleInvocation gradleInvocation = new GradleInvocation();
+        gradleInvocation.environmentVars.putAll(environmentVars);
+        gradleInvocation.buildJvmArgs.addAll(buildJvmOpts);
+        if (!useOnlyRequestedJvmOpts) {
+            gradleInvocation.buildJvmArgs.addAll(getImplicitBuildJvmArgs());
+        }
+        calculateLauncherJvmArgs(gradleInvocation);
+        gradleInvocation.args.addAll(getAllArgs());
+
+        transformInvocation(gradleInvocation);
+
+        if (!gradleInvocation.implicitLauncherJvmArgs.isEmpty()) {
+            throw new IllegalStateException("Implicit JVM args have not been handled.");
+        }
+
+        return gradleInvocation;
+    }
+
+    protected void validateDaemonVisibility() {
+        if (isUseDaemon() && isSharedDaemons()) {
+            throw new IllegalStateException("Daemon that will be visible to other tests has been requested.");
+        }
+    }
+
     /**
-     * Returns the gradle opts set with withGradleOpts() (does not consider any set via withEnvironmentVars())
+     * Adjusts the calculated invocation prior to execution. This method is responsible for handling the implicit launcher JVM args in some way, by mutating the invocation appropriately.
      */
-    protected List<String> getGradleOpts() {
-        List<String> gradleOpts = new ArrayList<String>(this.gradleOpts);
+    protected void transformInvocation(GradleInvocation gradleInvocation) {
+        gradleInvocation.launcherJvmArgs.addAll(gradleInvocation.implicitLauncherJvmArgs);
+        gradleInvocation.implicitLauncherJvmArgs.clear();
+    }
+
+    /**
+     * Returns the JVM opts that should be used to start a forked JVM.
+     */
+    private void calculateLauncherJvmArgs(GradleInvocation gradleInvocation) {
+        // Add JVM args that were explicitly requested
+        gradleInvocation.launcherJvmArgs.addAll(commandLineJvmOpts);
+
+        if (isUseDaemon() && !gradleInvocation.buildJvmArgs.isEmpty()) {
+            // Pass build JVM args through to daemon via system property on the launcher JVM
+            String quotedArgs = join(" ", collect(gradleInvocation.buildJvmArgs, new Transformer<String, String>() {
+                public String transform(String input) {
+                    return String.format("'%s'", input);
+                }
+            }));
+            gradleInvocation.implicitLauncherJvmArgs.add("-Dorg.gradle.jvmargs=" + quotedArgs);
+        } else {
+            // Have to pass build JVM args directly to launcher JVM
+            gradleInvocation.launcherJvmArgs.addAll(gradleInvocation.buildJvmArgs);
+        }
+
+        // Set the implicit system properties regardless of whether default JVM args are required or not, this should not interfere with tests' intentions
+        // These will also be copied across to any daemon used
         for (Map.Entry<String, String> entry : getImplicitJvmSystemProperties().entrySet()) {
             String key = entry.getKey();
             String value = entry.getValue();
-            gradleOpts.add(String.format("-D%s=%s", key, value));
+            gradleInvocation.implicitLauncherJvmArgs.add(String.format("-D%s=%s", key, value));
         }
-        gradleOpts.add("-ea");
 
+        gradleInvocation.implicitLauncherJvmArgs.add("-ea");
+    }
+
+    /**
+     * Returns additional JVM args that should be used to start the build JVM.
+     */
+    protected List<String> getImplicitBuildJvmArgs() {
+        List<String> buildJvmOpts = new ArrayList<String>();
+        buildJvmOpts.add("-ea");
         if (isDebug()) {
-            gradleOpts.addAll(DEBUG_ARGS);
+            buildJvmOpts.addAll(DEBUG_ARGS);
         }
-
-        return gradleOpts;
+        if (isProfile()) {
+            buildJvmOpts.add(profiler);
+        }
+        return buildJvmOpts;
     }
 
     public GradleExecuter withUserHomeDir(File userHomeDir) {
@@ -331,18 +409,27 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return executable;
     }
 
-    public GradleExecuter withStdIn(String text) {
-        this.stdin = new ByteArrayInputStream(TextUtil.toPlatformLineSeparators(text).getBytes());
+    @Override
+    public GradleExecuter withStdinPipe() {
+        return withStdinPipe(new PipedOutputStream());
+    }
+
+    @Override
+    public GradleExecuter withStdinPipe(PipedOutputStream stdInPipe) {
+        this.stdinPipe = stdInPipe;
         return this;
     }
 
-    public GradleExecuter withStdIn(InputStream stdin) {
-        this.stdin = stdin;
-        return this;
+    public InputStream connectStdIn() {
+        try {
+            return stdinPipe == null ? SafeStreams.emptyInput() : new PipedInputStream(stdinPipe);
+        } catch (IOException e) {
+            throw UncheckedException.throwAsUncheckedException(e);
+        }
     }
 
-    public InputStream getStdin() {
-        return stdin == null ? new ByteArrayInputStream(new byte[0]) : stdin;
+    public PipedOutputStream getStdinPipe() {
+        return stdinPipe;
     }
 
     public GradleExecuter withDefaultCharacterEncoding(String defaultCharacterEncoding) {
@@ -415,20 +502,6 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return this;
     }
 
-    /**
-     * Returns the effective env vars, having merged in specific settings.
-     *
-     * For example, GRADLE_OPTS will be anything that was specified via withEnvironmentVars() and withGradleOpts(). JAVA_HOME will also be set according to getJavaHome().
-     */
-    protected Map<String, String> getMergedEnvironmentVars() {
-        Map<String, String> environmentVars = new HashMap<String, String>(getEnvironmentVars());
-        environmentVars.put("GRADLE_OPTS", toJvmArgsString(getMergedGradleOpts()));
-        if (!environmentVars.containsKey("JAVA_HOME")) {
-            environmentVars.put("JAVA_HOME", getJavaHome().getAbsolutePath());
-        }
-        return environmentVars;
-    }
-
     protected String toJvmArgsString(Iterable<String> jvmArgs) {
         StringBuilder result = new StringBuilder();
         for (String jvmArg : jvmArgs) {
@@ -448,20 +521,6 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return result.toString();
     }
 
-    private List<String> getMergedGradleOpts() {
-        List<String> gradleOpts = new ArrayList<String>(getGradleOpts());
-        String gradleOptsEnv = getEnvironmentVars().get("GRADLE_OPTS");
-        if (gradleOptsEnv != null) {
-            gradleOpts.addAll(JvmOptions.fromString(gradleOptsEnv));
-        }
-
-        return gradleOpts;
-    }
-
-    protected Map<String, String> getEnvironmentVars() {
-        return environmentVars;
-    }
-
     public GradleExecuter withTasks(String... names) {
         return withTasks(Arrays.asList(names));
     }
@@ -477,8 +536,8 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return this;
     }
 
-    public GradleExecuter withNoDefaultJvmArgs() {
-        noDefaultJvmArgs = true;
+    public GradleExecuter useDefaultBuildJvmArgs() {
+        useOnlyRequestedJvmOpts = true;
         return this;
     }
 
@@ -491,12 +550,30 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return withDaemonBaseDir(testDirectoryProvider.getTestDirectory().file("daemon"));
     }
 
-    protected File getDaemonBaseDir() {
+    public File getDaemonBaseDir() {
         return daemonBaseDir;
     }
 
-    protected boolean isNoDefaultJvmArgs() {
-        return noDefaultJvmArgs;
+    @Override
+    public GradleExecuter requireDaemon() {
+        requireDaemon = true;
+        return this;
+    }
+
+    protected boolean isSharedDaemons() {
+        return daemonBaseDir.equals(buildContext.getDaemonBaseDir());
+    }
+
+    protected boolean isUseDaemon() {
+        for (int i = args.size() - 1; i >= 0; i--) {
+            if (args.get(i).equals("--daemon")) {
+                return true;
+            }
+            if (args.get(i).equals("--no-daemon")) {
+                return false;
+            }
+        }
+        return requireDaemon;
     }
 
     protected List<String> getAllArgs() {
@@ -520,6 +597,10 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         if (quiet) {
             allArgs.add("--quiet");
         }
+        if (isUseDaemon()) {
+            allArgs.add("--daemon");
+        }
+        allArgs.add("--stacktrace");
         if (taskList) {
             allArgs.add("tasks");
         }
@@ -558,6 +639,9 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return allArgs;
     }
 
+    /**
+     * Returns the set of system properties that should be set on every JVM used by this executer.
+     */
     protected Map<String, String> getImplicitJvmSystemProperties() {
         Map<String, String> properties = new LinkedHashMap<String, String>();
 
@@ -593,6 +677,10 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
             properties.put(DaemonParameters.INTERACTIVE_TOGGLE, "true");
         }
 
+        if (daemonStartingMessageDisabled) {
+            properties.put(DISABLE_STARTING_DAEMON_MESSAGE_PROPERTY, "true");
+        }
+
         return properties;
     }
 
@@ -647,11 +735,27 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
 
     protected abstract ExecutionFailure doRunWithFailure();
 
-    /**
-     * {@inheritDoc}
-     */
-    public AbstractGradleExecuter withGradleOpts(String... gradleOpts) {
-        this.gradleOpts.addAll(asList(gradleOpts));
+    @Override
+    public GradleExecuter withCommandLineGradleOpts(Iterable<String> jvmOpts) {
+        CollectionUtils.addAll(commandLineJvmOpts, jvmOpts);
+        return this;
+    }
+
+    @Override
+    public GradleExecuter withCommandLineGradleOpts(String... jvmOpts) {
+        CollectionUtils.addAll(commandLineJvmOpts, jvmOpts);
+        return this;
+    }
+
+    @Override
+    public AbstractGradleExecuter withBuildJvmOpts(String... jvmOpts) {
+        CollectionUtils.addAll(buildJvmOpts, jvmOpts);
+        return this;
+    }
+
+    @Override
+    public GradleExecuter withBuildJvmOpts(Iterable<String> jvmOpts) {
+        CollectionUtils.addAll(buildJvmOpts, jvmOpts);
         return this;
     }
 
@@ -666,7 +770,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
                     String error = executionResult.getError();
                     if (executionResult instanceof ExecutionFailure) {
                         // Axe everything after the expected exception
-                        int pos = error.indexOf("* Exception is:" + TextUtil.getPlatformLineSeparator());
+                        int pos = error.indexOf("* Exception is:\n");
                         if (pos >= 0) {
                             error = error.substring(0, pos);
                         }
@@ -746,16 +850,17 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return this;
     }
 
-    public boolean isDaemonStartingMessageDisabled() {
-        return daemonStartingMessageDisabled;
-    }
-
     @Override
     public GradleExecuter withDebug(boolean flag) {
         debug = flag;
         return this;
     }
 
+    public GradleExecuter withProfiler(String args) {
+        profiler = args;
+        return this;
+    }
+
     @Override
     public GradleExecuter withForceInteractive(boolean flag) {
         interactive = flag;
@@ -766,4 +871,20 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     public boolean isDebug() {
         return debug;
     }
+
+    public boolean isProfile() {
+        return !profiler.isEmpty();
+    }
+
+    protected static class GradleInvocation {
+        final Map<String, String> environmentVars = new HashMap<String, String>();
+        final List<String> args = new ArrayList<String>();
+        // JVM args that must be used for the build JVM
+        final List<String> buildJvmArgs = new ArrayList<String>();
+        // JVM args that must be used to fork a JVM
+        final List<String> launcherJvmArgs = new ArrayList<String>();
+        // Implicit JVM args that should be used to fork a JVM
+        final List<String> implicitLauncherJvmArgs = new ArrayList<String>();
+    }
+
 }
