@@ -33,7 +33,7 @@ import java.util.Map;
 
 import static org.junit.Assert.fail;
 
-class ForkingGradleExecuter extends AbstractGradleExecuter {
+public class ForkingGradleExecuter extends AbstractGradleExecuter {
 
     public ForkingGradleExecuter(GradleDistribution distribution, TestDirectoryProvider testDirectoryProvider) {
         super(distribution, testDirectoryProvider);
@@ -41,21 +41,62 @@ class ForkingGradleExecuter extends AbstractGradleExecuter {
 
     public void assertCanExecute() throws AssertionError {
         if (!getDistribution().isSupportsSpacesInGradleAndJavaOpts()) {
-            Map<String, String> mergedEnvironmentVars = getMergedEnvironmentVars();
+            Map<String, String> environmentVars = buildInvocation().environmentVars;
             for (String envVarName : Arrays.asList("JAVA_OPTS", "GRADLE_OPTS")) {
-                String envVarValue = mergedEnvironmentVars.get(envVarName);
+                String envVarValue = environmentVars.get(envVarName);
                 if (envVarValue == null) {
                     continue;
                 }
                 for (String arg : JvmOptions.fromString(envVarValue)) {
                     if (arg.contains(" ")) {
-                        throw new AssertionError(String.format("Env var %s contains arg with space (%s) which is not supported", envVarName, arg));
+                        throw new AssertionError(String.format("Env var %s contains arg with space (%s) which is not supported by Gradle %s", envVarName, arg, getDistribution().getVersion().getVersion()));
                     }
                 }
             }
         }
     }
 
+    @Override
+    protected void transformInvocation(GradleInvocation invocation) {
+        if (getDistribution().isSupportsSpacesInGradleAndJavaOpts()) {
+            // Mix the implicit launcher JVM args in with the requested JVM args
+            invocation.launcherJvmArgs.addAll(invocation.implicitLauncherJvmArgs);
+        } else {
+            // Need to move those implicit JVM args that contain a space to the Gradle command-line (if possible)
+            // Note that this isn't strictly correct as some system properties can only be set on JVM start up.
+            // Should change the implementation to deal with these properly
+            for (String jvmArg : invocation.implicitLauncherJvmArgs) {
+                if (!jvmArg.contains(" ")) {
+                    invocation.launcherJvmArgs.add(jvmArg);
+                } else if (jvmArg.startsWith("-D")) {
+                    invocation.args.add(jvmArg);
+                } else {
+                    throw new UnsupportedOperationException(String.format("Cannot handle launcher JVM arg '%s' as it contains whitespace. This is not supported by Gradle %s.",
+                            jvmArg, getDistribution().getVersion().getVersion()));
+                }
+            }
+        }
+        invocation.implicitLauncherJvmArgs.clear();
+
+        // Inject the launcher JVM args via one of the environment variables
+        Map<String, String> environmentVars = invocation.environmentVars;
+        String jvmOptsEnvVar;
+        if (!environmentVars.containsKey("GRADLE_OPTS")) {
+            jvmOptsEnvVar = "GRADLE_OPTS";
+        } else if (!environmentVars.containsKey("JAVA_OPTS")) {
+            jvmOptsEnvVar = "JAVA_OPTS";
+        } else {
+            // This could be handled, just not implemented yet
+            throw new UnsupportedOperationException(String.format("Both GRADLE_OPTS and JAVA_OPTS environment variables are being used. Cannot provide JVM args %s to Gradle command.", invocation.launcherJvmArgs));
+        }
+        environmentVars.put(jvmOptsEnvVar, toJvmArgsString(invocation.launcherJvmArgs));
+
+        // Add a JAVA_HOME if none provided
+        if (!environmentVars.containsKey("JAVA_HOME")) {
+            environmentVars.put("JAVA_HOME", getJavaHome().getAbsolutePath());
+        }
+    }
+
     @Override
     protected List<String> getAllArgs() {
         List<String> args = new ArrayList<String>();
@@ -97,11 +138,13 @@ class ForkingGradleExecuter extends AbstractGradleExecuter {
         builder.environment("GRADLE_OPTS", "");
         builder.environment("JAVA_OPTS", "");
 
-        builder.environment(getMergedEnvironmentVars());
-        builder.workingDir(getWorkingDir());
-        builder.setStandardInput(getStdin());
+        GradleInvocation invocation = buildInvocation();
 
-        builder.args(getAllArgs());
+        builder.environment(invocation.environmentVars);
+        builder.workingDir(getWorkingDir());
+        builder.setStandardInput(connectStdIn());
+
+        builder.args(invocation.args);
 
         ExecHandlerConfigurer configurer = OperatingSystem.current().isWindows() ? new WindowsConfigurer() : new UnixConfigurer();
         configurer.configure(builder);
@@ -121,7 +164,7 @@ class ForkingGradleExecuter extends AbstractGradleExecuter {
     }
 
     protected ForkingGradleHandle createGradleHandle(Action<ExecutionResult> resultAssertion, String encoding, Factory<ExecHandleBuilder> execHandleFactory) {
-        return new ForkingGradleHandle(resultAssertion, encoding, execHandleFactory);
+        return new ForkingGradleHandle(getStdinPipe(), isUseDaemon(), resultAssertion, encoding, execHandleFactory);
     }
 
     protected ExecutionResult doRun() {
