@@ -50,6 +50,7 @@ import org.gradle.launcher.daemon.configuration.DaemonUsage;
 import org.gradle.launcher.exec.BuildActionExecuter;
 import org.gradle.launcher.exec.BuildActionParameters;
 import org.gradle.launcher.exec.DefaultBuildActionParameters;
+import org.gradle.logging.LoggingManagerInternal;
 import org.gradle.logging.LoggingServiceRegistry;
 import org.gradle.logging.ShowStacktrace;
 import org.gradle.process.internal.JavaExecHandleBuilder;
@@ -84,6 +85,11 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
 
     public static final TestFile COMMON_TMP = new TestFile(new File("build/tmp"));
 
+    static {
+        LoggingManagerInternal loggingManager = GLOBAL_SERVICES.getFactory(LoggingManagerInternal.class).create();
+        loggingManager.start();
+    }
+
     InProcessGradleExecuter(GradleDistribution distribution, TestDirectoryProvider testDirectoryProvider) {
         super(distribution, testDirectoryProvider);
     }
@@ -165,11 +171,32 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
         InputStream originalStdIn = System.in;
         Properties originalSysProperties = new Properties();
         originalSysProperties.putAll(System.getProperties());
-        File originalUserDir = new File(originalSysProperties.getProperty("user.dir"));
+        File originalUserDir = new File(originalSysProperties.getProperty("user.dir")).getAbsoluteFile();
         Map<String, String> originalEnv = new HashMap<String, String>(System.getenv());
 
         GradleInvocation invocation = buildInvocation();
+        Set<String> changedEnvVars = new HashSet<String>(invocation.environmentVars.keySet());
 
+        try {
+            return executeBuild(invocation, outputListener, errorListener, listener);
+        } finally {
+            // Restore the environment
+            System.setProperties(originalSysProperties);
+            processEnvironment.maybeSetProcessDir(originalUserDir);
+            for (String envVar : changedEnvVars) {
+                String oldValue = originalEnv.get(envVar);
+                if (oldValue != null) {
+                    processEnvironment.maybeSetEnvironmentVariable(envVar, oldValue);
+                } else {
+                    processEnvironment.maybeRemoveEnvironmentVariable(envVar);
+                }
+            }
+            System.setProperty("user.dir", originalSysProperties.getProperty("user.dir"));
+            System.setIn(originalStdIn);
+        }
+    }
+
+    private BuildResult executeBuild(GradleInvocation invocation, StandardOutputListener outputListener, StandardOutputListener errorListener, BuildListenerImpl listener) {
         // Augment the environment for the execution
         System.setIn(connectStdIn());
         processEnvironment.maybeSetProcessDir(getWorkingDir());
@@ -218,19 +245,7 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
         } catch (ReportedException e) {
             return new BuildResult(null, e.getCause());
         } finally {
-            // Restore the environment
-            System.setProperties(originalSysProperties);
-            processEnvironment.maybeSetProcessDir(originalUserDir);
-            for (String envVar : invocation.environmentVars.keySet()) {
-                String oldValue = originalEnv.get(envVar);
-                if (oldValue != null) {
-                    processEnvironment.maybeSetEnvironmentVariable(envVar, oldValue);
-                } else {
-                    processEnvironment.maybeRemoveEnvironmentVariable(envVar);
-                }
-            }
             listenerManager.removeListener(listener);
-            System.setIn(originalStdIn);
         }
     }
 
