@@ -17,7 +17,7 @@
 package org.gradle.api.publish.ivy;
 
 import org.gradle.api.Incubating;
-import org.gradle.api.internal.HasInternalProtocol;
+import org.gradle.internal.HasInternalProtocol;
 
 /**
  * A module dependency declared in an ivy dependency descriptor published as part of an {@link IvyPublication}.
@@ -26,6 +26,21 @@ import org.gradle.api.internal.HasInternalProtocol;
 @HasInternalProtocol
 public interface IvyDependency {
 
+    /**
+     * The organisation value for this dependency.
+     */
+    String getOrganisation();
+
+    /**
+     * The module value for this dependency.
+     */
+    String getModule();
+
+    /**
+     * The revision value for this dependency.
+     */
+    String getRevision();
+
     /**
      * The configuration mapping string that will be output for this dependency.
      * A null value indicates that no "conf" attribute will be written for this dependency.
