@@ -16,9 +16,9 @@
 
 package org.gradle.api.publish.ivy.internal.dependency;
 
-import org.gradle.api.artifacts.ModuleDependency;
+import org.gradle.api.artifacts.DependencyArtifact;
 import org.gradle.api.publish.ivy.IvyDependency;
 
 public interface IvyDependencyInternal extends IvyDependency {
-    ModuleDependency getModuleDependency();
+    Iterable<DependencyArtifact> getArtifacts();
 }
