@@ -15,8 +15,6 @@
  */
 package org.gradle.launcher.daemon.protocol;
 
-import java.io.Serializable;
-
 /**
  * The supertype of all objects sent from the daemon server back to the client.
  * <p>
@@ -24,8 +22,10 @@ import java.io.Serializable;
  * <p>
  * The meaning of the value parameter is specific to each concrete subclass. The validity of {@code null}
  * is also to be defined by each subclass. This implementation does allow null values.
+ *
+ * <p>The result is the last message sent from the daemon back to the daemon client.
  */
-abstract public class Result<T> implements Serializable {
+abstract public class Result<T> extends Message {
 
     private final T value;
 
@@ -39,6 +39,6 @@ abstract public class Result<T> implements Serializable {
     
     @Override
     public String toString() {
-        return String.format("DaemonCommandResult[type=%s, value=%s]", getClass().getSimpleName(), getValue());
+        return String.format("%s[value=%s]", getClass().getSimpleName(), getValue());
     }
 }
