@@ -17,14 +17,11 @@
 package org.gradle.model.internal.core;
 
 import org.gradle.model.internal.manage.schema.extract.NodeInitializerExtractionStrategy;
-import org.gradle.model.internal.type.ModelType;
 
 public interface NodeInitializerRegistry {
-    NodeInitializer getNodeInitializer(NodeInitializerContext nodeInitializerContext);
+    NodeInitializer getNodeInitializer(NodeInitializerContext<?, ?, ?> nodeInitializerContext);
 
-    void ensureHasInitializer(NodeInitializerContext nodeInitializer);
+    void ensureHasInitializer(NodeInitializerContext<?, ?, ?> nodeInitializer);
 
     void registerStrategy(NodeInitializerExtractionStrategy strategy);
-
-    <T> boolean hasNodeInitializer(ModelType<T> type);
 }
