@@ -16,14 +16,6 @@
 
 package org.gradle.model.internal.manage.schema.extract;
 
-import org.gradle.api.Nullable;
-import org.gradle.model.internal.manage.schema.cache.ModelSchemaCache;
-
 public interface ModelSchemaExtractionStrategy {
-
-    @Nullable
-    public <T> ModelSchemaExtractionResult<T> extract(ModelSchemaExtractionContext<T> extractionContext, ModelSchemaCache cache);
-
-    Iterable<String> getSupportedManagedTypes();
-
+    <T> void extract(ModelSchemaExtractionContext<T> extractionContext);
 }
