@@ -17,16 +17,15 @@
 package org.gradle.model.internal.manage.schema.extract;
 
 import net.jcip.annotations.ThreadSafe;
-import org.gradle.internal.Factory;
 import org.gradle.model.ModelSet;
 import org.gradle.model.internal.type.ModelType;
 
 @ThreadSafe
 public class ModelSetStrategy extends SetStrategy {
 
-    public ModelSetStrategy(Factory<String> supportedTypeDescriptions) {
+    public ModelSetStrategy() {
         super(new ModelType<ModelSet<?>>() {
-        }, supportedTypeDescriptions);
+        });
     }
 
 }
