@@ -33,13 +33,10 @@ class PathBinderCreationListener extends ModelBinding {
 
     @Override
     public boolean canBindInState(ModelNode.State state) {
-        return predicate.isUntyped() || state.isAtLeast(ModelNode.State.ProjectionsDefined);
+        return predicate.isUntyped() || state.isAtLeast(ModelNode.State.Discovered);
     }
 
-    public void onCreate(ModelNodeInternal node) {
-        if (boundTo != null) {
-            throw new IllegalStateException(String.format("Reference %s for %s is already bound to %s.", predicate.getReference(), referrer, boundTo));
-        }
+    public void doOnBind(ModelNodeInternal node) {
         if (predicate.isUntyped() || isTypeCompatible(node.getPromise())) {
             boundTo = node;
             bindAction.execute(this);
