@@ -21,8 +21,10 @@ import groovy.lang.GroovyObjectSupport;
 import groovy.lang.MissingMethodException;
 import groovy.lang.MissingPropertyException;
 import net.jcip.annotations.NotThreadSafe;
+import org.gradle.api.Action;
 import org.gradle.api.GradleException;
 import org.gradle.api.internal.ClosureBackedAction;
+import org.gradle.internal.Actions;
 import org.gradle.model.internal.core.*;
 import org.gradle.model.internal.core.rule.describe.ModelRuleDescriptor;
 import org.gradle.model.internal.core.rule.describe.SimpleModelRuleDescriptor;
@@ -31,6 +33,8 @@ import org.gradle.model.internal.type.ModelType;
 
 import java.util.concurrent.atomic.AtomicBoolean;
 
+import static org.gradle.model.internal.core.DefaultNodeInitializerRegistry.DEFAULT_REFERENCE;
+
 @NotThreadSafe
 public class NonTransformedModelDslBacking extends GroovyObjectSupport {
 
@@ -64,15 +68,18 @@ public class NonTransformedModelDslBacking extends GroovyObjectSupport {
             ));
     }
 
-    private <T> void registerCreator(Class<T> type, Closure<?> closure) {
+    private <T> void register(Class<T> type, Closure<?> closure) {
+        register(type, new ClosureBackedAction<T>(closure));
+    }
+
+    private <T> void register(Class<T> type, Action<? super T> action) {
         ModelRuleDescriptor descriptor = new SimpleModelRuleDescriptor("model." + modelPath);
-        ModelReference<NodeInitializerRegistry> nodeInitializerRegistryRef = ModelReference.of(NodeInitializerRegistry.class);
-        NodeInitializerRegistry nodeInitializerRegistry = modelRegistry.realize(nodeInitializerRegistryRef.getPath(), nodeInitializerRegistryRef.getType());
-        NodeInitializer nodeInitializer = nodeInitializerRegistry.getNodeInitializer(ModelType.of(type));
-        modelRegistry.create(
-            ModelCreators.of(modelPath, nodeInitializer)
+        NodeInitializerRegistry nodeInitializerRegistry = modelRegistry.realize(DEFAULT_REFERENCE.getPath(), DEFAULT_REFERENCE.getType());
+        NodeInitializer nodeInitializer = nodeInitializerRegistry.getNodeInitializer(NodeInitializerContext.forType(ModelType.of(type)));
+        modelRegistry.register(
+            ModelRegistrations.of(modelPath, nodeInitializer)
                 .descriptor(descriptor)
-                .action(ModelActionRole.Initialize, NoInputsModelAction.of(ModelReference.of(modelPath, type), descriptor, new ClosureBackedAction<T>(closure)))
+                .action(ModelActionRole.Initialize, NoInputsModelAction.of(ModelReference.of(modelPath, type), descriptor, action))
                 .build()
         );
     }
@@ -110,12 +117,11 @@ public class NonTransformedModelDslBacking extends GroovyObjectSupport {
             } else if (args.length == 2 && args[0] instanceof Class && args[1] instanceof Closure) {
                 Class<?> clazz = (Class<?>) args[0];
                 Closure<?> closure = (Closure<?>) args[1];
-                getChildPath(name).registerCreator(clazz, closure);
+                getChildPath(name).register(clazz, closure);
                 return null;
             } else if (args.length == 1 && args[0] instanceof Class) {
                 Class<?> clazz = (Class<?>) args[0];
-                Closure<?> closure = Closure.IDENTITY;
-                getChildPath(name).registerCreator(clazz, closure);
+                getChildPath(name).register(clazz, Actions.doNothing());
                 return null;
             } else {
                 throw new MissingMethodException(name, getClass(), args);
