@@ -25,15 +25,17 @@ public class OperationTimer {
     public MeasuredOperation measure(Action<? super MeasuredOperation> action) {
         MeasuredOperation result = new MeasuredOperation();
         DateTime start = DateTime.now();
+        long startNanos = System.nanoTime();
         try {
             action.execute(result);
         } catch (Exception e) {
             result.setException(e);
         }
         DateTime end = DateTime.now();
+        long endNanos = System.nanoTime();
         result.setStart(start);
         result.setEnd(end);
-        result.setTotalTime(Duration.millis(end.getMillis() - start.getMillis()));
+        result.setTotalTime(Duration.millis((endNanos - startNanos) / 1000000L));
         return result;
     }
 }
