@@ -16,36 +16,50 @@
 
 package org.gradle.language.base.internal.model;
 
+import com.google.common.base.Joiner;
 import org.gradle.api.Action;
-import org.gradle.api.NamedDomainObjectFactory;
-import org.gradle.api.file.SourceDirectorySet;
-import org.gradle.language.base.FunctionalSourceSet;
+import org.gradle.api.internal.project.ProjectIdentifier;
 import org.gradle.language.base.LanguageSourceSet;
+import org.gradle.language.base.ProjectSourceSet;
 import org.gradle.language.base.internal.registry.LanguageRegistration;
 import org.gradle.language.base.internal.registry.LanguageRegistry;
 import org.gradle.language.base.internal.registry.LanguageTransform;
 import org.gradle.language.base.internal.registry.LanguageTransformContainer;
 import org.gradle.model.Defaults;
 import org.gradle.model.RuleSource;
+import org.gradle.model.internal.type.ModelType;
 import org.gradle.platform.base.ComponentSpec;
 import org.gradle.platform.base.internal.ComponentSpecInternal;
 
+import java.io.File;
+
+import static com.google.common.base.Strings.emptyToNull;
+
 /**
  * Cross-cutting rules for all {@link org.gradle.platform.base.ComponentSpec} instances.
  */
 @SuppressWarnings("UnusedDeclaration")
 public class ComponentRules extends RuleSource {
     @Defaults
-    void initializeSourceSets(ComponentSpec component, LanguageRegistry languageRegistry, LanguageTransformContainer languageTransforms) {
+    void initializeSourceSets(ComponentSpecInternal component, LanguageRegistry languageRegistry, LanguageTransformContainer languageTransforms) {
         for (LanguageRegistration<?> languageRegistration : languageRegistry) {
-            // TODO - allow view as internal type and remove the cast
-            ComponentSourcesRegistrationAction.create(languageRegistration, languageTransforms).execute((ComponentSpecInternal) component);
+            ComponentSourcesRegistrationAction.create(languageRegistration, languageTransforms).execute(component);
         }
     }
 
     @Defaults
-    void applyDefaultSourceConventions(final ComponentSpec component) {
-        component.getSources().afterEach(new AddDefaultSourceLocation(component.getName()));
+    void applyDefaultSourceConventions(final ComponentSpec component, final ProjectIdentifier projectIdentifier) {
+        component.getSources().afterEach(new AddDefaultSourceLocation(projectIdentifier.getProjectDir()));
+    }
+
+    @Defaults
+    void addSourcesSetsToProjectSourceSet(final ComponentSpec component, final ProjectSourceSet projectSourceSet) {
+        component.getSources().afterEach(new Action<LanguageSourceSet>() {
+            @Override
+            public void execute(LanguageSourceSet languageSourceSet) {
+                projectSourceSet.add(languageSourceSet);
+            }
+        });
     }
 
     // Currently needs to be a separate action since can't have parameterized utility methods in a RuleSource
@@ -63,22 +77,15 @@ public class ComponentRules extends RuleSource {
         }
 
         public void execute(ComponentSpecInternal componentSpecInternal) {
-            registerLanguageSourceSetFactory(componentSpecInternal);
-            createDefaultSourceSetForComponents(componentSpecInternal);
-        }
-
-        void registerLanguageSourceSetFactory(final ComponentSpecInternal component) {
-            final FunctionalSourceSet functionalSourceSet = component.getFunctionalSourceSet();
-            NamedDomainObjectFactory<? extends U> sourceSetFactory = languageRegistration.getSourceSetFactory(functionalSourceSet.getName());
-            functionalSourceSet.registerFactory(languageRegistration.getSourceSetType(), sourceSetFactory);
+            registerLanguageTypes(componentSpecInternal);
         }
 
         // If there is a transform for the language into one of the component inputs, add a default source set
-        void createDefaultSourceSetForComponents(final ComponentSpecInternal component) {
+        void registerLanguageTypes(final ComponentSpecInternal component) {
             for (LanguageTransform<?, ?> languageTransform : languageTransforms) {
-                if (languageTransform.getSourceSetType().equals(languageRegistration.getSourceSetType())
-                        && component.getInputTypes().contains(languageTransform.getOutputType())) {
-                    component.getSources().create(languageRegistration.getName(), languageRegistration.getSourceSetType());
+                if (ModelType.of(languageTransform.getSourceSetType()).equals(languageRegistration.getSourceSetType())
+                    && component.getInputTypes().contains(languageTransform.getOutputType())) {
+                    component.getSources().create(languageRegistration.getName(), languageRegistration.getSourceSetType().getConcreteClass());
                     return;
                 }
             }
@@ -86,19 +93,23 @@ public class ComponentRules extends RuleSource {
     }
 
     private static class AddDefaultSourceLocation implements Action<LanguageSourceSet> {
-        private String name;
+        private File baseDir;
 
-        public AddDefaultSourceLocation(String name) {
-            this.name = name;
+        public AddDefaultSourceLocation(File baseDir) {
+            this.baseDir = baseDir;
         }
 
         @Override
         public void execute(LanguageSourceSet languageSourceSet) {
             // Only apply default locations when none explicitly configured
-            final SourceDirectorySet source = languageSourceSet.getSource();
-            if (source.getSrcDirs().isEmpty()) {
-                source.srcDir(String.format("src/%s/%s", name, languageSourceSet.getName()));
+            if (languageSourceSet.getSource().getSrcDirs().isEmpty()) {
+                String defaultSourceDir = calculateDefaultPath(languageSourceSet);
+                languageSourceSet.getSource().srcDir(defaultSourceDir);
             }
         }
+
+        private String calculateDefaultPath(LanguageSourceSet languageSourceSet) {
+            return Joiner.on(File.separator).skipNulls().join(baseDir.getPath(), "src", emptyToNull(languageSourceSet.getParentName()), emptyToNull(languageSourceSet.getName()));
+        }
     }
 }
