@@ -17,8 +17,9 @@
 package org.gradle.language.base.internal.registry;
 
 
-import org.gradle.api.NamedDomainObjectFactory;
 import org.gradle.language.base.LanguageSourceSet;
+import org.gradle.model.internal.core.rule.describe.ModelRuleDescriptor;
+import org.gradle.model.internal.type.ModelType;
 
 /**
  * A registered language.
@@ -32,7 +33,15 @@ public interface LanguageRegistration<U extends LanguageSourceSet> {
     /**
      * The interface type of the language source set.
      */
-    Class<U> getSourceSetType();
+    ModelType<U> getSourceSetType();
 
-    NamedDomainObjectFactory<? extends U> getSourceSetFactory(String parentName);
+    /**
+     * The implementation type of the language source set.
+     */
+    ModelType<? extends U> getSourceSetImplementationType();
+
+    /**
+     * The rule that registered this type.
+     */
+    ModelRuleDescriptor getRuleDescriptor();
 }
