@@ -18,36 +18,32 @@ package org.gradle.language.base.plugins;
 import com.google.common.collect.Lists;
 import org.gradle.api.*;
 import org.gradle.api.internal.TaskInternal;
+import org.gradle.api.internal.file.FileResolver;
 import org.gradle.api.internal.project.taskfactory.ITaskFactory;
 import org.gradle.api.tasks.TaskContainer;
 import org.gradle.internal.BiAction;
-import org.gradle.internal.Factories;
-import org.gradle.internal.Transformers;
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.internal.text.TreeFormatter;
-import org.gradle.language.base.FunctionalSourceSet;
+import org.gradle.language.base.LanguageSourceSet;
 import org.gradle.language.base.ProjectSourceSet;
 import org.gradle.language.base.internal.DefaultProjectSourceSet;
-import org.gradle.language.base.internal.model.BinarySpecFactoryRegistry;
+import org.gradle.language.base.internal.LanguageSourceSetFactory;
 import org.gradle.language.base.internal.model.ComponentSpecInitializer;
-import org.gradle.language.base.internal.model.FunctionalSourceSetNodeInitializer;
+import org.gradle.language.base.internal.registry.DefaultLanguageRegistry;
+import org.gradle.language.base.internal.registry.LanguageRegistration;
+import org.gradle.language.base.internal.registry.LanguageRegistry;
 import org.gradle.model.*;
-import org.gradle.model.collection.internal.BridgedCollections;
-import org.gradle.model.collection.internal.ChildNodeInitializerStrategyAccessors;
-import org.gradle.model.collection.internal.PolymorphicModelMapProjection;
 import org.gradle.model.internal.core.*;
-import org.gradle.model.internal.core.rule.describe.ModelRuleDescriptor;
 import org.gradle.model.internal.core.rule.describe.SimpleModelRuleDescriptor;
-import org.gradle.model.internal.manage.schema.ModelSchemaStore;
-import org.gradle.model.internal.manage.schema.extract.ConstructableTypesRegistry;
-import org.gradle.model.internal.manage.schema.extract.DefaultConstructableTypesRegistry;
+import org.gradle.model.internal.manage.schema.extract.FactoryBasedNodeInitializerExtractionStrategy;
 import org.gradle.model.internal.registry.ModelRegistry;
-import org.gradle.model.internal.type.ModelType;
 import org.gradle.platform.base.BinaryContainer;
 import org.gradle.platform.base.BinarySpec;
+import org.gradle.platform.base.BinaryType;
+import org.gradle.platform.base.BinaryTypeBuilder;
+import org.gradle.platform.base.binary.BaseBinarySpec;
 import org.gradle.platform.base.internal.BinarySpecInternal;
-import org.gradle.platform.base.internal.DefaultBinaryContainer;
 
 import javax.inject.Inject;
 import java.util.List;
@@ -56,58 +52,39 @@ import java.util.Set;
 /**
  * Base plugin for language support.
  *
- * Adds a {@link org.gradle.platform.base.BinaryContainer} named {@code binaries} to the project. Adds a {@link org.gradle.language.base.ProjectSourceSet} named {@code sources} to the project.
+ * Adds a {@link BinarySpec} container named {@code binaries} to the project. Adds a {@link org.gradle.language.base.ProjectSourceSet} named {@code sources} to the project.
  *
  * For each binary instance added to the binaries container, registers a lifecycle task to create that binary.
  */
 @Incubating
 public class LanguageBasePlugin implements Plugin<Project> {
 
-    private final Instantiator instantiator;
     private final ModelRegistry modelRegistry;
 
     @Inject
-    public LanguageBasePlugin(Instantiator instantiator, ModelRegistry modelRegistry) {
-        this.instantiator = instantiator;
+    public LanguageBasePlugin(ModelRegistry modelRegistry) {
         this.modelRegistry = modelRegistry;
     }
 
     public void apply(final Project target) {
         target.getPluginManager().apply(LifecycleBasePlugin.class);
-        DefaultBinaryContainer binaries = target.getExtensions().create("binaries", DefaultBinaryContainer.class, instantiator);
-        applyRules(modelRegistry, binaries);
+        applyRules(modelRegistry);
     }
 
-    private static void applyRules(ModelRegistry modelRegistry, DefaultBinaryContainer binaries) {
-        final String descriptor = LanguageBasePlugin.class.getSimpleName() + ".apply()";
-        final ModelRuleDescriptor ruleDescriptor = new SimpleModelRuleDescriptor(descriptor);
-        ModelPath binariesPath = ModelPath.path("binaries");
+    private void applyRules(ModelRegistry modelRegistry) {
+        final String baseDescriptor = LanguageBasePlugin.class.getSimpleName() + "#";
 
-        ModelType<BinarySpec> binarySpecModelType = ModelType.of(BinarySpec.class);
-        modelRegistry.createOrReplace(
-            BridgedCollections.creator(
-                ModelReference.of(binariesPath, DefaultBinaryContainer.class),
-                Transformers.constant(binaries),
-                Named.Namer.INSTANCE,
-                descriptor,
-                BridgedCollections.itemDescriptor(descriptor)
-            )
-                .descriptor(descriptor)
-                .ephemeral(true)
-                .withProjection(PolymorphicModelMapProjection.of(binarySpecModelType,
-                    ChildNodeInitializerStrategyAccessors.constant(NodeBackedModelMap.createUsingParentNode(binarySpecModelType))))
-                .withProjection(UnmanagedModelProjection.of(DefaultBinaryContainer.class))
-                .build()
-        );
-
-        modelRegistry.configure(ModelActionRole.Defaults, DirectNodeNoInputsModelAction.of(ModelReference.of(binariesPath), ruleDescriptor, new Action<MutableModelNode>() {
+        modelRegistry.configure(ModelActionRole.Defaults, DirectNodeNoInputsModelAction.of(ModelReference.of("binaries"),
+                new SimpleModelRuleDescriptor(baseDescriptor + "attachBuildTasks"), new Action<MutableModelNode>() {
             @Override
             public void execute(MutableModelNode binariesNode) {
-                binariesNode.applyToAllLinks(ModelActionRole.Finalize, InputUsingModelAction.single(ModelReference.of(BinarySpec.class), ruleDescriptor, ModelReference.of(ITaskFactory.class), new BiAction<BinarySpec, ITaskFactory>() {
+                binariesNode.applyToAllLinks(ModelActionRole.Finalize, InputUsingModelAction.single(ModelReference.of(BinarySpec.class),
+                                new SimpleModelRuleDescriptor(baseDescriptor + "attachBuildTask"), ModelReference.of(ITaskFactory.class), new BiAction<BinarySpec, ITaskFactory>() {
                     @Override
                     public void execute(BinarySpec binary, ITaskFactory taskFactory) {
-                        if (!((BinarySpecInternal) binary).isLegacyBinary()) {
-                            TaskInternal binaryLifecycleTask = taskFactory.create(binary.getName(), DefaultTask.class);
+                        BinarySpecInternal binarySpecInternal = (BinarySpecInternal) binary;
+                        if (!binarySpecInternal.isLegacyBinary()) {
+                            TaskInternal binaryLifecycleTask = taskFactory.create(binarySpecInternal.getProjectScopedName(), DefaultTask.class);
                             binaryLifecycleTask.setGroup(LifecycleBasePlugin.BUILD_GROUP);
                             binaryLifecycleTask.setDescription(String.format("Assembles %s.", binary));
                             binary.setBuildTask(binaryLifecycleTask);
@@ -117,43 +94,43 @@ public class LanguageBasePlugin implements Plugin<Project> {
             }
         }));
 
-        modelRegistry.createOrReplace(ModelCreators.unmanagedInstance(ModelReference.of(BinarySpecFactoryRegistry.class), Factories.constant(new BinarySpecFactoryRegistry()))
-            .descriptor(ruleDescriptor)
-            .ephemeral(true)
-            .hidden(true)
-            .build());
-
         modelRegistry.getRoot().applyToAllLinksTransitive(ModelActionRole.Defaults,
             DirectNodeNoInputsModelAction.of(
                 ModelReference.of(BinarySpec.class),
-                new SimpleModelRuleDescriptor(descriptor),
+                new SimpleModelRuleDescriptor(baseDescriptor + ComponentSpecInitializer.class.getSimpleName() + ".binaryAction()"),
                 ComponentSpecInitializer.binaryAction()));
     }
 
     @SuppressWarnings("UnusedDeclaration")
     static class Rules extends RuleSource {
-
-        @Model
-        ModelSchemaStore schemaStore(ServiceRegistry serviceRegistry) {
-            return serviceRegistry.get(ModelSchemaStore.class);
+        @Service
+        LanguageSourceSetFactory languageSourceSetFactory(ServiceRegistry serviceRegistry) {
+            return new LanguageSourceSetFactory("sourceSets", serviceRegistry.get(FileResolver.class));
         }
 
         @Model
-        ConstructableTypesRegistry constructableTypesRegistry() {
-            return new DefaultConstructableTypesRegistry();
+        void binaries(BinaryContainer binaries) {}
+
+        @BinaryType
+        void registerBaseBinarySpec(BinaryTypeBuilder<BinarySpec> builder) {
+            builder.defaultImplementation(BaseBinarySpec.class);
+            builder.internalView(BinarySpecInternal.class);
         }
 
         @Mutate
-        // Path needed to avoid closing root scope before `NodeInitializerRegistry` can be finalized
-        void registerFunctionalSourceSetNodeInitializer(ConstructableTypesRegistry constructableTypesRegistry, ServiceRegistry serviceRegistry) {
-            Instantiator instantiator = serviceRegistry.get(Instantiator.class);
-            constructableTypesRegistry.registerConstructableType(ModelType.of(FunctionalSourceSet.class), new FunctionalSourceSetNodeInitializer(instantiator));
+        void registerSourceSetTypes(LanguageSourceSetFactory languageSourceSetFactory, LanguageRegistry languageRegistry) {
+            for (LanguageRegistration<?> languageRegistration : languageRegistry) {
+                register(languageSourceSetFactory, languageRegistration);
+            }
+        }
+
+        private <T extends LanguageSourceSet> void register(LanguageSourceSetFactory lssFactory, LanguageRegistration<T> languageRegistration) {
+            lssFactory.register(languageRegistration.getSourceSetType(), languageRegistration.getSourceSetImplementationType(), languageRegistration.getRuleDescriptor());
         }
 
         @Mutate
-        // Path needed to avoid closing root scope before `NodeInitializerRegistry` can be finalized
-        void registerNodeInitializerExtractionStrategies(NodeInitializerRegistry nodeInitializerRegistry, ConstructableTypesRegistry constructableTypesRegistry) {
-            nodeInitializerRegistry.registerStrategy(constructableTypesRegistry);
+        void registerSourceSetNodeInitializer(NodeInitializerRegistry nodeInitializerRegistry, LanguageSourceSetFactory languageSourceSetFactory) {
+            nodeInitializerRegistry.registerStrategy(new FactoryBasedNodeInitializerExtractionStrategy<LanguageSourceSet>(languageSourceSetFactory));
         }
 
         @Model
@@ -161,8 +138,13 @@ public class LanguageBasePlugin implements Plugin<Project> {
             return serviceRegistry.get(Instantiator.class).newInstance(DefaultProjectSourceSet.class);
         }
 
+        @Service
+        LanguageRegistry languages() {
+            return new DefaultLanguageRegistry();
+        }
+
         @Mutate
-        void copyBinaryTasksToTaskContainer(TaskContainer tasks, BinaryContainer binaries) {
+        void copyBinaryTasksToTaskContainer(TaskContainer tasks, ModelMap<BinarySpec> binaries) {
             for (BinarySpec binary : binaries) {
                 tasks.addAll(binary.getTasks());
                 Task buildTask = binary.getBuildTask();
@@ -173,10 +155,10 @@ public class LanguageBasePlugin implements Plugin<Project> {
         }
 
         @Mutate
-        void attachBinariesToAssembleLifecycle(@Path("tasks.assemble") Task assemble, BinaryContainer binaries) {
+        void attachBinariesToAssembleLifecycle(@Path("tasks.assemble") Task assemble, ModelMap<BinarySpecInternal> binaries) {
             List<BinarySpecInternal> notBuildable = Lists.newArrayList();
             boolean hasBuildableBinaries = false;
-            for (BinarySpecInternal binary : binaries.withType(BinarySpecInternal.class)) {
+            for (BinarySpecInternal binary : binaries) {
                 if (!binary.isLegacyBinary()) {
                     if (binary.isBuildable()) {
                         assemble.dependsOn(binary);
@@ -191,11 +173,6 @@ public class LanguageBasePlugin implements Plugin<Project> {
             }
         }
 
-        @Defaults
-        void registerBinaryFactories(BinaryContainer binaries, BinarySpecFactoryRegistry binaryFactoryRegistry) {
-            binaryFactoryRegistry.copyInto(binaries);
-        }
-
         private static class CheckForNotBuildableBinariesAction implements Action<Task> {
             private final List<BinarySpecInternal> notBuildable;
 
@@ -212,7 +189,7 @@ public class LanguageBasePlugin implements Plugin<Project> {
                     formatter.node("No buildable binaries found");
                     formatter.startChildren();
                     for (BinarySpecInternal binary : notBuildable) {
-                        formatter.node(binary.getName());
+                        formatter.node(binary.getDisplayName());
                         formatter.startChildren();
                         binary.getBuildAbility().explain(formatter);
                         formatter.endChildren();
