@@ -15,11 +15,12 @@
  */
 package org.gradle.platform.base;
 
-import org.gradle.api.*;
+import org.gradle.api.Incubating;
+import org.gradle.model.ModelMap;
 
 /**
  * A container for project binaries, which represent physical artifacts that can run on a particular platform or runtime.
  * Added to a project by the {@link org.gradle.language.base.plugins.LanguageBasePlugin}.
  */
 @Incubating
-public interface BinaryContainer extends ExtensiblePolymorphicDomainObjectContainer<BinarySpec> {}
+public interface BinaryContainer extends ModelMap<BinarySpec> {}
