@@ -16,13 +16,13 @@
 
 package org.gradle.platform.base.internal;
 
-import org.gradle.api.internal.rules.NamedDomainObjectFactoryRegistry;
-import org.gradle.language.base.LanguageSourceSet;
-import org.gradle.model.internal.type.ModelType;
 import org.gradle.platform.base.BinarySpec;
 
 public interface BinarySpecInternal extends BinarySpec {
-    ModelType<BinarySpec> PUBLIC_MODEL_TYPE = ModelType.of(BinarySpec.class);
+    /**
+     * Returns a name for this binary that is unique for all binaries in the current project.
+     */
+    String getProjectScopedName();
 
     Class<? extends BinarySpec> getPublicType();
 
@@ -32,5 +32,7 @@ public interface BinarySpecInternal extends BinarySpec {
 
     boolean isLegacyBinary();
 
-    NamedDomainObjectFactoryRegistry<LanguageSourceSet> getEntityInstantiator();
+    BinaryNamingScheme getNamingScheme();
+
+    void setNamingScheme(BinaryNamingScheme namingScheme);
 }
