@@ -16,7 +16,6 @@
 
 package org.gradle.platform.base.internal;
 
-import org.gradle.language.base.FunctionalSourceSet;
 import org.gradle.platform.base.ComponentSpec;
 import org.gradle.platform.base.TransformationFileType;
 
@@ -24,8 +23,6 @@ import java.util.Set;
 
 public interface ComponentSpecInternal extends ComponentSpec {
 
-    FunctionalSourceSet getSources();
-
-    Set<Class<? extends TransformationFileType>> getInputTypes();
+    Set<? extends Class<? extends TransformationFileType>> getInputTypes();
 
 }
