@@ -22,7 +22,6 @@ import org.gradle.api.internal.resolve.DefaultProjectModelResolver;
 import org.gradle.api.internal.resolve.ProjectModelResolver;
 import org.gradle.internal.service.ServiceRegistration;
 import org.gradle.internal.service.scopes.PluginServiceRegistry;
-import org.gradle.language.base.internal.resolve.DependentSourceSetLocalComponentConverter;
 import org.gradle.model.internal.inspect.MethodModelRuleExtractor;
 import org.gradle.model.internal.manage.schema.ModelSchemaStore;
 import org.gradle.model.internal.manage.schema.extract.ModelSchemaAspectExtractionStrategy;
@@ -49,10 +48,6 @@ public class ComponentModelBaseServiceRegistry implements PluginServiceRegistry
     }
 
     private static class BuildScopeServices {
-        DependentSourceSetLocalComponentConverter createLocalComponentFactory() {
-            return new DependentSourceSetLocalComponentConverter();
-        }
-
         ProjectModelResolver createProjectLocator(final ProjectRegistry<ProjectInternal> projectRegistry) {
             return new DefaultProjectModelResolver(projectRegistry);
         }
