@@ -17,15 +17,22 @@
 package org.gradle.jvm;
 
 import org.gradle.api.Incubating;
+import org.gradle.api.artifacts.component.LibraryBinaryIdentifier;
 import org.gradle.internal.HasInternalProtocol;
 
 import java.io.File;
+import java.util.Set;
 
 /**
  * Definition of a Jar file binary that is to be built by Gradle.
  */
 @Incubating @HasInternalProtocol
 public interface JarBinarySpec extends JvmBinarySpec {
+    /**
+     * The unique identifier of this JarBinarySpec.
+     */
+    LibraryBinaryIdentifier getId();
+
     /**
      * The jar file output for this binary.
      */
@@ -35,4 +42,18 @@ public interface JarBinarySpec extends JvmBinarySpec {
      * Sets the jar file output for this binary.
      */
     void setJarFile(File jarFile);
+
+    /**
+     * The API jar file output for this binary.
+     */
+    File getApiJarFile();
+
+    /**
+     * Sets the API jar file output for this binary.
+     */
+    void setApiJarFile(File jarFile);
+
+    void setExportedPackages(Set<String> exportedPackages);
+
+    Set<String> getExportedPackages();
 }
