@@ -16,11 +16,27 @@
 
 package org.gradle.jvm.internal;
 
-import org.gradle.platform.base.internal.BinarySpecInternal;
 import org.gradle.jvm.JarBinarySpec;
+import org.gradle.platform.base.DependencySpec;
+import org.gradle.platform.base.internal.BinarySpecInternal;
+
+import java.util.Collection;
 
 public interface JarBinarySpecInternal extends JarBinarySpec, BinarySpecInternal {
-    String getBaseName();
 
-    void setBaseName(String baseName);
+    void setApiDependencies(Collection<DependencySpec> apiDependencies);
+
+    Collection<DependencySpec> getApiDependencies();
+
+    /**
+     * Sets the component level dependencies of this binary.
+     */
+    void setDependencies(Collection<DependencySpec> dependencies);
+
+    /**
+     * The component level dependencies of this binary.
+     */
+    Collection<DependencySpec> getDependencies();
+
+    JarFile getApiJar();
 }
