@@ -16,8 +16,8 @@
 
 package org.gradle.jvm.plugins;
 
-import com.google.common.collect.ImmutableList;
 import org.gradle.api.*;
+import org.gradle.api.internal.project.ProjectIdentifier;
 import org.gradle.api.tasks.Copy;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.jvm.JarBinarySpec;
@@ -27,10 +27,10 @@ import org.gradle.jvm.internal.toolchain.JavaToolChainInternal;
 import org.gradle.jvm.platform.JavaPlatform;
 import org.gradle.jvm.platform.internal.DefaultJavaPlatform;
 import org.gradle.jvm.tasks.Jar;
+import org.gradle.jvm.tasks.api.ApiJar;
 import org.gradle.jvm.toolchain.JavaToolChainRegistry;
 import org.gradle.jvm.toolchain.internal.DefaultJavaToolChainRegistry;
-import org.gradle.language.base.internal.BuildDirHolder;
-import org.gradle.language.base.internal.tasks.apigen.ApiStubGenerator;
+import org.gradle.language.base.internal.ProjectLayout;
 import org.gradle.model.*;
 import org.gradle.model.internal.registry.ModelRegistry;
 import org.gradle.model.internal.type.ModelType;
@@ -48,8 +48,9 @@ import java.util.Set;
 import static org.apache.commons.lang.StringUtils.capitalize;
 
 /**
- * Base plugin for JVM component support. Applies the {@link org.gradle.language.base.plugins.ComponentModelBasePlugin}. Registers the {@link JvmLibrarySpec} library type for the components
- * container.
+ * Base plugin for JVM component support. Applies the
+ * {@link org.gradle.language.base.plugins.ComponentModelBasePlugin}.
+ * Registers the {@link JvmLibrarySpec} library type for the components container.
  */
 @Incubating
 public class JvmComponentPlugin implements Plugin<Project> {
@@ -80,11 +81,6 @@ public class JvmComponentPlugin implements Plugin<Project> {
             builder.internalView(JarBinarySpecInternal.class);
         }
 
-        @Model
-        public BinaryNamingSchemeBuilder binaryNamingSchemeBuilder() {
-            return new DefaultBinaryNamingSchemeBuilder();
-        }
-
         @Model
         public JavaToolChainRegistry javaToolChain(ServiceRegistry serviceRegistry) {
             JavaToolChainInternal toolChain = serviceRegistry.get(JavaToolChainInternal.class);
@@ -92,8 +88,8 @@ public class JvmComponentPlugin implements Plugin<Project> {
         }
 
         @Model
-        public BuildDirHolder buildDirHolder(@Path("buildDir") File buildDir) {
-            return new BuildDirHolder(buildDir);
+        public ProjectLayout projectLayout(ProjectIdentifier projectIdentifier, @Path("buildDir") File buildDir) {
+            return new ProjectLayout(projectIdentifier, buildDir);
         }
 
         @Mutate
@@ -102,28 +98,30 @@ public class JvmComponentPlugin implements Plugin<Project> {
         }
 
         @ComponentBinaries
-        public void createBinaries(ModelMap<JarBinarySpec> binaries, BinaryNamingSchemeBuilder namingSchemeBuilder,
-                                   PlatformResolvers platforms, final JvmLibrarySpec jvmLibrary) {
+        public void createBinaries(ModelMap<JarBinarySpec> binaries, PlatformResolvers platforms, final JvmLibrarySpecInternal jvmLibrary) {
             List<JavaPlatform> selectedPlatforms = resolvePlatforms(platforms, jvmLibrary);
-            final Set<String> exportedPackages = jvmLibrary.getExportedPackages();
-            final Collection<DependencySpec> apiDependencies = jvmLibrary.getApiDependencies();
+            final Set<String> exportedPackages = exportedPackagesOf(jvmLibrary);
+            final Collection<DependencySpec> apiDependencies = apiDependenciesOf(jvmLibrary);
+            final Collection<DependencySpec> dependencies = componentDependenciesOf(jvmLibrary);
             for (final JavaPlatform platform : selectedPlatforms) {
-                String binaryName = buildBinaryName(jvmLibrary, namingSchemeBuilder, selectedPlatforms, platform);
-                binaries.create(binaryName, new Action<JarBinarySpec>() {
+                final BinaryNamingScheme namingScheme = namingSchemeFor(jvmLibrary, selectedPlatforms, platform);
+                binaries.create(namingScheme.getBinaryName(), new Action<JarBinarySpec>() {
                     @Override
-                    public void execute(JarBinarySpec jarBinary) {
+                    public void execute(JarBinarySpec jarBinarySpec) {
+                        JarBinarySpecInternal jarBinary = (JarBinarySpecInternal) jarBinarySpec;
+                        jarBinary.setNamingScheme(namingScheme);
                         jarBinary.setTargetPlatform(platform);
                         jarBinary.setExportedPackages(exportedPackages);
                         jarBinary.setApiDependencies(apiDependencies);
+                        jarBinary.setDependencies(dependencies);
                     }
                 });
             }
         }
 
         private List<JavaPlatform> resolvePlatforms(final PlatformResolvers platformResolver,
-                                                    JvmLibrarySpec jvmLibrarySpec) {
-            List<PlatformRequirement> targetPlatforms =
-                ((JvmLibrarySpecInternal) jvmLibrarySpec).getTargetPlatforms();
+                                                    JvmLibrarySpecInternal jvmLibrarySpec) {
+            List<PlatformRequirement> targetPlatforms = jvmLibrarySpec.getTargetPlatforms();
             if (targetPlatforms.isEmpty()) {
                 targetPlatforms = Collections.singletonList(
                     DefaultPlatformRequirement.create(DefaultJavaPlatform.current().getName()));
@@ -136,68 +134,80 @@ public class JvmComponentPlugin implements Plugin<Project> {
             });
         }
 
-        private String buildBinaryName(JvmLibrarySpec jvmLibrary, BinaryNamingSchemeBuilder namingSchemeBuilder,
-                                       List<JavaPlatform> selectedPlatforms, JavaPlatform platform) {
-            BinaryNamingSchemeBuilder componentBuilder = namingSchemeBuilder
-                .withComponentName(jvmLibrary.getName())
-                .withTypeString("jar");
-            if (selectedPlatforms.size() > 1) {
-                componentBuilder = componentBuilder.withVariantDimension(platform.getName());
-            }
-            return componentBuilder.build().getLifecycleTaskName();
+        private static Set<String> exportedPackagesOf(JvmLibrarySpecInternal jvmLibrary) {
+            return jvmLibrary.getApi().getExports();
+        }
+
+        private static Collection<DependencySpec> apiDependenciesOf(JvmLibrarySpecInternal jvmLibrary) {
+            return jvmLibrary.getApi().getDependencies().getDependencies();
+        }
+
+        private static Collection<DependencySpec> componentDependenciesOf(JvmLibrarySpecInternal jvmLibrary) {
+            return jvmLibrary.getDependencies().getDependencies();
+        }
+
+        private BinaryNamingScheme namingSchemeFor(JvmLibrarySpec jvmLibrary, List<JavaPlatform> selectedPlatforms, JavaPlatform platform) {
+            return DefaultBinaryNamingScheme.component(jvmLibrary.getName())
+                    .withBinaryType("Jar")
+                    .withRole("jar", true)
+                    .withVariantDimension(platform, selectedPlatforms);
         }
 
         @BinaryTasks
-        public void createTasks(ModelMap<Task> tasks, final JarBinarySpec binary, final @Path("buildDir") File buildDir) {
-           final File runtimeClassesDir = binary.getClassesDir();
+        public void createTasks(ModelMap<Task> tasks, final JarBinarySpecInternal binary, final @Path("buildDir") File buildDir) {
+            final File runtimeClassesDir = binary.getClassesDir();
+            final File resourcesDir = binary.getResourcesDir();
             final File runtimeJarDestDir = binary.getJarFile().getParentFile();
             final String runtimeJarArchiveName = binary.getJarFile().getName();
-            final String createRuntimeJar = "create" + capitalize(binary.getName());
+            final String createRuntimeJar = "create" + capitalize(binary.getProjectScopedName());
             tasks.create(createRuntimeJar, Jar.class, new Action<Jar>() {
                 @Override
                 public void execute(Jar jar) {
                     jar.setDescription(String.format("Creates the binary file for %s.", binary));
                     jar.from(runtimeClassesDir);
-                    jar.from(binary.getResourcesDir());
+                    jar.from(resourcesDir);
                     jar.setDestinationDir(runtimeJarDestDir);
                     jar.setArchiveName(runtimeJarArchiveName);
                 }
             });
 
-            String binaryName = binary.getName();
-            if (!binaryName.endsWith("Jar")) {
-                return;
-            }
-
-            String libName = binaryName.substring(0, binaryName.lastIndexOf("Jar"));
-            String createApiJar = "create" + capitalize(libName + "ApiJar");
-            final ImmutableList<String> allowedPackages = ImmutableList.copyOf(binary.getExportedPackages());
-            if (allowedPackages.isEmpty()) {
-                tasks.create(createApiJar, Copy.class, new Action<Copy>() {
+            final JarFile apiJar = binary.getApiJar();
+            final Set<String> exportedPackages = binary.getExportedPackages();
+            String apiJarTaskName = apiJarTaskName(binary);
+            if (exportedPackages.isEmpty()) {
+                tasks.create(apiJarTaskName, Copy.class, new Action<Copy>() {
                     @Override
                     public void execute(Copy copy) {
                         copy.setDescription(String.format("Creates the API binary file for %s.", binary));
                         copy.from(new File(runtimeJarDestDir, runtimeJarArchiveName));
-                        copy.setDestinationDir(binary.getApiJarFile().getParentFile());
+                        copy.setDestinationDir(apiJar.getFile().getParentFile());
                         copy.dependsOn(createRuntimeJar);
+                        apiJar.setBuildTask(copy);
                     }
                 });
             } else {
-                final ApiStubGenerator stubGenerator = new ApiStubGenerator(allowedPackages);
-                tasks.create(createApiJar, StubbedJar.class, new Action<StubbedJar>() {
+                tasks.create(apiJarTaskName, ApiJar.class, new Action<ApiJar>() {
                     @Override
-                    public void execute(StubbedJar jar) {
-                        final File apiClassesDir = new File(new File(buildDir, "apiClasses"), runtimeClassesDir.getName());
+                    public void execute(ApiJar jar) {
+                        final File apiClassesDir = binary.getNamingScheme().getOutputDirectory(buildDir, "apiClasses");
                         jar.setDescription(String.format("Creates the API binary file for %s.", binary));
                         jar.setRuntimeClassesDir(runtimeClassesDir);
-                        jar.setExportedPackages(allowedPackages);
+                        jar.setExportedPackages(exportedPackages);
                         jar.setApiClassesDir(apiClassesDir);
-                        jar.setDestinationDir(binary.getApiJarFile().getParentFile());
-                        jar.setArchiveName(binary.getApiJarFile().getName());
-                        jar.dependsOn(createRuntimeJar);
+                        jar.setDestinationDir(apiJar.getFile().getParentFile());
+                        jar.setArchiveName(apiJar.getFile().getName());
+                        apiJar.setBuildTask(jar);
                     }
                 });
             }
         }
+
+        private String apiJarTaskName(JarBinarySpecInternal binary) {
+            String binaryName = binary.getProjectScopedName();
+            String libName = binaryName.endsWith("Jar")
+                    ? binaryName.substring(0, binaryName.length() - 3)
+                    : binaryName;
+            return libName + "ApiJar";
+        }
     }
 }
