@@ -19,15 +19,10 @@ package org.gradle.nativeplatform.internal.prebuilt;
 import org.gradle.api.DomainObjectSet;
 import org.gradle.api.NamedDomainObjectSet;
 import org.gradle.api.internal.resolve.ProjectModelResolver;
-import org.gradle.model.internal.core.ModelPath;
 import org.gradle.model.internal.registry.ModelRegistry;
-import org.gradle.model.internal.type.ModelType;
 import org.gradle.nativeplatform.*;
 import org.gradle.nativeplatform.internal.resolve.LibraryBinaryLocator;
 
-import java.util.ArrayList;
-import java.util.List;
-
 public class PrebuiltLibraryBinaryLocator implements LibraryBinaryLocator {
     private final ProjectModelResolver projectModelResolver;
 
@@ -37,24 +32,21 @@ public class PrebuiltLibraryBinaryLocator implements LibraryBinaryLocator {
 
     public DomainObjectSet<NativeLibraryBinary> getBinaries(NativeLibraryRequirement requirement) {
         ModelRegistry projectModel = projectModelResolver.resolveProjectModel(requirement.getProjectPath());
-        NamedDomainObjectSet<PrebuiltLibraries> repositories = projectModel.realize(ModelPath.path("repositories"), ModelType.of(Repositories.class)).withType(PrebuiltLibraries.class);
+        NamedDomainObjectSet<PrebuiltLibraries> repositories = projectModel.realize("repositories", Repositories.class).withType(PrebuiltLibraries.class);
         if (repositories.isEmpty()) {
-            throw new PrebuiltLibraryResolveException("Project does not have any prebuilt library repositories.");
+            return null;
         }
         PrebuiltLibrary prebuiltLibrary = getPrebuiltLibrary(repositories, requirement.getLibraryName());
-        return prebuiltLibrary.getBinaries();
+        return prebuiltLibrary != null ? prebuiltLibrary.getBinaries() : null;
     }
 
     private PrebuiltLibrary getPrebuiltLibrary(NamedDomainObjectSet<PrebuiltLibraries> repositories, String libraryName) {
-        List<String> repositoryNames = new ArrayList<String>();
         for (PrebuiltLibraries prebuiltLibraries : repositories) {
-            repositoryNames.add(prebuiltLibraries.getName());
             PrebuiltLibrary prebuiltLibrary = prebuiltLibraries.resolveLibrary(libraryName);
             if (prebuiltLibrary != null) {
                 return prebuiltLibrary;
             }
         }
-        throw new PrebuiltLibraryResolveException(
-                String.format("Prebuilt library with name '%s' not found in repositories '%s'.", libraryName, repositoryNames));
+        return null;
     }
 }
