@@ -15,9 +15,11 @@
  */
 package org.gradle.nativeplatform.plugins;
 
+import com.google.common.collect.Lists;
 import org.apache.commons.lang.StringUtils;
 import org.gradle.api.*;
 import org.gradle.api.artifacts.repositories.ArtifactRepository;
+import org.gradle.api.file.FileCollection;
 import org.gradle.api.file.SourceDirectorySet;
 import org.gradle.api.internal.DefaultPolymorphicDomainObjectContainer;
 import org.gradle.api.internal.file.FileResolver;
@@ -31,6 +33,7 @@ import org.gradle.language.base.internal.LanguageSourceSetInternal;
 import org.gradle.language.base.internal.SourceTransformTaskConfig;
 import org.gradle.language.base.internal.registry.LanguageTransformContainer;
 import org.gradle.language.base.plugins.ComponentModelBasePlugin;
+import org.gradle.language.base.plugins.LifecycleBasePlugin;
 import org.gradle.language.nativeplatform.DependentSourceSet;
 import org.gradle.language.nativeplatform.HeaderExportingSourceSet;
 import org.gradle.language.nativeplatform.internal.DependentSourceSetInternal;
@@ -46,15 +49,19 @@ import org.gradle.nativeplatform.internal.prebuilt.PrebuiltLibraryInitializer;
 import org.gradle.nativeplatform.platform.NativePlatform;
 import org.gradle.nativeplatform.platform.internal.DefaultNativePlatform;
 import org.gradle.nativeplatform.platform.internal.NativePlatforms;
-import org.gradle.nativeplatform.tasks.PrefixHeaderFileGenerateTask;
+import org.gradle.nativeplatform.tasks.*;
+import org.gradle.nativeplatform.test.internal.NativeTestSuiteBinarySpecInternal;
 import org.gradle.nativeplatform.toolchain.internal.DefaultNativeToolChainRegistry;
 import org.gradle.nativeplatform.toolchain.internal.NativeToolChainRegistryInternal;
 import org.gradle.nativeplatform.toolchain.internal.PreCompiledHeader;
 import org.gradle.platform.base.*;
+import org.gradle.platform.base.internal.BinaryNamingScheme;
 import org.gradle.platform.base.internal.PlatformResolvers;
 
 import javax.inject.Inject;
 import java.io.File;
+import java.util.List;
+import java.util.concurrent.Callable;
 
 /**
  * A plugin that sets up the infrastructure for defining native binaries.
@@ -176,7 +183,7 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
             componentSpecs.afterEach(new Action<NativeComponentSpec>() {
                 @Override
                 public void execute(NativeComponentSpec componentSpec) {
-                    componentSpec.getSource().withType(LanguageSourceSet.class).afterEach(new Action<LanguageSourceSet>() {
+                    componentSpec.getSources().withType(LanguageSourceSet.class).afterEach(new Action<LanguageSourceSet>() {
                         @Override
                         public void execute(LanguageSourceSet languageSourceSet) {
                             LanguageSourceSetInternal internalSourceSet = (LanguageSourceSetInternal) languageSourceSet;
@@ -199,7 +206,7 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
             componentSpecs.afterEach(new Action<NativeComponentSpec>() {
                 @Override
                 public void execute(final NativeComponentSpec componentSpec) {
-                    componentSpec.getSource().withType(DependentSourceSet.class).afterEach(new Action<DependentSourceSet>() {
+                    componentSpec.getSources().withType(DependentSourceSet.class).afterEach(new Action<DependentSourceSet>() {
                         @Override
                         public void execute(DependentSourceSet dependentSourceSet) {
                             if (dependentSourceSet.getPreCompiledHeader() != null) {
@@ -218,7 +225,7 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
         @Mutate
         void configurePrefixHeaderGenerationTasks(final TaskContainer tasks, ModelMap<NativeComponentSpec> nativeComponents) {
             for (final NativeComponentSpec nativeComponentSpec : nativeComponents.values()) {
-                for (final DependentSourceSet dependentSourceSet : nativeComponentSpec.getSource().withType(DependentSourceSet.class).values()) {
+                for (final DependentSourceSet dependentSourceSet : nativeComponentSpec.getSources().withType(DependentSourceSet.class).values()) {
                     final DependentSourceSetInternal internalSourceSet = (DependentSourceSetInternal) dependentSourceSet;
                     if (internalSourceSet.getPrefixHeaderFile() != null) {
                         String taskName = String.format("generate%s%sPrefixHeaderFile", StringUtils.capitalize(nativeComponentSpec.getName()), StringUtils.capitalize(dependentSourceSet.getName()));
@@ -235,31 +242,29 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
         }
 
         @Mutate
-        void configurePreCompiledHeaderCompileTasks(ModelMap<NativeBinarySpecInternal> binaries, final ServiceRegistry serviceRegistry, final LanguageTransformContainer languageTransforms, final @Path("buildDir") File buildDir) {
-            binaries.all(new Action<NativeBinarySpecInternal>() {
-                @Override
-                public void execute(final NativeBinarySpecInternal nativeBinarySpec) {
-                    for (final PchEnabledLanguageTransform<?> transform : languageTransforms.withType(PchEnabledLanguageTransform.class)) {
-                        nativeBinarySpec.getSource().withType(transform.getSourceSetType(), new Action<LanguageSourceSet>() {
-                            @Override
-                            public void execute(final LanguageSourceSet languageSourceSet) {
-                                final DependentSourceSet dependentSourceSet = (DependentSourceSet) languageSourceSet;
-                                if (dependentSourceSet.getPreCompiledHeader() != null) {
-                                    nativeBinarySpec.getPrefixFileToPCH().put(((DependentSourceSetInternal) dependentSourceSet).getPrefixHeaderFile(), new PreCompiledHeader());
-                                    final SourceTransformTaskConfig pchTransformTaskConfig = transform.getPchTransformTask();
-                                    String pchTaskName = String.format("%s%s%sPreCompiledHeader", pchTransformTaskConfig.getTaskPrefix(), StringUtils.capitalize(nativeBinarySpec.getName()), StringUtils.capitalize(dependentSourceSet.getName()));
-                                    nativeBinarySpec.getTasks().create(pchTaskName, pchTransformTaskConfig.getTaskType(), new Action<DefaultTask>() {
-                                        @Override
-                                        public void execute(DefaultTask task) {
-                                            pchTransformTaskConfig.configureTask(task, nativeBinarySpec, dependentSourceSet);
-                                        }
-                                    });
-                                }
+        void configurePreCompiledHeaderCompileTasks(final TaskContainer tasks, ModelMap<NativeBinarySpecInternal> binaries, final LanguageTransformContainer languageTransforms, final ServiceRegistry serviceRegistry) {
+            for (final NativeBinarySpecInternal nativeBinarySpec : binaries.values()) {
+                for (final PchEnabledLanguageTransform<?> transform : languageTransforms.withType(PchEnabledLanguageTransform.class)) {
+                    nativeBinarySpec.getInputs().withType(transform.getSourceSetType(), new Action<LanguageSourceSet>() {
+                        @Override
+                        public void execute(final LanguageSourceSet languageSourceSet) {
+                            final DependentSourceSet dependentSourceSet = (DependentSourceSet) languageSourceSet;
+                            if (dependentSourceSet.getPreCompiledHeader() != null) {
+                                nativeBinarySpec.getPrefixFileToPCH().put(((DependentSourceSetInternal) dependentSourceSet).getPrefixHeaderFile(), new PreCompiledHeader());
+                                final SourceTransformTaskConfig pchTransformTaskConfig = transform.getPchTransformTask();
+                                String pchTaskName = String.format("%s%s%sPreCompiledHeader", pchTransformTaskConfig.getTaskPrefix(), StringUtils.capitalize(nativeBinarySpec.getProjectScopedName()), StringUtils.capitalize(dependentSourceSet.getName()));
+                                Task pchTask = tasks.create(pchTaskName, pchTransformTaskConfig.getTaskType(), new Action<DefaultTask>() {
+                                    @Override
+                                    public void execute(DefaultTask task) {
+                                        pchTransformTaskConfig.configureTask(task, nativeBinarySpec, dependentSourceSet, serviceRegistry);
+                                    }
+                                });
+                                nativeBinarySpec.getTasks().add(pchTask);
                             }
-                        });
-                    }
+                        }
+                    });
                 }
-            });
+            }
         }
 
         private void maybeSetSourceDir(SourceDirectorySet sourceSet, Task task, String propertyName) {
@@ -269,6 +274,133 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
             }
         }
 
+        @BinaryTasks
+        public void createTasks(ModelMap<Task> tasks, final SharedLibraryBinarySpecInternal binary) {
+            String taskName = binary.getNamingScheme().getTaskName("link");
+            tasks.create(taskName, LinkSharedLibrary.class, new Action<LinkSharedLibrary>() {
+                @Override
+                public void execute(LinkSharedLibrary linkTask) {
+                    linkTask.setDescription("Links " + binary.getDisplayName());
+                    linkTask.setToolChain(binary.getToolChain());
+                    linkTask.setTargetPlatform(binary.getTargetPlatform());
+                    linkTask.setOutputFile(binary.getSharedLibraryFile());
+                    linkTask.setInstallName(binary.getSharedLibraryFile().getName());
+                    linkTask.setLinkerArgs(binary.getLinker().getArgs());
+
+                    linkTask.lib(new BinaryLibs(binary) {
+                        @Override
+                        protected FileCollection getFiles(NativeDependencySet nativeDependencySet) {
+                            return nativeDependencySet.getLinkFiles();
+                        }
+                    });
+                }
+            });
+        }
+
+        @BinaryTasks
+        public void createTasks(ModelMap<Task> tasks, final StaticLibraryBinarySpecInternal binary) {
+            String taskName = binary.getNamingScheme().getTaskName("create");
+            tasks.create(taskName, CreateStaticLibrary.class, new Action<CreateStaticLibrary>() {
+                @Override
+                public void execute(CreateStaticLibrary task) {
+                    task.setDescription("Creates " + binary.getDisplayName());
+                    task.setToolChain(binary.getToolChain());
+                    task.setTargetPlatform(binary.getTargetPlatform());
+                    task.setOutputFile(binary.getStaticLibraryFile());
+                    task.setStaticLibArgs(binary.getStaticLibArchiver().getArgs());
+                }
+            });
+        }
+
+        @BinaryTasks
+        public void createTasks(ModelMap<Task> tasks, final NativeExecutableBinarySpecInternal executableBinary) {
+            createExecutableTask(tasks, executableBinary, executableBinary.getExecutable().getFile());
+        }
+
+        @BinaryTasks
+        public void createTasks(ModelMap<Task> tasks, final NativeTestSuiteBinarySpecInternal testSuiteBinary) {
+            createExecutableTask(tasks, testSuiteBinary, testSuiteBinary.getExecutableFile());
+        }
+
+        private void createExecutableTask(ModelMap<Task> tasks, final NativeBinarySpecInternal binary, final File executableFile) {
+            String taskName = binary.getNamingScheme().getTaskName("link");
+            tasks.create(taskName, LinkExecutable.class, new Action<LinkExecutable>() {
+                @Override
+                public void execute(LinkExecutable linkTask) {
+                    linkTask.setDescription("Links " + binary.getDisplayName());
+                    linkTask.setToolChain(binary.getToolChain());
+                    linkTask.setTargetPlatform(binary.getTargetPlatform());
+                    linkTask.setOutputFile(executableFile);
+                    linkTask.setLinkerArgs(binary.getLinker().getArgs());
+
+                    linkTask.lib(new BinaryLibs(binary) {
+                        @Override
+                        protected FileCollection getFiles(NativeDependencySet nativeDependencySet) {
+                            return nativeDependencySet.getLinkFiles();
+                        }
+                    });
+                }
+            });
+        }
+
+        /**
+         * Can't use @BinaryTasks because the binary is not _built-by_ the install task, but it is associated with it.
+         * Rule is called multiple times, so need to check for task existence before creating.
+         */
+        @Defaults
+        void createInstallTasks(TaskContainer tasks, ModelMap<NativeBinarySpecInternal> binaries, @Path("buildDir") File buildDir) {
+            // There's no common API for NativeExecutableBinarySpec and NativeTestSuiteBinarySpec
+            for (NativeExecutableBinarySpecInternal binary : binaries.withType(NativeExecutableBinarySpecInternal.class).values()) {
+                Task installTask = createInstallTask(tasks, binary, binary.getInstallation(), binary.getExecutable(), buildDir, binary.getNamingScheme());
+                binary.getTasks().add(installTask);
+            }
+            for (NativeTestSuiteBinarySpecInternal binary : binaries.withType(NativeTestSuiteBinarySpecInternal.class).values()) {
+                Task installTask = createInstallTask(tasks, binary, binary.getInstallation(), binary.getExecutable(), buildDir, binary.getNamingScheme());
+                binary.getTasks().add(installTask);
+            }
+        }
+
+        private Task createInstallTask(TaskContainer tasks, final NativeBinarySpecInternal binary, final NativeInstallationSpec installation, final NativeExecutableFileSpec executable, final File buildDirectory, final BinaryNamingScheme namingScheme) {
+            return tasks.create(namingScheme.getTaskName("install"), InstallExecutable.class, new Action<InstallExecutable>() {
+                @Override
+                public void execute(InstallExecutable installTask) {
+                    installTask.setDescription("Installs a development image of " + binary.getDisplayName());
+                    installTask.setGroup(LifecycleBasePlugin.BUILD_GROUP);
+                    installTask.setToolChain(executable.getToolChain());
+                    installTask.setExecutable(executable.getFile());
+                    installTask.setDestinationDir(installation.getDirectory());
+                    //TODO:HH wire binary libs via executable
+                    installTask.lib(new BinaryLibs(binary) {
+                        @Override
+                        protected FileCollection getFiles(NativeDependencySet nativeDependencySet) {
+                            return nativeDependencySet.getRuntimeFiles();
+                        }
+                    });
+
+                    //TODO:HH installTask.dependsOn(executable)
+                    installTask.dependsOn(binary);
+                }
+            });
+        }
+    }
+
+    private abstract static class BinaryLibs implements Callable<List<FileCollection>> {
+        private final NativeBinarySpec binary;
+
+        public BinaryLibs(NativeBinarySpec binary) {
+            this.binary = binary;
+        }
+
+        @Override
+        public List<FileCollection> call() throws Exception {
+            List<FileCollection> runtimeFiles = Lists.newArrayList();
+            for (NativeDependencySet nativeDependencySet : binary.getLibs()) {
+                runtimeFiles.add(getFiles(nativeDependencySet));
+            }
+            return runtimeFiles;
+        }
+
+        protected abstract FileCollection getFiles(NativeDependencySet nativeDependencySet);
     }
 
     private static class DefaultRepositories extends DefaultPolymorphicDomainObjectContainer<ArtifactRepository> implements Repositories {
