@@ -18,28 +18,23 @@ package org.gradle.play.internal;
 
 import org.gradle.api.file.FileCollection;
 import org.gradle.platform.base.internal.BinarySpecInternal;
-import org.gradle.platform.base.internal.toolchain.ToolResolver;
 import org.gradle.play.PlayApplicationBinarySpec;
-import org.gradle.play.PlayApplicationSpec;
+import org.gradle.play.internal.toolchain.PlayToolChainInternal;
 import org.gradle.play.platform.PlayPlatform;
 
 import java.io.File;
 
 public interface PlayApplicationBinarySpecInternal extends PlayApplicationBinarySpec, BinarySpecInternal {
-    void setApplication(PlayApplicationSpec application);
-
     void setTargetPlatform(PlayPlatform platform);
 
-    void setToolResolver(ToolResolver toolResolver);
+    void setToolChain(PlayToolChainInternal toolChain);
 
-    ToolResolver getToolResolver();
+    PlayToolChainInternal getToolChain();
 
     void setJarFile(File file);
 
     void setAssetsJarFile(File file);
 
-    // TODO:DAZ Should be taken from the LanguageSourceSet instances?
-    // TODO:DAZ Should be a Classpath instance
     FileCollection getClasspath();
 
     void setClasspath(FileCollection applicationClasspath);
