@@ -15,8 +15,8 @@
  */
 package org.gradle.play.plugins;
 
-import org.apache.commons.lang.StringUtils;
 import org.gradle.api.*;
+import org.gradle.api.artifacts.Dependency;
 import org.gradle.api.internal.artifacts.publish.DefaultPublishArtifact;
 import org.gradle.api.internal.file.FileResolver;
 import org.gradle.api.internal.file.copy.CopySpecInternal;
@@ -26,15 +26,11 @@ import org.gradle.api.tasks.scala.IncrementalCompileOptions;
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.jvm.tasks.Jar;
-import org.gradle.language.base.FunctionalSourceSet;
 import org.gradle.language.base.LanguageSourceSet;
-import org.gradle.language.base.internal.compile.Compiler;
 import org.gradle.language.base.sources.BaseLanguageSourceSet;
 import org.gradle.language.java.JavaSourceSet;
-import org.gradle.language.java.internal.DefaultJavaLanguageSourceSet;
 import org.gradle.language.java.plugins.JavaLanguagePlugin;
 import org.gradle.language.jvm.JvmResourceSet;
-import org.gradle.language.jvm.internal.DefaultJvmResourceLanguageSourceSet;
 import org.gradle.language.routes.RoutesSourceSet;
 import org.gradle.language.routes.internal.DefaultRoutesSourceSet;
 import org.gradle.language.scala.ScalaLanguageSourceSet;
@@ -44,28 +40,29 @@ import org.gradle.language.scala.tasks.PlatformScalaCompile;
 import org.gradle.language.twirl.TwirlSourceSet;
 import org.gradle.language.twirl.internal.DefaultTwirlSourceSet;
 import org.gradle.model.*;
+import org.gradle.model.internal.core.Service;
+import org.gradle.model.internal.registry.ModelRegistry;
+import org.gradle.model.internal.type.ModelType;
 import org.gradle.platform.base.*;
-import org.gradle.platform.base.internal.ComponentSpecInternal;
 import org.gradle.platform.base.internal.DefaultPlatformRequirement;
 import org.gradle.platform.base.internal.PlatformRequirement;
 import org.gradle.platform.base.internal.PlatformResolvers;
-import org.gradle.platform.base.internal.toolchain.ResolvedTool;
-import org.gradle.platform.base.internal.toolchain.ToolResolver;
 import org.gradle.play.JvmClasses;
 import org.gradle.play.PlayApplicationBinarySpec;
 import org.gradle.play.PlayApplicationSpec;
+import org.gradle.play.PlayPlatformAwareComponentSpec;
 import org.gradle.play.PublicAssets;
 import org.gradle.play.internal.*;
 import org.gradle.play.internal.platform.PlayPlatformInternal;
-import org.gradle.play.internal.routes.RoutesCompileSpec;
-import org.gradle.play.internal.run.PlayApplicationRunner;
-import org.gradle.play.internal.twirl.TwirlCompileSpec;
-import org.gradle.play.internal.twirl.TwirlCompilerFactory;
+import org.gradle.play.internal.run.PlayApplicationRunnerFactory;
+import org.gradle.play.internal.toolchain.PlayToolChainInternal;
 import org.gradle.play.platform.PlayPlatform;
 import org.gradle.play.tasks.PlayRun;
 import org.gradle.play.tasks.RoutesCompile;
 import org.gradle.play.tasks.TwirlCompile;
+import org.gradle.util.VersionNumber;
 
+import javax.inject.Inject;
 import java.io.File;
 import java.util.ArrayList;
 import java.util.Date;
@@ -73,17 +70,24 @@ import java.util.Date;
 /**
  * Plugin for Play Framework component support. Registers the {@link org.gradle.play.PlayApplicationSpec} component type for the components container.
  */
-
 @Incubating
 public class PlayApplicationPlugin implements Plugin<Project> {
-    private final static String DEFAULT_PLAY_VERSION = "2.3.7";
     public static final int DEFAULT_HTTP_PORT = 9000;
+    public static final String RUN_GROUP = "Run";
+    private final ModelRegistry modelRegistry;
+
+    @Inject
+    public PlayApplicationPlugin(ModelRegistry modelRegistry) {
+        this.modelRegistry = modelRegistry;
+    }
 
     @Override
     public void apply(Project project) {
         project.getPluginManager().apply(JavaLanguagePlugin.class);
         project.getPluginManager().apply(ScalaLanguagePlugin.class);
         project.getExtensions().create("playConfigurations", PlayPluginConfigurations.class, project.getConfigurations(), project.getDependencies());
+
+        modelRegistry.getRoot().applyToAllLinksTransitive(ModelType.of(PlayApplicationSpec.class), PlaySourceSetRules.class);
     }
 
     @SuppressWarnings("UnusedDeclaration")
@@ -94,13 +98,24 @@ public class PlayApplicationPlugin implements Plugin<Project> {
         }
 
         @Model
+        PlayToolChainInternal playToolChain(ServiceRegistry serviceRegistry) {
+            return serviceRegistry.get(PlayToolChainInternal.class);
+        }
+
+        @Service
         FileResolver fileResolver(ServiceRegistry serviceRegistry) {
             return serviceRegistry.get(FileResolver.class);
         }
+        
+        @ComponentType
+        void registerPlayPlatformAwareComponentSpecType(ComponentTypeBuilder<PlayPlatformAwareComponentSpec> builder) {
+            builder.defaultImplementation(DefaultPlayPlatformAwareComponentSpec.class);
+            builder.internalView(PlayPlatformAwareComponentSpecInternal.class);
+        }
 
         @ComponentType
-        void register(ComponentTypeBuilder<PlayApplicationSpec> builder) {
-            builder.defaultImplementation(DefaultPlayApplicationSpec.class);
+        void registerPlayApplicationSpecType(ComponentTypeBuilder<PlayApplicationSpec> builder) {
+            builder.internalView(PlayApplicationSpecInternal.class);
         }
 
         @Mutate
@@ -116,6 +131,7 @@ public class PlayApplicationPlugin implements Plugin<Project> {
         @BinaryType
         void registerApplication(BinaryTypeBuilder<PlayApplicationBinarySpec> builder) {
             builder.defaultImplementation(DefaultPlayApplicationBinarySpec.class);
+            builder.internalView(PlayApplicationBinarySpecInternal.class);
         }
 
         @LanguageType
@@ -130,34 +146,6 @@ public class PlayApplicationPlugin implements Plugin<Project> {
             builder.defaultImplementation(DefaultRoutesSourceSet.class);
         }
 
-        @Mutate
-        void configureDefaultPlaySources(ModelMap<PlayApplicationSpec> playApplicationComponents, ServiceRegistry serviceRegistry) {
-            final FileResolver fileResolver = serviceRegistry.get(FileResolver.class);
-            final Instantiator instantiator = serviceRegistry.get(Instantiator.class);
-            playApplicationComponents.all(new Action<PlayApplicationSpec>() {
-                public void execute(PlayApplicationSpec playComponent) {
-                    // TODO:DAZ Scala source set type should be registered via scala-lang plugin
-                    ScalaLanguageSourceSet scalaSources = BaseLanguageSourceSet.create(DefaultScalaLanguageSourceSet.class, "scala", playComponent.getName(), fileResolver, instantiator);
-
-                    // Compile scala/java sources under /app\
-                    // TODO:DAZ Should be selecting 'controllers/**' and 'model/**' I think, allowing user to add more includes
-                    scalaSources.getSource().srcDir("app");
-                    scalaSources.getSource().include("**/*.scala");
-                    FunctionalSourceSet sources = ((ComponentSpecInternal) playComponent).getSources();
-                    sources.add(scalaSources);
-
-                    JavaSourceSet javaSources = BaseLanguageSourceSet.create(DefaultJavaLanguageSourceSet.class, "java", playComponent.getName(), fileResolver, instantiator);
-                    javaSources.getSource().srcDir("app");
-                    javaSources.getSource().include("**/*.java");
-                    sources.add(javaSources);
-
-                    JvmResourceSet appResources = BaseLanguageSourceSet.create(DefaultJvmResourceLanguageSourceSet.class, "resources", playComponent.getName(), fileResolver, instantiator);
-                    appResources.getSource().srcDirs("conf");
-                    sources.add(appResources);
-                }
-            });
-        }
-
         @Validate
         void failOnMultiplePlayComponents(ModelMap<PlayApplicationSpec> container) {
             if (container.size() >= 2) {
@@ -167,37 +155,47 @@ public class PlayApplicationPlugin implements Plugin<Project> {
 
         @Validate
         void failOnMultipleTargetPlatforms(ModelMap<PlayApplicationSpec> playApplications) {
-            playApplications.afterEach(new Action<PlayApplicationSpec>() {
-                public void execute(PlayApplicationSpec playApplication) {
-                    PlayApplicationSpecInternal playApplicationInternal = (PlayApplicationSpecInternal) playApplication;
-                    if (playApplicationInternal.getTargetPlatforms().size() > 1) {
+            playApplications.withType(PlayPlatformAwareComponentSpecInternal.class).afterEach(new Action<PlayPlatformAwareComponentSpecInternal>() {
+                public void execute(PlayPlatformAwareComponentSpecInternal playApplication) {
+                    if (playApplication.getTargetPlatforms().size() > 1) {
                         throw new GradleException("Multiple target platforms for 'PlayApplicationSpec' is not (yet) supported.");
                     }
                 }
             });
         }
 
+        @Validate
+        void failIfInjectedRouterIsUsedWithOldVersion(ModelMap<PlayApplicationBinarySpec> playApplicationBinaries) {
+            playApplicationBinaries.afterEach(new Action<PlayApplicationBinarySpec>() {
+                @Override
+                public void execute(PlayApplicationBinarySpec playApplicationBinary) {
+                    if (playApplicationBinary.getApplication().getInjectedRoutesGenerator()) {
+                        final PlayPlatform playPlatform = playApplicationBinary.getTargetPlatform();
+                        VersionNumber minSupportedVersion = VersionNumber.parse("2.4.0");
+                        VersionNumber playVersion = VersionNumber.parse(playPlatform.getPlayVersion());
+                        if (playVersion.compareTo(minSupportedVersion) < 0) {
+                            throw new GradleException("Injected routers are only supported in Play 2.4 or newer.");
+                        }
+                    }
+                }
+            });
+        }
+
         @ComponentBinaries
-        void createBinaries(ModelMap<PlayApplicationBinarySpec> binaries, final PlayApplicationSpec componentSpec,
-                            final PlatformResolvers platforms, final PlayPluginConfigurations configurations, final ServiceRegistry serviceRegistry,
+        void createBinaries(ModelMap<PlayApplicationBinarySpec> binaries, final PlayApplicationSpecInternal componentSpec,
+                            final PlatformResolvers platforms, final PlayToolChainInternal playToolChainInternal, final PlayPluginConfigurations configurations,
                             @Path("buildDir") final File buildDir, final ProjectIdentifier projectIdentifier) {
 
-            final FileResolver fileResolver = serviceRegistry.get(FileResolver.class);
-            final Instantiator instantiator = serviceRegistry.get(Instantiator.class);
-            final ToolResolver toolResolver = serviceRegistry.get(ToolResolver.class);
-            final String binaryName = String.format("%sBinary", componentSpec.getName());
-
-            binaries.create(binaryName, new Action<PlayApplicationBinarySpec>() {
+            binaries.create("binary", new Action<PlayApplicationBinarySpec>() {
                 public void execute(PlayApplicationBinarySpec playBinary) {
                     PlayApplicationBinarySpecInternal playBinaryInternal = (PlayApplicationBinarySpecInternal) playBinary;
-                    playBinaryInternal.setApplication(componentSpec);
-                    final File binaryBuildDir = new File(buildDir, binaryName);
+                    final File binaryBuildDir = new File(buildDir, playBinaryInternal.getProjectScopedName());
 
-                    final PlayPlatform chosenPlatform = resolveTargetPlatform(componentSpec, platforms, configurations);
+                    final PlayPlatform chosenPlatform = resolveTargetPlatform(componentSpec, platforms);
                     initialiseConfigurations(configurations, chosenPlatform);
 
                     playBinaryInternal.setTargetPlatform(chosenPlatform);
-                    playBinaryInternal.setToolResolver(toolResolver);
+                    playBinaryInternal.setToolChain(playToolChainInternal);
 
                     File mainJar = new File(binaryBuildDir, String.format("lib/%s.jar", projectIdentifier.getName()));
                     File assetsJar = new File(binaryBuildDir, String.format("lib/%s-assets.jar", projectIdentifier.getName()));
@@ -210,30 +208,29 @@ public class PlayApplicationPlugin implements Plugin<Project> {
                     JvmClasses classes = playBinary.getClasses();
                     classes.setClassesDir(new File(binaryBuildDir, "classes"));
 
-                    ModelMap<JvmResourceSet> jvmResourceSets = componentSpec.getSource().withType(JvmResourceSet.class);
+                    ModelMap<JvmResourceSet> jvmResourceSets = componentSpec.getSources().withType(JvmResourceSet.class);
                     for (JvmResourceSet jvmResourceSet : jvmResourceSets.values()) {
-                        for (File resourceDir : jvmResourceSet.getSource()) {
+                        for (File resourceDir : jvmResourceSet.getSource().getSrcDirs()) {
                             classes.addResourceDir(resourceDir);
                         }
                     }
 
-                    // TODO:DAZ These should be configured on the component
                     PublicAssets assets = playBinary.getAssets();
                     assets.addAssetDir(new File(projectIdentifier.getProjectDir(), "public"));
 
-                    playBinaryInternal.setClasspath(configurations.getPlay().getFileCollection());
+                    playBinaryInternal.setClasspath(configurations.getPlay().getAllArtifacts());
                 }
             });
         }
 
-        private PlayPlatform resolveTargetPlatform(PlayApplicationSpec componentSpec, final PlatformResolvers platforms, PlayPluginConfigurations configurations) {
-            PlatformRequirement targetPlatform = getTargetPlatform((PlayApplicationSpecInternal) componentSpec);
+        private PlayPlatform resolveTargetPlatform(PlayApplicationSpecInternal componentSpec, final PlatformResolvers platforms) {
+            PlatformRequirement targetPlatform = getTargetPlatform(componentSpec);
             return platforms.resolve(PlayPlatform.class, targetPlatform);
         }
 
         private PlatformRequirement getTargetPlatform(PlayApplicationSpecInternal playApplicationSpec) {
             if (playApplicationSpec.getTargetPlatforms().isEmpty()) {
-                String defaultPlayPlatform = String.format("play-%s", DEFAULT_PLAY_VERSION);
+                String defaultPlayPlatform = String.format("play-%s", DefaultPlayPlatform.DEFAULT_PLAY_VERSION);
                 return DefaultPlatformRequirement.create(defaultPlayPlatform);
             }
             return playApplicationSpec.getTargetPlatforms().get(0);
@@ -243,31 +240,17 @@ public class PlayApplicationPlugin implements Plugin<Project> {
             configurations.getPlayPlatform().addDependency(((PlayPlatformInternal) playPlatform).getDependencyNotation("play"));
             configurations.getPlayTest().addDependency(((PlayPlatformInternal) playPlatform).getDependencyNotation("play-test"));
             configurations.getPlayRun().addDependency(((PlayPlatformInternal) playPlatform).getDependencyNotation("play-docs"));
+
+            addRunSupportDependencies(configurations, playPlatform);
         }
 
-        @Mutate
-        void createTwirlSourceSets(ModelMap<PlayApplicationSpec> components) {
-            components.beforeEach(new Action<PlayApplicationSpec>() {
-                @Override
-                public void execute(PlayApplicationSpec playComponent) {
-                    TwirlSourceSet twirlSourceSet = ((ComponentSpecInternal) playComponent).getSources().create("twirlTemplates", TwirlSourceSet.class);
-                    twirlSourceSet.getSource().srcDir("app");
-                    twirlSourceSet.getSource().include("**/*.html");
-                }
-            });
-        }
-
-        @Mutate
-        void createRoutesSourceSets(ModelMap<PlayApplicationSpec> components) {
-            components.beforeEach(new Action<PlayApplicationSpec>() {
-                @Override
-                public void execute(PlayApplicationSpec playComponent) {
-                    RoutesSourceSet routesSourceSet = ((ComponentSpecInternal) playComponent).getSources().create("routesSources", RoutesSourceSet.class);
-                    routesSourceSet.getSource().srcDir("conf");
-                    routesSourceSet.getSource().include("routes");
-                    routesSourceSet.getSource().include("*.routes");
-                }
-            });
+        private void addRunSupportDependencies(PlayPluginConfigurations configurations, PlayPlatform playPlatform) {
+            String playVersion = playPlatform.getPlayVersion();
+            String scalaCompatibilityVersion = playPlatform.getScalaPlatform().getScalaCompatibilityVersion();
+            Iterable<Dependency> runSupportDependencies = PlayApplicationRunnerFactory.createPlayRunAdapter(playPlatform).getRunsupportClasspathDependencies(playVersion, scalaCompatibilityVersion);
+            for (Dependency dependencyNotation : runSupportDependencies) {
+                configurations.getPlayRun().addDependency(dependencyNotation);
+            }
         }
 
         @Mutate
@@ -276,14 +259,15 @@ public class PlayApplicationPlugin implements Plugin<Project> {
             createGeneratedScalaSourceSetsForType(RoutesSourceSet.class, binaries, serviceRegistry);
         }
 
-        void createGeneratedScalaSourceSetsForType(final Class<? extends LanguageSourceSet> languageSourceSetType, ModelMap<PlayApplicationBinarySpec> binaries, ServiceRegistry serviceRegistry) {
+        private void createGeneratedScalaSourceSetsForType(final Class<? extends LanguageSourceSet> languageSourceSetType, ModelMap<PlayApplicationBinarySpec> binaries, ServiceRegistry serviceRegistry) {
             final FileResolver fileResolver = serviceRegistry.get(FileResolver.class);
             final Instantiator instantiator = serviceRegistry.get(Instantiator.class);
             binaries.all(new Action<PlayApplicationBinarySpec>() {
                 @Override
                 public void execute(PlayApplicationBinarySpec playApplicationBinarySpec) {
-                    for (LanguageSourceSet languageSourceSet : playApplicationBinarySpec.getSource().withType(languageSourceSetType)) {
-                        ScalaLanguageSourceSet twirlScalaSources = BaseLanguageSourceSet.create(DefaultScalaLanguageSourceSet.class, String.format("%sScalaSources", languageSourceSet.getName()), playApplicationBinarySpec.getName(), fileResolver, instantiator);
+                    for (LanguageSourceSet languageSourceSet : playApplicationBinarySpec.getInputs().withType(languageSourceSetType)) {
+                        String name = String.format("%sScalaSources", languageSourceSet.getName());
+                        ScalaLanguageSourceSet twirlScalaSources = BaseLanguageSourceSet.create(ScalaLanguageSourceSet.class, DefaultScalaLanguageSourceSet.class, name, playApplicationBinarySpec.getName(), fileResolver);
                         playApplicationBinarySpec.getGeneratedScala().put(languageSourceSet, twirlScalaSources);
                     }
                 }
@@ -291,19 +275,17 @@ public class PlayApplicationPlugin implements Plugin<Project> {
         }
 
         @BinaryTasks
-        void createTwirlCompileTasks(ModelMap<Task> tasks, final PlayApplicationBinarySpec binary, ServiceRegistry serviceRegistry, @Path("buildDir") final File buildDir) {
-            final ToolResolver toolResolver = serviceRegistry.get(ToolResolver.class);
-            final ResolvedTool<Compiler<TwirlCompileSpec>> compilerTool = toolResolver.resolveCompiler(TwirlCompileSpec.class, binary.getTargetPlatform());
-            for (final TwirlSourceSet twirlSourceSet : binary.getSource().withType(TwirlSourceSet.class)) {
-                final String twirlCompileTaskName = String.format("twirlCompile%s%s", StringUtils.capitalize(twirlSourceSet.getName()), StringUtils.capitalize(binary.getName()));
+        void createTwirlCompileTasks(ModelMap<Task> tasks, final PlayApplicationBinarySpecInternal binary, ServiceRegistry serviceRegistry, @Path("buildDir") final File buildDir) {
+            for (final TwirlSourceSet twirlSourceSet : binary.getInputs().withType(TwirlSourceSet.class)) {
+                final String twirlCompileTaskName = binary.getTasks().taskName("compile", twirlSourceSet.getName());
                 final File twirlCompileOutputDirectory = srcOutputDirectory(buildDir, binary, twirlCompileTaskName);
 
                 tasks.create(twirlCompileTaskName, TwirlCompile.class, new Action<TwirlCompile>() {
                     public void execute(TwirlCompile twirlCompile) {
-                        twirlCompile.setDependencyNotation(TwirlCompilerFactory.createAdapter(binary.getTargetPlatform()).getDependencyNotation());
+                        twirlCompile.setDescription("Compiles twirl templates for the '" + twirlSourceSet.getName() + "' source set.");
+                        twirlCompile.setPlatform(binary.getTargetPlatform());
                         twirlCompile.setSource(twirlSourceSet.getSource());
                         twirlCompile.setOutputDirectory(twirlCompileOutputDirectory);
-                        twirlCompile.setCompilerTool(compilerTool);
 
                         ScalaLanguageSourceSet twirlScalaSources = binary.getGeneratedScala().get(twirlSourceSet);
                         twirlScalaSources.getSource().srcDir(twirlCompileOutputDirectory);
@@ -314,19 +296,19 @@ public class PlayApplicationPlugin implements Plugin<Project> {
         }
 
         @BinaryTasks
-        void createRoutesCompileTasks(ModelMap<Task> tasks, final PlayApplicationBinarySpec binary, ServiceRegistry serviceRegistry, @Path("buildDir") final File buildDir) {
-            final ToolResolver toolResolver = serviceRegistry.get(ToolResolver.class);
-            final ResolvedTool<Compiler<RoutesCompileSpec>> compilerTool = toolResolver.resolveCompiler(RoutesCompileSpec.class, binary.getTargetPlatform());
-            for (final RoutesSourceSet routesSourceSet : binary.getSource().withType(RoutesSourceSet.class)) {
-                final String routesCompileTaskName = String.format("routesCompile%s%s", StringUtils.capitalize(routesSourceSet.getName()), StringUtils.capitalize(binary.getName()));
+        void createRoutesCompileTasks(ModelMap<Task> tasks, final PlayApplicationBinarySpecInternal binary, ServiceRegistry serviceRegistry, @Path("buildDir") final File buildDir) {
+            for (final RoutesSourceSet routesSourceSet : binary.getInputs().withType(RoutesSourceSet.class)) {
+                final String routesCompileTaskName = binary.getTasks().taskName("compile", routesSourceSet.getName());
                 final File routesCompilerOutputDirectory = srcOutputDirectory(buildDir, binary, routesCompileTaskName);
 
                 tasks.create(routesCompileTaskName, RoutesCompile.class, new Action<RoutesCompile>() {
                     public void execute(RoutesCompile routesCompile) {
-                        routesCompile.setCompilerTool(compilerTool);
+                        routesCompile.setDescription("Generates routes for the '" + routesSourceSet.getName() + "' source set.");
+                        routesCompile.setPlatform(binary.getTargetPlatform());
                         routesCompile.setAdditionalImports(new ArrayList<String>());
                         routesCompile.setSource(routesSourceSet.getSource());
                         routesCompile.setOutputDirectory(routesCompilerOutputDirectory);
+                        routesCompile.setInjectedRoutesGenerator(binary.getApplication().getInjectedRoutesGenerator());
 
                         ScalaLanguageSourceSet routesScalaSources = binary.getGeneratedScala().get(routesSourceSet);
                         routesScalaSources.getSource().srcDir(routesCompilerOutputDirectory);
@@ -338,9 +320,10 @@ public class PlayApplicationPlugin implements Plugin<Project> {
 
         @BinaryTasks
         void createScalaCompileTask(ModelMap<Task> tasks, final PlayApplicationBinarySpec binary, @Path("buildDir") final File buildDir) {
-            final String scalaCompileTaskName = String.format("scalaCompile%s", StringUtils.capitalize(binary.getName()));
+            final String scalaCompileTaskName = binary.getTasks().taskName("compile", "Scala");
             tasks.create(scalaCompileTaskName, PlatformScalaCompile.class, new Action<PlatformScalaCompile>() {
                 public void execute(PlatformScalaCompile scalaCompile) {
+                    scalaCompile.setDescription("Compiles all scala and java source sets for the " + binary.getDisplayName() + ".");
 
                     scalaCompile.setDestinationDir(binary.getClasses().getClassesDir());
                     scalaCompile.setPlatform(binary.getTargetPlatform().getScalaPlatform());
@@ -352,19 +335,19 @@ public class PlayApplicationPlugin implements Plugin<Project> {
                     IncrementalCompileOptions incrementalOptions = scalaCompile.getScalaCompileOptions().getIncrementalOptions();
                     incrementalOptions.setAnalysisFile(new File(buildDir, String.format("tmp/scala/compilerAnalysis/%s.analysis", scalaCompileTaskName)));
 
-                    for (LanguageSourceSet appSources : binary.getSource().withType(ScalaLanguageSourceSet.class)) {
+                    for (LanguageSourceSet appSources : binary.getInputs().withType(ScalaLanguageSourceSet.class)) {
                         scalaCompile.source(appSources.getSource());
                         scalaCompile.dependsOn(appSources);
                     }
 
-                    for (LanguageSourceSet appSources : binary.getSource().withType(JavaSourceSet.class)) {
+                    for (LanguageSourceSet appSources : binary.getInputs().withType(JavaSourceSet.class)) {
                         scalaCompile.source(appSources.getSource());
                         scalaCompile.dependsOn(appSources);
                     }
 
                     for (LanguageSourceSet generatedSourceSet : binary.getGeneratedScala().values()) {
                         scalaCompile.source(generatedSourceSet.getSource());
-                        scalaCompile.dependsOn(generatedSourceSet.getBuildDependencies());
+                        scalaCompile.dependsOn(generatedSourceSet);
                     }
 
                     scalaCompile.setClasspath(((PlayApplicationBinarySpecInternal) binary).getClasspath());
@@ -376,9 +359,10 @@ public class PlayApplicationPlugin implements Plugin<Project> {
 
         @BinaryTasks
         void createJarTasks(ModelMap<Task> tasks, final PlayApplicationBinarySpec binary) {
-            String jarTaskName = String.format("create%sJar", StringUtils.capitalize(binary.getName()));
+            String jarTaskName = binary.getTasks().taskName("create", "Jar");
             tasks.create(jarTaskName, Jar.class, new Action<Jar>() {
                 public void execute(Jar jar) {
+                    jar.setDescription("Assembles the application jar for the " + binary.getDisplayName() + ".");
                     jar.setDestinationDir(binary.getJarFile().getParentFile());
                     jar.setArchiveName(binary.getJarFile().getName());
                     jar.from(binary.getClasses().getClassesDir());
@@ -387,9 +371,10 @@ public class PlayApplicationPlugin implements Plugin<Project> {
                 }
             });
 
-            String assetsJarTaskName = String.format("create%sAssetsJar", StringUtils.capitalize(binary.getName()));
+            String assetsJarTaskName = binary.getTasks().taskName("create", "assetsJar");
             tasks.create(assetsJarTaskName, Jar.class, new Action<Jar>() {
                 public void execute(Jar jar) {
+                    jar.setDescription("Assembles the assets jar for the " + binary.getDisplayName() + ".");
                     jar.setDestinationDir(binary.getAssetsJarFile().getParentFile());
                     jar.setArchiveName(binary.getAssetsJarFile().getName());
                     jar.setClassifier("assets");
@@ -401,28 +386,31 @@ public class PlayApplicationPlugin implements Plugin<Project> {
             });
         }
 
-        // TODO:DAZ Need a nice way to create tasks that are associated with a binary but not part of _building_ it.
         @Mutate
-        void createPlayRunTask(ModelMap<Task> tasks, BinaryContainer binaryContainer, ServiceRegistry serviceRegistry, final PlayPluginConfigurations configurations) {
-            ToolResolver toolResolver = serviceRegistry.get(ToolResolver.class);
-            for (final PlayApplicationBinarySpecInternal binary : binaryContainer.withType(PlayApplicationBinarySpecInternal.class)) {
-                final ResolvedTool<PlayApplicationRunner> playApplicationRunnerTool = toolResolver.resolve(PlayApplicationRunner.class, binary.getTargetPlatform());
-                String runTaskName = String.format("run%s", StringUtils.capitalize(binary.getName()));
+        void createPlayRunTask(ModelMap<Task> tasks, ModelMap<PlayApplicationBinarySpecInternal> playBinaries, final ServiceRegistry serviceRegistry, final PlayPluginConfigurations configurations, ProjectIdentifier projectIdentifier, final PlayToolChainInternal playToolChain) {
+
+            for (final PlayApplicationBinarySpecInternal binary : playBinaries) {
+                String runTaskName = binary.getTasks().taskName("run");
+
                 tasks.create(runTaskName, PlayRun.class, new Action<PlayRun>() {
                     public void execute(PlayRun playRun) {
+                        playRun.setDescription("Runs the Play application for local development.");
+                        playRun.setGroup(RUN_GROUP);
                         playRun.setHttpPort(DEFAULT_HTTP_PORT);
-                        playRun.setPlayApplicationRunnerTool(playApplicationRunnerTool);
+                        playRun.setPlayToolProvider(playToolChain.select(binary.getTargetPlatform()));
                         playRun.setApplicationJar(binary.getJarFile());
                         playRun.setAssetsJar(binary.getAssetsJarFile());
-                        playRun.setRuntimeClasspath(configurations.getPlayRun().getFileCollection());
+                        playRun.setAssetsDirs(binary.getAssets().getAssetDirs());
+                        playRun.setRuntimeClasspath(configurations.getPlayRun().getNonChangingArtifacts());
+                        playRun.setChangingClasspath(configurations.getPlayRun().getChangingArtifacts());
                         playRun.dependsOn(binary.getBuildTask());
                     }
                 });
             }
         }
 
-        private File srcOutputDirectory(File buildDir, PlayApplicationBinarySpec binary, String taskName) {
-            return new File(buildDir, String.format("%s/src/%s", binary.getName(), taskName));
+        private File srcOutputDirectory(File buildDir, PlayApplicationBinarySpecInternal binary, String taskName) {
+            return new File(buildDir, String.format("%s/src/%s", binary.getProjectScopedName(), taskName));
         }
     }
 }
