@@ -16,7 +16,6 @@
 
 package org.gradle.play.plugins;
 
-import org.apache.commons.lang.StringUtils;
 import org.gradle.api.Action;
 import org.gradle.api.Incubating;
 import org.gradle.api.Task;
@@ -33,7 +32,6 @@ import org.gradle.model.ModelMap;
 import org.gradle.model.Mutate;
 import org.gradle.model.Path;
 import org.gradle.model.RuleSource;
-import org.gradle.platform.base.BinaryContainer;
 import org.gradle.play.PlayApplicationBinarySpec;
 import org.gradle.play.internal.PlayApplicationBinarySpecInternal;
 import org.gradle.play.internal.toolchain.PlayToolProvider;
@@ -48,19 +46,19 @@ import java.util.Arrays;
 @Incubating
 public class PlayTestPlugin extends RuleSource {
     @Mutate
-    void createTestTasks(ModelMap<Task> tasks, BinaryContainer binaryContainer, final PlayPluginConfigurations configurations,
+    void createTestTasks(ModelMap<Task> tasks, ModelMap<PlayApplicationBinarySpecInternal> playBinaries, final PlayPluginConfigurations configurations,
                          final FileResolver fileResolver, final ProjectIdentifier projectIdentifier, @Path("buildDir") final File buildDir) {
-        for (final PlayApplicationBinarySpecInternal binary : binaryContainer.withType(PlayApplicationBinarySpecInternal.class)) {
+        for (final PlayApplicationBinarySpecInternal binary : playBinaries) {
             final PlayToolProvider playToolProvider = binary.getToolChain().select(binary.getTargetPlatform());
             final FileCollection testCompileClasspath = getTestCompileClasspath(binary, playToolProvider, configurations);
 
-            final String testCompileTaskName = String.format("compile%sTests", StringUtils.capitalize(binary.getName()));
+            final String testCompileTaskName = binary.getTasks().taskName("compile", "tests");
             final File testSourceDir = fileResolver.resolve("test");
             final FileCollection testSources = new SimpleFileCollection(testSourceDir).getAsFileTree().matching(new PatternSet().include("**/*.scala", "**/*.java"));
-            final File testClassesDir = new File(buildDir, String.format("%s/testClasses", binary.getName()));
+            final File testClassesDir = new File(buildDir, String.format("%s/testClasses", binary.getProjectScopedName()));
             tasks.create(testCompileTaskName, PlatformScalaCompile.class, new Action<PlatformScalaCompile>() {
                 public void execute(PlatformScalaCompile scalaCompile) {
-                    scalaCompile.setDescription("Compiles the scala and java test sources for the '" + binary.getName() + "' binary.");
+                    scalaCompile.setDescription("Compiles the scala and java test sources for the " + binary.getDisplayName() + ".");
 
                     scalaCompile.setClasspath(testCompileClasspath);
 
@@ -79,11 +77,11 @@ public class PlayTestPlugin extends RuleSource {
                 }
             });
 
-            final String testTaskName = String.format("test%s", StringUtils.capitalize(binary.getName()));
-            final File binaryBuildDir = new File(buildDir, binary.getName());
+            final String testTaskName = binary.getTasks().taskName("test");
+            final File binaryBuildDir = new File(buildDir, binary.getProjectScopedName());
             tasks.create(testTaskName, Test.class, new Action<Test>() {
                 public void execute(Test test) {
-                    test.setDescription("Runs tests for the '" + binary.getName() + "' binary.");
+                    test.setDescription("Runs tests for the " + binary.getDisplayName() + ".");
 
                     test.setClasspath(getRuntimeClasspath(testClassesDir, testCompileClasspath));
 
@@ -109,13 +107,13 @@ public class PlayTestPlugin extends RuleSource {
     }
 
     @Mutate
-    void attachTestSuitesToCheckTask(ModelMap<Task> tasks, final BinaryContainer binaries) {
+    void attachTestSuitesToCheckTask(ModelMap<Task> tasks, final ModelMap<PlayApplicationBinarySpec> playBinaries) {
         // TODO - binaries aren't an input to this rule, they're an input to the action
         tasks.named(LifecycleBasePlugin.CHECK_TASK_NAME, new Action<Task>() {
             @Override
             public void execute(Task checkTask) {
                 // TODO Need a better mechanism to wire tasks into lifecycle
-                for (PlayApplicationBinarySpec binary : binaries.withType(PlayApplicationBinarySpec.class)) {
+                for (PlayApplicationBinarySpec binary : playBinaries) {
                     checkTask.dependsOn(binary.getTasks().withType(Test.class));
                 }
             }
