@@ -15,12 +15,10 @@
  */
 package org.gradle.api.internal.jvm;
 
-import org.gradle.api.Action;
 import org.gradle.api.DomainObjectSet;
 import org.gradle.api.internal.AbstractBuildableModelElement;
 import org.gradle.api.internal.DefaultDomainObjectSet;
 import org.gradle.api.internal.project.taskfactory.ITaskFactory;
-import org.gradle.api.internal.rules.NamedDomainObjectFactoryRegistry;
 import org.gradle.api.tasks.SourceSet;
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.jvm.ClassDirectoryBinarySpec;
@@ -32,11 +30,7 @@ import org.gradle.jvm.toolchain.JavaToolChain;
 import org.gradle.language.base.LanguageSourceSet;
 import org.gradle.model.ModelMap;
 import org.gradle.platform.base.BinarySpec;
-import org.gradle.platform.base.BinaryTasksCollection;
-import org.gradle.platform.base.internal.BinaryBuildAbility;
-import org.gradle.platform.base.internal.DefaultBinaryTasksCollection;
-import org.gradle.platform.base.internal.FixedBuildAbility;
-import org.gradle.platform.base.internal.ToolSearchBuildAbility;
+import org.gradle.platform.base.internal.*;
 
 import java.io.File;
 
@@ -65,6 +59,11 @@ public class DefaultClassDirectoryBinarySpec extends AbstractBuildableModelEleme
         return name;
     }
 
+    @Override
+    public String getProjectScopedName() {
+        return getName();
+    }
+
     @Override
     public Class<? extends BinarySpec> getPublicType() {
         return ClassDirectoryBinarySpec.class;
@@ -74,11 +73,6 @@ public class DefaultClassDirectoryBinarySpec extends AbstractBuildableModelEleme
         return tasks;
     }
 
-    @Override
-    public void tasks(Action<? super BinaryTasksCollection> action) {
-        action.execute(tasks);
-    }
-
     public JavaToolChain getToolChain() {
         return toolChain;
     }
@@ -127,11 +121,6 @@ public class DefaultClassDirectoryBinarySpec extends AbstractBuildableModelEleme
         sourceSet.getOutput().setResourcesDir(resourcesDir);
     }
 
-    @Override
-    public void sources(Action<? super ModelMap<LanguageSourceSet>> action) {
-        throw new UnsupportedOperationException();
-    }
-
     @Override
     public DomainObjectSet<LanguageSourceSet> getSource() {
         return getInputs();
@@ -142,6 +131,16 @@ public class DefaultClassDirectoryBinarySpec extends AbstractBuildableModelEleme
         throw new UnsupportedOperationException();
     }
 
+    @Override
+    public BinaryNamingScheme getNamingScheme() {
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
+    public void setNamingScheme(BinaryNamingScheme namingScheme) {
+        throw new UnsupportedOperationException();
+    }
+
     @Override
     public DomainObjectSet<LanguageSourceSet> getInputs() {
         return sourceSets;
@@ -152,11 +151,6 @@ public class DefaultClassDirectoryBinarySpec extends AbstractBuildableModelEleme
         sourceSets.add(sourceSet);
     }
 
-    @Override
-    public NamedDomainObjectFactoryRegistry<LanguageSourceSet> getEntityInstantiator() {
-        throw new UnsupportedOperationException();
-    }
-
     public String getDisplayName() {
         return "classes '" + removeClassesSuffix(name) + "'";
     }
