@@ -20,11 +20,11 @@ import groovy.lang.Closure;
 import org.gradle.api.Action;
 import org.gradle.api.GradleException;
 import org.gradle.api.Incubating;
-import org.gradle.api.file.FileCollection;
-import org.gradle.api.file.FileTree;
-import org.gradle.api.file.FileTreeElement;
+import org.gradle.api.file.*;
 import org.gradle.api.internal.ConventionTask;
 import org.gradle.api.internal.file.FileResolver;
+import org.gradle.api.internal.file.FileTreeElementComparator;
+import org.gradle.api.internal.file.FileTreeElementHasher;
 import org.gradle.api.internal.initialization.loadercache.ClassLoaderCache;
 import org.gradle.api.internal.tasks.options.Option;
 import org.gradle.api.internal.tasks.testing.DefaultTestTaskReports;
@@ -71,6 +71,7 @@ import org.gradle.util.ConfigureUtil;
 import javax.inject.Inject;
 import java.io.File;
 import java.util.*;
+import java.util.concurrent.Callable;
 
 /**
  * Executes JUnit (3.8.x or 4.x) or TestNG tests. Test are always run in (one or more) separate JVMs.
@@ -159,6 +160,8 @@ public class Test extends ConventionTask implements JavaForkOptions, PatternFilt
         reports.getHtml().setEnabled(true);
 
         filter = instantiator.newInstance(DefaultTestFilter.class);
+
+        addCandidateClassFilesHashProperty();
     }
 
     @Inject
@@ -495,7 +498,7 @@ public class Test extends ConventionTask implements JavaForkOptions, PatternFilt
         TestEventLogger eventLogger = new TestEventLogger(getTextOutputFactory(), currentLevel, levelLogging, exceptionFormatter);
         addTestListener(eventLogger);
         addTestOutputListener(eventLogger);
-        if (!getFilter().getIncludePatterns().isEmpty()) {
+        if (getFilter().isFailOnNoMatchingTests() && !getFilter().getIncludePatterns().isEmpty()) {
             addTestListener(new NoMatchingTestsReporter("No tests found for given includes: " + getFilter().getIncludePatterns()));
         }
 
@@ -1013,11 +1016,36 @@ public class Test extends ConventionTask implements JavaForkOptions, PatternFilt
      * @return The candidate class files.
      */
     @InputFiles
-    @Input
     public FileTree getCandidateClassFiles() {
         return getProject().fileTree(getTestClassesDir()).matching(patternSet);
     }
 
+    private void addCandidateClassFilesHashProperty() {
+        // force tests to run when the set of candidate class files changes
+        getInputs().property("candidateClassFilesHash", new Callable<Integer>() {
+            Integer candidateClassFilesHash;
+
+            @Override
+            public Integer call() throws Exception {
+                if (candidateClassFilesHash == null) {
+                    candidateClassFilesHash = calculateCandidateClassFilesHash();
+                }
+                return candidateClassFilesHash;
+            }
+        });
+    }
+
+    private Integer calculateCandidateClassFilesHash() {
+        final SortedSet<FileTreeElement> sortedFiles = new TreeSet<FileTreeElement>(FileTreeElementComparator.INSTANCE);
+        getCandidateClassFiles().visit(new EmptyFileVisitor() {
+            @Override
+            public void visitFile(FileVisitDetails fileDetails) {
+                sortedFiles.add(fileDetails);
+            }
+        });
+        return FileTreeElementHasher.calculateHashForFilePaths(sortedFiles);
+    }
+
     /**
      * Allows to set options related to which test events are logged to the console, and on which detail level. For example, to show more information about exceptions use:
      *
