@@ -16,18 +16,16 @@
 
 package org.gradle.testkit.runner.internal;
 
-import org.gradle.internal.SystemProperties;
-
 import java.io.File;
 
 public class TempTestKitDirProvider extends ConstantTestKitDirProvider {
 
     public TempTestKitDirProvider() {
-        this(new File(SystemProperties.getInstance().getJavaIoTmpDir()));
+        this(new File(System.getProperty("java.io.tmpdir")));
     }
 
     TempTestKitDirProvider(File rootDir) {
-        super(new File(rootDir, String.format(".gradle-test-kit-%s", SystemProperties.getInstance().getUserName())));
+        super(new File(rootDir, String.format(".gradle-test-kit-%s", System.getProperty("user.name"))));
     }
 
 }
