@@ -23,35 +23,27 @@ import org.gradle.api.Project;
 import org.gradle.api.internal.project.taskfactory.ITaskFactory;
 import org.gradle.api.tasks.TaskContainer;
 import org.gradle.internal.service.ServiceRegistry;
-import org.gradle.language.base.FunctionalSourceSet;
 import org.gradle.language.base.internal.registry.LanguageTransformContainer;
 import org.gradle.language.c.CSourceSet;
 import org.gradle.language.c.plugins.CLangPlugin;
 import org.gradle.model.*;
-import org.gradle.nativeplatform.NativeBinarySpec;
 import org.gradle.nativeplatform.NativeComponentSpec;
-import org.gradle.nativeplatform.SharedLibraryBinary;
-import org.gradle.nativeplatform.internal.NativeBinarySpecInternal;
-import org.gradle.nativeplatform.internal.configure.ToolSettingNativeBinaryInitializer;
-import org.gradle.nativeplatform.internal.resolve.NativeDependencyResolver;
 import org.gradle.nativeplatform.test.cunit.CUnitTestSuiteBinarySpec;
 import org.gradle.nativeplatform.test.cunit.CUnitTestSuiteSpec;
 import org.gradle.nativeplatform.test.cunit.internal.DefaultCUnitTestSuiteBinary;
 import org.gradle.nativeplatform.test.cunit.internal.DefaultCUnitTestSuiteSpec;
 import org.gradle.nativeplatform.test.cunit.tasks.GenerateCUnitLauncher;
 import org.gradle.nativeplatform.test.plugins.NativeBinariesTestPlugin;
-import org.gradle.nativeplatform.toolchain.internal.PlatformToolProvider;
 import org.gradle.platform.base.BinaryType;
 import org.gradle.platform.base.BinaryTypeBuilder;
 import org.gradle.platform.base.ComponentType;
 import org.gradle.platform.base.ComponentTypeBuilder;
-import org.gradle.platform.base.internal.BinaryNamingScheme;
-import org.gradle.platform.base.internal.ComponentSpecInternal;
-import org.gradle.platform.base.internal.DefaultBinaryNamingSchemeBuilder;
 import org.gradle.platform.base.test.TestSuiteContainer;
 
 import java.io.File;
 
+import static org.gradle.nativeplatform.test.internal.NativeTestSuites.createNativeTestSuiteBinaries;
+
 /**
  * A plugin that sets up the infrastructure for testing native binaries with CUnit.
  */
@@ -76,30 +68,37 @@ public class CUnitPlugin implements Plugin<Project> {
                 testSuites.create(suiteName, CUnitTestSuiteSpec.class, new Action<CUnitTestSuiteSpec>() {
                     @Override
                     public void execute(CUnitTestSuiteSpec testSuite) {
-                        DefaultCUnitTestSuiteSpec cunitTestSuite = (DefaultCUnitTestSuiteSpec) testSuite;
-                        cunitTestSuite.setTestedComponent(component);
+                        testSuite.setTestedComponent(component);
                     }
                 });
             }
         }
-
+        
         @ComponentType
-        public  void registerCUnitTestSuiteSpecType(ComponentTypeBuilder<CUnitTestSuiteSpec> builder) {
+        public void registerCUnitTestSuiteSpecType(ComponentTypeBuilder<CUnitTestSuiteSpec> builder) {
             builder.defaultImplementation(DefaultCUnitTestSuiteSpec.class);
         }
 
         @Finalize
-        public void configureCUnitTestSuiteSources(TestSuiteContainer testSuites, @Path("buildDir") File buildDir) {
+        public void configureCUnitTestSuiteSources(TestSuiteContainer testSuites, @Path("buildDir") final File buildDir) {
 
             for (final CUnitTestSuiteSpec suite : testSuites.withType(CUnitTestSuiteSpec.class).values()) {
-                FunctionalSourceSet suiteSourceSet = ((ComponentSpecInternal) suite).getFunctionalSourceSet();
-                CSourceSet launcherSources = suiteSourceSet.maybeCreate(CUNIT_LAUNCHER_SOURCE_SET, CSourceSet.class);
-                File baseDir = new File(buildDir, String.format("src/%s/cunitLauncher", suite.getName()));
-                launcherSources.getSource().srcDir(new File(baseDir, "c"));
-                launcherSources.getExportedHeaders().srcDir(new File(baseDir, "headers"));
+                suite.getSources().create(CUNIT_LAUNCHER_SOURCE_SET, CSourceSet.class, new Action<CSourceSet>() {
+                    @Override
+                    public void execute(CSourceSet launcherSources) {
+                        File baseDir = new File(buildDir, String.format("src/%s/cunitLauncher", suite.getName()));
+                        launcherSources.getSource().srcDir(new File(baseDir, "c"));
+                        launcherSources.getExportedHeaders().srcDir(new File(baseDir, "headers"));
+                    }
+                });
+
+                suite.getSources().withType(CSourceSet.class).named("c", new Action<CSourceSet>() {
+                    @Override
+                    public void execute(CSourceSet cSourceSet) {
+                        cSourceSet.lib(suite.getSources().get(CUNIT_LAUNCHER_SOURCE_SET));
+                    }
+                });
 
-                CSourceSet testSources = suiteSourceSet.maybeCreate("c", CSourceSet.class);
-                testSources.lib(launcherSources);
             }
         }
 
@@ -129,46 +128,9 @@ public class CUnitPlugin implements Plugin<Project> {
         @Mutate
         public void createCUnitTestBinaries(TestSuiteContainer testSuites, @Path("buildDir") final File buildDir,
                                             LanguageTransformContainer languageTransforms, final ServiceRegistry serviceRegistry, final ITaskFactory taskFactory) {
-            final Action<NativeBinarySpec> setToolsAction = new ToolSettingNativeBinaryInitializer(languageTransforms);
-
-            testSuites.withType(CUnitTestSuiteSpec.class).afterEach(new Action<CUnitTestSuiteSpec>() {
-                @Override
-                public void execute(final CUnitTestSuiteSpec cUnitTestSuite) {
-                    for (final NativeBinarySpec testedBinary : cUnitTestSuite.getTestedComponent().getBinaries().withType(NativeBinarySpec.class).values()) {
-
-                        if (testedBinary instanceof SharedLibraryBinary) {
-                            // TODO:DAZ For now, we only create test suites for static library variants
-                            continue;
-                        }
-
-                        final BinaryNamingScheme namingScheme = new DefaultBinaryNamingSchemeBuilder(((NativeBinarySpecInternal) testedBinary).getNamingScheme())
-                            .withComponentName(cUnitTestSuite.getBaseName())
-                            .withTypeString("CUnitExe").build();
-                        final NativeDependencyResolver resolver = serviceRegistry.get(NativeDependencyResolver.class);
-
-                        cUnitTestSuite.getBinaries().create(namingScheme.getLifecycleTaskName(), CUnitTestSuiteBinarySpec.class, new Action<CUnitTestSuiteBinarySpec>() {
-                            @Override
-                            public void execute(CUnitTestSuiteBinarySpec binary) {
-                                DefaultCUnitTestSuiteBinary testBinary = (DefaultCUnitTestSuiteBinary) binary;
-                                testBinary.setTestedBinary((NativeBinarySpecInternal) testedBinary);
-                                testBinary.setNamingScheme(namingScheme);
-                                testBinary.setResolver(resolver);
-                                setToolsAction.execute(testBinary);
-                                configure(testBinary, buildDir);
-                            }
-                        });
-                    }
-                }
-            });
+            createNativeTestSuiteBinaries(testSuites, CUnitTestSuiteSpec.class, CUnitTestSuiteBinarySpec.class, "CUnitExe", buildDir, serviceRegistry);
         }
 
-        private void configure(DefaultCUnitTestSuiteBinary testBinary, File buildDir) {
-            BinaryNamingScheme namingScheme = testBinary.getNamingScheme();
-            PlatformToolProvider toolProvider = testBinary.getPlatformToolProvider();
-            File binaryOutputDir = new File(new File(buildDir, "binaries"), namingScheme.getOutputDirectoryBase());
-            String baseName = testBinary.getComponent().getBaseName();
-
-            testBinary.setExecutableFile(new File(binaryOutputDir, toolProvider.getExecutableName(baseName)));
-        }
     }
+
 }
