@@ -16,19 +16,27 @@
 
 package org.gradle.tooling.internal.consumer.connection;
 
+import org.gradle.internal.concurrent.Stoppable;
+import org.gradle.tooling.BuildAction;
+import org.gradle.tooling.internal.consumer.TestExecutionRequest;
 import org.gradle.tooling.internal.consumer.parameters.ConsumerOperationParameters;
-import org.gradle.tooling.internal.consumer.versioning.VersionDetails;
 
 /**
- * by Szczepan Faber, created at: 12/22/11
+ * Implementations must be thread-safe.
  */
-public interface ConsumerConnection {
-
+public interface ConsumerConnection extends Stoppable {
+    /**
+     * Cleans up resources used by this connection. Blocks until complete.
+     */
     void stop();
-    
-    String getDisplayName();
-    
-    VersionDetails getVersionDetails();
 
-    <T> T run(Class<T> type, ConsumerOperationParameters operationParameters) throws UnsupportedOperationException, IllegalStateException;
+    String getDisplayName();
+
+    <T> T run(Class<T> type, ConsumerOperationParameters operationParameters)
+            throws UnsupportedOperationException, IllegalStateException;
+
+    <T> T run(BuildAction<T> action, ConsumerOperationParameters operationParameters)
+            throws UnsupportedOperationException, IllegalStateException;
+
+    void runTests(TestExecutionRequest testExecutionRequest, ConsumerOperationParameters operationParameters);
 }
