@@ -18,6 +18,7 @@ package org.gradle.tooling.internal.consumer.parameters;
 import com.google.common.collect.Lists;
 import org.gradle.api.GradleException;
 import org.gradle.initialization.BuildCancellationToken;
+import org.gradle.internal.classpath.ClassPath;
 import org.gradle.tooling.CancellationToken;
 import org.gradle.tooling.events.ProgressListener;
 import org.gradle.tooling.internal.adapter.ProtocolToModelAdapter;
@@ -31,7 +32,10 @@ import org.gradle.tooling.model.Task;
 import java.io.File;
 import java.io.InputStream;
 import java.io.OutputStream;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.LinkedHashSet;
+import java.util.List;
+import java.util.Set;
 import java.util.concurrent.TimeUnit;
 
 public class ConsumerOperationParameters implements BuildOperationParametersVersion1, BuildParametersVersion1, BuildParameters {
@@ -45,6 +49,7 @@ public class ConsumerOperationParameters implements BuildOperationParametersVers
         private final List<ProgressListener> testProgressListeners = new ArrayList<ProgressListener>();
         private final List<ProgressListener> taskProgressListeners = new ArrayList<ProgressListener>();
         private final List<ProgressListener> buildOperationProgressListeners = new ArrayList<ProgressListener>();
+        private String entryPoint;
         private CancellationToken cancellationToken;
         private ConnectionParameters parameters;
         private OutputStream stdout;
@@ -56,10 +61,16 @@ public class ConsumerOperationParameters implements BuildOperationParametersVers
         private List<String> arguments;
         private List<String> tasks;
         private List<InternalLaunchable> launchables;
+        private ClassPath injectedPluginClasspath = ClassPath.EMPTY;
 
         private Builder() {
         }
 
+        public Builder setEntryPoint(String entryPoint) {
+            this.entryPoint = entryPoint;
+            return this;
+        }
+
         public Builder setParameters(ConnectionParameters parameters) {
             this.parameters = parameters;
             return this;
@@ -91,13 +102,13 @@ public class ConsumerOperationParameters implements BuildOperationParametersVers
             return this;
         }
 
-        public Builder setJvmArguments(String... jvmArguments) {
-            this.jvmArguments = rationalizeInput(jvmArguments);
+        public Builder setJvmArguments(List<String> jvmArguments) {
+            this.jvmArguments = jvmArguments;
             return this;
         }
 
-        public Builder setArguments(String[] arguments) {
-            this.arguments = rationalizeInput(arguments);
+        public Builder setArguments(List<String> arguments) {
+            this.arguments = arguments;
             return this;
         }
 
@@ -130,6 +141,11 @@ public class ConsumerOperationParameters implements BuildOperationParametersVers
             return this;
         }
 
+        public Builder setInjectedPluginClasspath(ClassPath classPath) {
+            this.injectedPluginClasspath = classPath;
+            return this;
+        }
+
         public void addProgressListener(org.gradle.tooling.ProgressListener listener) {
             legacyProgressListeners.add(listener);
         }
@@ -151,17 +167,22 @@ public class ConsumerOperationParameters implements BuildOperationParametersVers
         }
 
         public ConsumerOperationParameters build() {
+            if (entryPoint == null) {
+                throw new IllegalStateException("No entry point specified.");
+            }
+
             // create the listener adapters right when the ConsumerOperationParameters are instantiated but no earlier,
             // this ensures that when multiple requests are issued that are built from the same builder, such requests do not share any state kept in the listener adapters
             // e.g. if the listener adapters do per-request caching, such caching must not leak between different requests built from the same builder
             ProgressListenerAdapter progressListenerAdapter = new ProgressListenerAdapter(this.legacyProgressListeners);
             FailsafeBuildProgressListenerAdapter buildProgressListenerAdapter = new FailsafeBuildProgressListenerAdapter(
                 new BuildProgressListenerAdapter(this.testProgressListeners, this.taskProgressListeners, this.buildOperationProgressListeners));
-            return new ConsumerOperationParameters(parameters, stdout, stderr, colorOutput, stdin, javaHome, jvmArguments, arguments, tasks, launchables,
+            return new ConsumerOperationParameters(entryPoint, parameters, stdout, stderr, colorOutput, stdin, javaHome, jvmArguments, arguments, tasks, launchables, injectedPluginClasspath,
                 progressListenerAdapter, buildProgressListenerAdapter, cancellationToken);
         }
     }
 
+    private final String entryPointName;
     private final ProgressListenerAdapter progressListener;
     private final FailsafeBuildProgressListenerAdapter buildProgressListener;
     private final CancellationToken cancellationToken;
@@ -178,10 +199,12 @@ public class ConsumerOperationParameters implements BuildOperationParametersVers
     private final List<String> arguments;
     private final List<String> tasks;
     private final List<InternalLaunchable> launchables;
+    private final ClassPath injectedPluginClasspath;
 
-    private ConsumerOperationParameters(ConnectionParameters parameters, OutputStream stdout, OutputStream stderr, Boolean colorOutput, InputStream stdin,
-                                        File javaHome, List<String> jvmArguments, List<String> arguments, List<String> tasks, List<InternalLaunchable> launchables,
+    private ConsumerOperationParameters(String entryPointName, ConnectionParameters parameters, OutputStream stdout, OutputStream stderr, Boolean colorOutput, InputStream stdin,
+                                        File javaHome, List<String> jvmArguments, List<String> arguments, List<String> tasks, List<InternalLaunchable> launchables, ClassPath injectedPluginClasspath,
                                         ProgressListenerAdapter progressListener, FailsafeBuildProgressListenerAdapter buildProgressListener, CancellationToken cancellationToken) {
+        this.entryPointName = entryPointName;
         this.parameters = parameters;
         this.stdout = stdout;
         this.stderr = stderr;
@@ -192,15 +215,12 @@ public class ConsumerOperationParameters implements BuildOperationParametersVers
         this.arguments = arguments;
         this.tasks = tasks;
         this.launchables = launchables;
+        this.injectedPluginClasspath = injectedPluginClasspath;
         this.progressListener = progressListener;
         this.buildProgressListener = buildProgressListener;
         this.cancellationToken = cancellationToken;
     }
 
-    private static List<String> rationalizeInput(String[] arguments) {
-        return arguments != null && arguments.length > 0 ? Arrays.asList(arguments) : null;
-    }
-
     private static void validateJavaHome(File javaHome) {
         if (javaHome == null) {
             return;
@@ -210,6 +230,13 @@ public class ConsumerOperationParameters implements BuildOperationParametersVers
         }
     }
 
+    public String getEntryPointName() {
+        return entryPointName;
+    }
+
+    /**
+     * @since 1.0-milestone-3
+     */
     public long getStartTime() {
         return startTime;
     }
@@ -218,42 +245,72 @@ public class ConsumerOperationParameters implements BuildOperationParametersVers
         return parameters.getVerboseLogging();
     }
 
+    /**
+     * @since 1.0-milestone-3
+     */
     public File getGradleUserHomeDir() {
         return parameters.getGradleUserHomeDir();
     }
 
+    /**
+     * @since 1.0-milestone-3
+     */
     public File getProjectDir() {
         return parameters.getProjectDir();
     }
 
+    /**
+     * @since 1.0-milestone-3
+     */
     public Boolean isSearchUpwards() {
         return parameters.isSearchUpwards();
     }
 
+    /**
+     * @since 1.0-milestone-3
+     */
     public Boolean isEmbedded() {
         return parameters.isEmbedded();
     }
 
+    /**
+     * @since 1.0-milestone-3
+     */
     public TimeUnit getDaemonMaxIdleTimeUnits() {
         return parameters.getDaemonMaxIdleTimeUnits();
     }
 
+    /**
+     * @since 1.0-milestone-3
+     */
     public Integer getDaemonMaxIdleTimeValue() {
         return parameters.getDaemonMaxIdleTimeValue();
     }
 
+    /**
+     * @since 2.2-rc-1
+     */
     public File getDaemonBaseDir() {
         return parameters.getDaemonBaseDir();
     }
 
+    /**
+     * @since 1.0-milestone-3
+     */
     public OutputStream getStandardOutput() {
         return stdout;
     }
 
+    /**
+     * @since 1.0-milestone-3
+     */
     public OutputStream getStandardError() {
         return stderr;
     }
 
+    /**
+     * @since 2.3-rc-1
+     */
     public Boolean isColorOutput() {
         return colorOutput;
     }
@@ -278,14 +335,30 @@ public class ConsumerOperationParameters implements BuildOperationParametersVers
         return tasks;
     }
 
+    /**
+     * @since 1.12-rc-1
+     */
     public List<InternalLaunchable> getLaunchables() {
         return launchables;
     }
 
+    /**
+     * @since 2.8-rc-1
+     */
+    public List<File> getInjectedPluginClasspath() {
+        return injectedPluginClasspath.getAsFiles();
+    }
+
+    /**
+     * @since 1.0-milestone-3
+     */
     public ProgressListenerVersion1 getProgressListener() {
         return progressListener;
     }
 
+    /**
+     * @since 2.4-rc-1
+     */
     public FailsafeBuildProgressListenerAdapter getBuildProgressListener() {
         return buildProgressListener;
     }
