@@ -35,8 +35,6 @@ import java.util.*;
 
 /**
  * This tab contains general settings for the plugin.
- *
- * @author mhunsicker
  */
 public class SetupTab implements GradleTab, GradlePluginLord.SettingsObserver {
     private final Logger logger = Logging.getLogger(SetupTab.class);
@@ -193,7 +191,7 @@ public class SetupTab implements GradleTab, GradlePluginLord.SettingsObserver {
     private File browseForDirectory(File initialFile) {
 
         if (initialFile == null) {
-            initialFile = SystemProperties.getCurrentDir();
+            initialFile = SystemProperties.getInstance().getCurrentDir();
         }
 
         JFileChooser chooser = new JFileChooser(initialFile);
@@ -492,7 +490,7 @@ public class SetupTab implements GradleTab, GradlePluginLord.SettingsObserver {
      * Call this to browse for a custom gradle executor.
      */
     private void browseForCustomGradleExecutor() {
-        File startingDirectory = new File(SystemProperties.getUserHome());
+        File startingDirectory = new File(SystemProperties.getInstance().getUserHome());
         File currentFile = gradlePluginLord.getCustomGradleExecutor();
         if (currentFile != null) {
             startingDirectory = currentFile.getAbsoluteFile();
